MODULE_NAME='AMX_AXB_MIDI' (DEV vdvMIDI,DEV dvMIDI)
(***********************************************************)
(*  FILE CREATED ON: 01/21/2002 AT: 17:14:22               *)
(***********************************************************)
(*  FILE_LAST_MODIFIED_ON: 06/09/2010  AT: 18:47:59        *)
(***********************************************************)
(* The following code block is provided as a guide to      *)
(* programming the device(s) listed above. This is a       *)
(* sample only, and will most likely require modification  *)
(* to be integrated into a master program.                 *)
(*                                                         *)
(* Device-specific protocols should be obtained directly   *)
(* from the equipment manufacturer to ensure compliance    *)
(* with the most current versions. Within the limits       *)
(* imposed by any non-disclosure agreements, AMX will      *)
(* provide protocols upon request.                         *)
(*                                                         *)
(* If further programming assistance is required, please   *)
(* contact your AMX customer support team.                 *)
(***********************************************************)
(*  FILE REVISION: Rev 1.22                                *)
(*  REVISION DATE: 06/09/2010  AT: 18:47:59                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Compiled with:                                       *)
(*           DEV_TO_STRING.axi v2.55.                      *)
(*           debug.axi v2.60.                              *)
(*           tokenizer.axi v2.08.                          *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: Rev 1.21                                *)
(*  REVISION DATE: 11/10/2008  AT: 21:03:41                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Speed Tweaks:                                        *)
(*    * Avoid use of 0 in control expressions. For example *)
(*      use "IF (var)" instead of "IF (var > 0)", and      *)
(*      "IF (!var)" instead of "IF (var == 0)".            *)
(*  - Compiled with:                                       *)
(*           DEV_TO_STRING.axi v2.55.                      *)
(*           debug.axi v2.57.                              *)
(*           tokenizer.axi v2.08.                          *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: Rev 1.20                                *)
(*  REVISION DATE: 11/03/2008  AT: 19:11:20                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Speed tweaks                                         *)
(*    * Use of simple expressions in SWITCH(<exp>) is      *)
(*      slightly faster than                               *)
(*      <var> = <exp>                                      *)
(*      SWITCH (<var>)                                     *)
(*    * Use variables for status flags, not virtual        *)
(*      device channels.                                   *)
(*    * Use LOCAL_VAR <var> instead of STACK_VAR <var>     *)
(*      inside of FOR and WHILE loops.                     *)
(*    * Reduce FOR and WHILE loop control expressions to   *)
(*      only the variable name where possible.             *)
(*    * Avoid use of LENGTH_ARRAY or LENGTH_STRING() in    *)
(*      FOR or WHILE loop control expressions.             *)
(*    For example:                                         *)
(*    Bad  -   FOR (i = 1; i <= LENGTH_STRING(str); i++)   *)
(*    Good -   len = LENGTH_STRING(str)                    *)
(*             FOR (i = 1; i <= len; i++)                  *)
(*    Better - FOR (i = LENGTH_STRING(str); i > 0; i--)    *)
(*    Best -   FOR (i = LENGTH_STRING(str); i; i--)        *)
(*  - Compiled with:                                       *)
(*           DEV_TO_STRING.axi v2.54.                      *)
(*           debug.axi v2.56.                              *)
(*           tokenizer.axi v2.08.                          *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: Rev 1.19                                *)
(*  REVISION DATE: 10/06/2008  AT: 08:36:19                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Does not display included lib version info on        *)
(*    startup.                                             *)
(*  - Compiled with:                                       *)
(*           DEV_TO_STRING.axi v2.54.                      *)
(*           debug.axi v2.54.                              *)
(*           tokenizer.axi v2.07.                          *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: Rev 1.18, 1.17                          *)
(*  REVISION DATE: 09/22/2008  AT: 10:02:35                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Reworked vdvMIDI COMMAND: handler. It now gives the  *)
(*    highest precedence to outgoing MIDI messages that    *)
(*    start with a status byte. Also streamlined handling  *)
(*    of other commands.                                   *)
(*  - Tweaked help and comments.                           *)
(*  - DO_PUSH(vdvMIDI,1) when receiving data that affects  *)
(*    the running status buffer.                           *)
(*  - Added TERSE status queries.                          *)
(*  - Added ability to change dvDebug during runtime.      *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: Rev 1.16                                *)
(*  REVISION DATE: 09/15/2008  AT: 08:42:52                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Removed 2-byte orphan data wait code. It could       *)
(*    potentially pass incomplete                          *)
(*    "<status>,<1st data byte>" messages to main code.    *)
(*    Completed messages with 2 data bytes get sent        *)
(*    anyways, even without this extra code.               *)
(*  - Compiled with:                                       *)
(*      debug.axi v2.53                                    *)
(*      DEV_TO_STRING v2.53                                *)
(*      tokenizer.axi v2.06                                *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: Rev 1.15                                *)
(*  REVISION DATE: 08/26/2008  AT: 08:40:47                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - No CREATE_BUFFER for dvMIDI.                         *)
(*  - Increased "orphan data" wait for 2-byte messages to  *)
(*    200ms.                                               *)
(*  - Compiled with:                                       *)
(*      debug.axi v2.52                                    *)
(*      tokenizer.axi v2.06                                *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: Rev 1.14                                *)
(*  REVISION DATE: 08/17/2008  AT: 16:34:26                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Midi Messages with 2-byte data do not send the data  *)
(*    until both bytes are received.                       *)
(*  - Debug messages for MIDI channel voice messages $DF   *)
(*    and lower are now printed with 'SEND MIDI TO DEBUG'. *)
(*  - Compiled with:                                       *)
(*      debug.axi v2.52                                    *)
(*      strings.axi v2.04                                  *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 1.13                                    *)
(*  REVISION DATE: 08/16/2008  AT: 12:01:28                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Tweaked help message.                                *)
(*  - Removed unused structs.                              *)
(*  - Compiled with:                                       *)
(*      debug.axi v2.51                                    *)
(*      strings.axi v2.04                                  *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: Rev 1.12                                *)
(*  REVISION DATE: 02/11/2008  AT: 06:46:04                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Increased length of string returned by textStatus()  *)
(*    from 67 to 131.                                      *)
(*  - Removed COMPARE_STRING()'s, replaced with ==         *)
(*  - Added commands for turning on/off "pass to main"     *)
(*    channels.                                            *)
(*  - Compiled with:                                       *)
(*      debug.axi v2.13                                    *)
(*      strings.axi v2.04                                  *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: Rev 1.11                                *)
(*  REVISION DATE: 01/11/2008  AT: 09:05:11                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Now can properly split all data-containing message   *)
(*    types into multiple packets, without echo or status  *)
(*    duplication.                                         *)
(*  - Module Info now ends with Header and dvMidi D:P:S,   *)
(*    similar to the Debug module info.                    *)
(*  - Tweaked some debug messages.                         *)
(***********************************************************)
(*  FILE REVISION: Rev 1.10                                *)
(*  REVISION DATE: 01/06/2008  AT: 18:48:00                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - added debug messages for received strings passed     *)
(*    from module to main program.                         *)
(*  - clear DATA after processing to eliminate echo.       *)
(*  - code cleanup.                                        *)
(***********************************************************)
(*  FILE REVISION: Rev 1.09                                *)
(*  REVISION DATE: 09/05/2007                              *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Added ability to change message header.              *)
(*  - DATA buffer in string handler for dvMIDI changed to  *)
(*    LOCAL_VAR. STACK_VAR will lose bytes if all data is  *)
(*    not received in one event.                           *)
(*    - cleanup, debug tweaks.                             *)
(***********************************************************)
(*  FILE REVISION: Rev 1.08                                *)
(*  REVISION DATE: 08/23/2006                              *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*    - simplify buffer parsing.                           *)
(***********************************************************)
(*!!FILE REVISION: Rev 1.07                                *)
(*  REVISION DATE: 08/21/2006                              *)
(*    - tweaked comments, no functional changes.           *)
(*  REVISION DATE: 08/11/2006                              *)
(*    - tweaked StatusText function                        *)
(*    - added '?' prefix to Status and VER commands        *)
(*    - tweaked command and string parsing                 *)
(***********************************************************)
(*!!FILE REVISION: Rev 1.06                                *)
(*  REVISION DATE: 11/26/2004                              *)
(*    - added Status command for terminal                  *)
(***********************************************************)
(*!!FILE REVISION: Rev 1.05                                *)
(*  REVISION DATE: 09/20/2003                              *)
(*    - correct MASK_MASTER value is $4                    *)
(*    - added more comments                                *)
(*    - tweaked 'MODULE HELP'                              *)
(*    - removed unneeded variable sTEMP                    *)
(*    - removed redundant $F7 parsing                      *)
(***********************************************************)
(*!!FILE REVISION: Rev 1.04                                *)
(*  REVISION DATE: 05/27/2003                              *)
(*    - pass commands to vdvMIDI as strings to dvMIDI      *)
(*    when they are not one of the help or setup commands. *)
(*    - add line ends ;                                    *)
(***********************************************************)
(*!!FILE REVISION: Rev 1.03                                *)
(*  REVISION DATE: 08/10/2002                              *)
(*    - send $F7 to master with rest of sysx data, instead *)
(*    of in a seperate string, when data_event[dvMIDI]     *)
(*    while loop starts with data byte.                    *)
(***********************************************************)
(*!!FILE REVISION: Rev 1.02                                *)
(*  REVISION DATE: 07/28/2002                              *)
(*    - updated help                                       *)
(*    - send $F7 to master with rest of sysx data, instead *)
(*    of in a seperate string.                             *)
(***********************************************************)
(*!!FILE REVISION: Rev 0                                   *)
(*  REVISION DATE: 05/15/2002                              *)
(*  - this module provides a starting point for parsing    *)
(*    MIDI data.                                           *)
(*  - the primary feature of this module is to filter the  *)
(*    amount of data being passed to your main program.    *)
(*  COMMENTS:                                              *)
          // Netlinx Programming Notes:
          // MIDI Programming Notes:
          //  Status BYTES:
          //  FOR CHANNEL VOICE MESSAGES;
          //  UPPER NYBBLE IS MESSAGE TYPE
          //  LOWER NYBBLE IS MIDI CHANNEL
          //  FOR SYSTEM MESSAGES;
          //  UPPER NYBBLE IS ALWAYS $F
          //  LOWER NYBBLE IS MESSAGE TYPE
(***********************************************************)
/*
 * tab-width: 4
 * indent-level: 2
 *
 */
DEFINE_VARIABLE
__display_lib_version__ = FALSE     // suppress #INCLUDE version info

#INCLUDE 'DEV_TO_STRING.axi'        // http://www.amx.com/techsupport/ TN461
#INCLUDE 'debug.axi'                // http://www.amx.com/techsupport/ TN461
#INCLUDE 'tokenizer.axi'            // StringTokenizer()
(***********************************************************)
(*          DEVICE NUMBER DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_DEVICE

(***********************************************************)
(*               CONSTANT DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_CONSTANT

(* MODULE INFO *********************************************)
                                    // copy of MODULE_NAME
__MODULE_NAME__     = 'AMX_AXB_MIDI';
__MODULE_VERSION__  = '1.22';       // module version
__MODULE_AUTHOR__   = 'GLM';        // author's initials


(* vdvMIDI CHANNELS ****************************************)
// ON = DATA IS PASSED TO MASTER VIA SEND_STRING TO VIRTUAL DEVICE
                                    // SEND TO MASTER CHANNELS
                                    // NOTE channels
PASS_NOTE[] = { 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,15,16};
                                    // AFTerTouch channels
PASS_AFTT[] = {17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32};
                                    // ConTRoLler channels
PASS_CTRL[] = {33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48};
                                    // PROGram channels
PASS_PROG[] = {49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64};
                                    // pitch BEND channels
PASS_BEND[] = {65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80};

PASS_SYSX       = 81;               // SYStem eXclusive
PASS_CMMN       = 82;               // system CoMMoN (EXCEPT SYSX)
PASS_REAL       = 83;               // system REALtime (EXCEPT ACTIVE SENSE)
PASS_ACSN       = 84;               // system realtime ACtive SeNse

(***********************************************************)
(*               VARIABLE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_VARIABLE

VOLATILE DEV dvDebug = 0:1:0;

VOLATILE CHAR sHeader[16] = 'AMX_AXB_MIDI ';

                                    // MASK VALUES:
CONSTANT INTEGER MASK_THRU    = $01;// this value means "send data to midi thru"
CONSTANT INTEGER MASK_OUT     = $02;// this value means "send data to midi out"
CONSTANT INTEGER MASK_MASTER  = $04;// this value means "send data to amx master"
CHAR    cMASK_A;
CHAR    cMASK_B;
CHAR    cMASK_M;
CHAR    sMASK_M[16];

CHAR    STATUS;                     // Running Status Buffer

INTEGER bStatus_Sent;               // Status-already-sent (to vdvMIDI) flag

CHAR sDV[17];                       // DEV_TO_STRING(dvMIDI)
CHAR sVDV[17];                      // DEV_TO_STRING(vdvMIDI)

VOLATILE INTEGER statusPass[84]

VOLATILE DEBUG_PASSED_RX;
VOLATILE DEBUG_RX;
VOLATILE DEBUG_TX;
VOLATILE DEBUG_CH;

INTEGER lengthData;

(***********************************************************)
(*           SUBROUTINE DEFINITIONS GO BELOW               *)
(***********************************************************)
(* EXAMPLE: DEFINE_FUNCTION <RETURN_TYPE> <NAME> (<PARAMETERS>) *)
(* EXAMPLE: DEFINE_CALL '<NAME>' (<PARAMETERS>) *)
DEFINE_FUNCTION CHAR[131] __MODULE_INFO__ ()
{
  RETURN "DEV_TO_STRING(vdvMIDI),',',   // a. Module control device D:P:S
		  __MODULE_NAME__,',',          // b. Module name
		  'v',__MODULE_VERSION__,',',   // c. Module version
		  __MODULE_AUTHOR__,',',        // d. Author's initials
		  __DATE__,',',                 // e. Compile data
		  __TIME__,',',                 // f. Compile time
		  '''',sHeader,''',',           // g. Header in single quotes
		  DEV_TO_STRING(dvMIDI)";       // h. dvMidi D:P:S
}

DEFINE_CALL 'MODULE HELP'
{
  SEND_STRING dvDEBUG,"'Help ',DEV_TO_STRING(DATA.DEVICE),' -- Commands Available:'";
  SEND_STRING dvDEBUG,"'  HELP or ?'";
  SEND_STRING dvDebug,"'  dvDebug-<debug D[:P:S]>'";
  SEND_STRING dvDEBUG,"'  ?STATUS'";
  SEND_STRING dvDEBUG,"'  ?VERSION'";
  SEND_STRING dvDEBUG,"'  HEADER-<header for debug msg>'";
  SEND_STRING dvDEBUG,"DEV_TO_STRING(dvMIDI),' input masks:'";
  SEND_STRING dvDEBUG,"'  INA-<mask>'";
  SEND_STRING dvDEBUG,"'  INB-<mask>'";
  SEND_STRING dvDEBUG,"'  INM-<mask>'";
  SEND_STRING dvDEBUG,"'Pass ',DEV_TO_STRING(dvMIDI),' data to main program:'";
  SEND_STRING dvDEBUG,"'  NOTE-<comma-delimited list of MIDI channels to pass>'";
  SEND_STRING dvDEBUG,"'  AFTT-<comma-delimited list of MIDI channels to pass>'";
  SEND_STRING dvDEBUG,"'  CTRL-<comma-delimited list of MIDI channels to pass>'";
  SEND_STRING dvDEBUG,"'  PROG-<comma-delimited list of MIDI channels to pass>'";
  SEND_STRING dvDEBUG,"'  BEND-<comma-delimited list of MIDI channels to pass>'";
  SEND_STRING dvDEBUG,"'  SYSX-1/0'";
  SEND_STRING dvDEBUG,"'  CMMN-1/0'";
  SEND_STRING dvDEBUG,"'  REAL-1/0'";
  SEND_STRING dvDEBUG,"'  ACSN-1/0'";
  SEND_STRING dvDebug,"'Examples'";
  SEND_STRING dvDebug,"'  Pass program changes from MIDI channels 1 and 10:'";
  SEND_STRING dvDebug,"'    SEND_COMMAND vdvMIDI,''PROG: 1,10'''";
  SEND_STRING dvDebug,"'  Pass system exclusive messages:'";
  SEND_STRING dvDebug,"'    SEND_COMMAND vdvMIDI,''SYSX: 81'''";
  SEND_STRING dvDebug,"'  DO NOT pass system exclusive messages:'";
  SEND_STRING dvDebug,"'    SEND_COMMAND vdvMIDI,''SYSX: 0'' or'";
  SEND_STRING dvDebug,"'    SEND_COMMAND vdvMIDI,''SYSX: '''";
  SEND_STRING dvDEBUG,"$09"
  SEND_STRING dvDEBUG,"'QUERY command status by dropping the ''-<data>'' and prefixing ''?'''";
  SEND_STRING dvDebug,"'Example:'";
  SEND_STRING dvDebug,"'  SEND_COMMAND vdvMIDI,''?HEADER'''";
  SEND_STRING dvDEBUG,"'All other commands pass as STRINGS to ',DEV_TO_STRING(dvMIDI)";
  SEND_STRING dvDEBUG,"$09"
  SEND_STRING dvDEBUG,"'Help ',DEV_TO_STRING(DATA.DEVICE),' -- Channels Available:'";
  SEND_STRING dvDEBUG,"'Push/Release:'";
  SEND_STRING dvDEBUG,"'  Data received that affects Running Status:  1'";
// Use INTEGER array, not DEVCHAN
//  SEND_STRING dvDEBUG,"'On/Off - Pass ',DEV_TO_STRING(dvMIDI),' data to main program:'";
//  SEND_STRING dvDEBUG,"'  Pass NOTE:  1 + (MIDI_CH - 1)'";
//  SEND_STRING dvDEBUG,"'  Pass AFTT: 17 + (MIDI_CH - 1)'";
//  SEND_STRING dvDEBUG,"'  Pass CTRL: 33 + (MIDI_CH - 1)'";
//  SEND_STRING dvDEBUG,"'  Pass PROG: 49 + (MIDI_CH - 1)'";
//  SEND_STRING dvDEBUG,"'  Pass BEND: 65 + (MIDI_CH - 1)'";
//  SEND_STRING dvDEBUG,"'  Pass SYSX: 81'";
//  SEND_STRING dvDEBUG,"'  Pass CMMN: 82'";
//  SEND_STRING dvDEBUG,"'  Pass REAL: 83'";
//  SEND_STRING dvDEBUG,"'  Pass ACSN: 84'";
  SEND_STRING dvDEBUG,"'Debug:'";
  SEND_STRING dvDEBUG,"'  Strings passed from ',sDV,' to ',sVDV,': 508'";
  SEND_STRING dvDEBUG,"'  MIDI data received on ',sDV,': 509'";
  SEND_STRING dvDEBUG,"'  Strings/Commands from ',sVDV,' to ',sDV,': 510'";
  SEND_STRING dvDEBUG,"'  Push/Release from ',sVDV,': 511'";
}

// text for Status command
DEFINE_FUNCTION CHAR[67] textMask (char cMask,char WhichMask[])
{
  STACK_VAR CHAR sTemp[19];

  SWITCH (cMask)
  {
	CASE '7': sTemp = 'THRU, OUT, MASTER';
	CASE '6': sTemp = 'OUT, MASTER';
	CASE '5': sTemp = 'THRU, MASTER';
	CASE '4': sTemp = 'MASTER';
	CASE '3': sTemp = 'THRU, OUT';
	CASE '2': sTemp = 'OUT';
	CASE '1': sTemp = 'THRU';
	CASE '0': sTemp = 'NULL';
	DEFAULT: sTemp = '!!!INVALID MASK!!!';
  }

  RETURN "WhichMask,' data passed to ',sTemp";
}

DEFINE_FUNCTION CHAR[64] textMidiChannels (CHAR sData[])
// MIDI Channel array to string conversion
// Populated MIDI channel indexes are included in returned string
{
  STACK_VAR INTEGER i;
  STACK_VAR CHAR sChannels[64];
  
  FOR (i = LENGTH_ARRAY(sData); i; i--)
  {                                 // if a device channel that corresponds
	IF (statusPass[sData[i]])  		// to midi channel data is passed in,
	{                        		// add midi channel to sChannels
	  IF (LENGTH_STRING(sChannels))
		sChannels = "ITOA(i),',',sChannels";
	  ELSE
		sChannels = "ITOA(i)";
	}
  }
  RETURN sChannels;
}

DEFINE_FUNCTION CHAR[131] textStatus (char sData[],char sText[])
// Verbose text for Status command
{
  STACK_VAR CHAR sChannels[64];

  sChannels = textMidiChannels(sData);

  SWITCH (LENGTH_STRING(sChannels))
  {
									// data not passed
	CASE 0: 	RETURN "'No ',LOWER_STRING(sText),' will be passed to main program'";
	CASE 1:
	{
	  IF (LENGTH_ARRAY(sData) = 16) // 1st channel of voice data
		RETURN "sText,' from MIDI CH ',sChannels,' passed to main program'";
	  ELSE							// system data
		RETURN "sText,' passed to main program'";
	}
									// other channels of voice data
	DEFAULT:	RETURN "sText,' from MIDI CH''s ',sChannels,' passed to main program'";
  }
}

DEFINE_CALL 'MODULE STATUS'
{
  SEND_STRING dvDEBUG,"'MODULE NAME : ',__MODULE_NAME__";
  SEND_STRING dvDEBUG,"'HEADER : ''',sHeader,''''";

  SEND_STRING dvDEBUG,"textMask(cMASK_A,'INA')";
  SEND_STRING dvDEBUG,"textMask(cMASK_B,'INB')";
  SEND_STRING dvDEBUG,"textMask(cMASK_M,'INM')";

  SEND_STRING dvDEBUG,"textStatus (PASS_NOTE,'NOTES')";
  SEND_STRING dvDEBUG,"textStatus (PASS_AFTT,'AFTERTOUCH')";
  SEND_STRING dvDEBUG,"textStatus (PASS_CTRL,'CONTROLLERS')";
  SEND_STRING dvDEBUG,"textStatus (PASS_PROG,'PROGRAM CHANGES')";
  SEND_STRING dvDEBUG,"textStatus (PASS_BEND,'PITCH BENDS')";
  SEND_STRING dvDEBUG,"textStatus ("PASS_SYSX",'SYSTEM EXCLUSIVE')";
  SEND_STRING dvDEBUG,"textStatus ("PASS_CMMN",'SYSTEM COMMON (except SYSX)')";
  SEND_STRING dvDEBUG,"textStatus ("PASS_REAL",'SYSTEM REALTIME (except ASCN)')";
  SEND_STRING dvDEBUG,"textStatus ("PASS_ACSN",'ACTIVE SENSE')";

  IF (DEBUG_PASSED_RX) 	SEND_STRING dvDEBUG,"'Debug RX passed to Main (508) is ON'";
  ELSE					SEND_STRING dvDEBUG,"'Debug RX passed to Main (508) is off'";
  IF (DEBUG_RX) SEND_STRING dvDEBUG,"'Debug RX (509) is ON'";
  ELSE			SEND_STRING dvDEBUG,"'Debug RX (509) is off'";
  IF (DEBUG_TX)	SEND_STRING dvDEBUG,"'Debug TX (510) is ON'";
  ELSE			SEND_STRING dvDEBUG,"'Debug TX (511) is off'";
  IF (DEBUG_CH)	SEND_STRING dvDEBUG,"'Debug CH (512) is ON'";
  ELSE			SEND_STRING dvDEBUG,"'Debug CH (512) is off'";

}

(*------------------------------------------------------------------------------*------------------------*)
DEFINE_FUNCTION char midiSEPERATE(char INCOMING[], char DATA[])
// FUNCTION: SEPERATE_MIDI
// REMOVES DATA FROM INCOMING, that is, INCOMING is parsed destructively
// SEPERATES DATA BYTES (IF ANY) FROM STATUS BYTE
// BUFFERS STATUS BYTE (DATA BYTES DON'T CHANGE RUNNING STATUS)

/* !!!IMPORTANT - because the Running Status is buffered by a
 * LOCAL_VAR, you MUST instance (i.e. make a seperate copy of)
 * this function for each MIDI data stream you will be parsing!!!
 */

// Note/Warning 2:
//    The DATA buffer passed in should not be a STACK_VAR, as some bytes can be lost.
//    Use a LOCAL_VAR for DATA if the buffer is not global.

// Note/Warning 1:
//    The End-of-Sysx byte ($F7) is not treated as a Status
//    byte, but as a Data byte.
//    This is to ensure SYSX messages being parsed will
//    contain the start byte AND end byte.
//    Before this tweak, SYSX end bytes ($F7) were always being
//    parsed seperately from the rest of the SYSX message.
{
  LOCAL_VAR INTEGER INDEX;
  LOCAL_VAR char    MIDI_BYTE;      // Running Status buffer
  LOCAL_VAR INTEGER LENGTH_INCOMING;

  if(INCOMING[1] == $F7)            // IF IT'S A LONE "END OF SYSX" BYTE
  {
	DATA = "DATA,GET_BUFFER_CHAR(INCOMING)";
  }
  else if(INCOMING[1] >= $80)       // new status byte
  {
	bStatus_Sent = FALSE;           // handshake necessary for running status and sysx messages

    MIDI_BYTE = GET_BUFFER_char(INCOMING);
    LENGTH_INCOMING = LENGTH_ARRAY(INCOMING);
    IF(!LENGTH_INCOMING)
    {
      //This is a one byte command
      DATA = '';
    }
    else if(INCOMING[1] >= $80)
    {
      //This is a one byte command
      DATA = '';
    }
    else
    {
      // Get bytes from this command
      for(INDEX=1; INDEX <= LENGTH_INCOMING; INDEX++)
      {
        IF(INCOMING[INDEX] == $F7)  // IF IT'S "END OF SYSX" BYTE (i WILL BE > 1)
        {
								    // GET THE DATA AND STATUS BYTE $F7 AND BREAK
		  DATA = GET_BUFFER_STRING(INCOMING,INDEX);
		  BREAK;
	    }
									// IF YOU FIND A NEW STATUS BYTE (i WILL BE > 1)
        ELSE IF(INCOMING[INDEX] >= $80)
        {
          DATA = GET_BUFFER_STRING(INCOMING,INDEX-1);
          BREAK;
        }
		                            // ELSE IF THE END OF THE STRING IS REACHED
        ELSE IF(INDEX == LENGTH_INCOMING)
        {
          DATA = GET_BUFFER_STRING(INCOMING,INDEX);
        }
      }
    }
  }
  else
  {
	//some number of bytes from old command
    LENGTH_INCOMING = LENGTH_ARRAY(INCOMING);
    for(INDEX=1; INDEX <= LENGTH_INCOMING; INDEX++)
    {
	  IF(INCOMING[INDEX] == $F7)    // IF IT'S "END OF SYSX" BYTE (i WILL BE > 1)
      {
                                    // GET THE DATA AND STATUS BYTE $F7 AND BREAK
        DATA = "DATA,GET_BUFFER_STRING(INCOMING,INDEX)";
        BREAK;
	  }
									// IF YOU FIND A NEW STATUS BYTE (i WILL BE > 1)
      else if(INCOMING[INDEX] >= $80)
      {
									// GET THE DATA AND BREAK
        DATA = "DATA,GET_BUFFER_STRING(INCOMING,INDEX-1)";
        BREAK;
      }
								    // ELSE IF THE END OF THE STRING IS REACHED
      else if(INDEX == LENGTH_INCOMING)
      {
        DATA = "DATA,GET_BUFFER_STRING(INCOMING,INDEX)";
      }
    }
  }
  return MIDI_BYTE;
}

DEFINE_FUNCTION midiTransmit()
// this function created to avoid code duplication under the vdvMIDI COMMAND: handler
{
  SEND_STRING dvMIDI,DATA.TEXT;

  IF (DEBUG_TX)
  {
	CALL 'SEND MIDI TO DEBUG' (dvDEBUG,"sHeader,sDV,' ',sMASK_M,': '",DATA.TEXT,0,0,DATA.TEXT[1]);
  }
}

(***********************************************************)
(*                STARTUP CODE GOES BELOW                  *)
(***********************************************************)
DEFINE_START

sDV = DEV_TO_STRING(dvMIDI);
sVDV = DEV_TO_STRING(vdvMIDI);

(***********************************************************)
(*                THE EVENTS GOES BELOW                    *)
(***********************************************************)
DEFINE_EVENT

DATA_EVENT[dvMIDI]
{
  ONLINE:
  {
    LOCAL_VAR INTEGER X;

    WAIT 20                         // WAIT FOR SEND_COMMANDS FROM MAIN PROGRAM TO vdvMIDI
    {
      IF (!cMASK_A)                 // AXB-MIDI DEFAULT : INA to THRU and OUT
      {
        cMASK_A = (MASK_THRU | MASK_OUT) + $30;
      }
      IF (!cMASK_B)                 // AXB-MIDI DEFAULT : INB to THRU and OUT
      {
        cMASK_B = (MASK_THRU | MASK_OUT) + $30;
      }
      IF (!cMASK_M)                 // AXB-MIDI DEFAULT : INM to THRU and OUT
      {
        cMASK_M = (MASK_THRU | MASK_OUT) + $30;
		sMASK_M = 'THRU & OUT';
      }
      SEND_COMMAND dvMIDI,"'INA-',cMASK_A";
      SEND_COMMAND dvMIDI,"'INB-',cMASK_B";
      SEND_COMMAND dvMIDI,"'INM-',cMASK_M";
      IF (DEBUG_TX)
      {
        SEND_STRING dvDEBUG,"sHeader,sDV,' <- INA-',cMASK_A";
        SEND_STRING dvDEBUG,"sHeader,sDV,' <- INB-',cMASK_B";
        SEND_STRING dvDEBUG,"sHeader,sDV,' <- INM-',cMASK_M";
      }
    }
  }
  STRING:
  {
    LOCAL_VAR INTEGER INDEX;
    LOCAL_VAR CHAR    STATUS_REAL;
    LOCAL_VAR INTEGER LENGTH_INCOMING;
    LOCAL_VAR CHAR    sData[1024];
    LOCAL_VAR CHAR    sPacket[1024];

    IF (DEBUG_RX)
    {
      CALL 'SEND ITOHEX TO DEBUG' (dvDEBUG,"sHeader,sDV,' str: '",DATA.TEXT,0,0);
    }

	WHILE (LENGTH_ARRAY(DATA.TEXT))
    {
									// SYSTEM REAL TIME messages,
									// must not affect Running Status per MIDI spec.
	  if (DATA.TEXT[1] == $FE)      // SYSTEM REALTIME ACTIVE SENSE BYTE
      {
        STATUS_REAL = GET_BUFFER_CHAR(DATA.TEXT);
        IF (statusPass[PASS_ACSN])
        {
          SEND_STRING vdvMIDI,"STATUS_REAL";

		  IF (DEBUG_PASSED_RX)
			CALL 'SEND ITOHEX TO DEBUG' (dvDebug,"sHeader,'AS: '","STATUS_REAL",0,0)
        }
      }
      else if (DATA.TEXT[1] >= $F8)   // ANY OTHER SYSTEM REALTIME BYTE
      {
        STATUS_REAL = GET_BUFFER_CHAR(DATA.TEXT);
        IF (statusPass[PASS_REAL])
        {
          SEND_STRING vdvMIDI,"STATUS_REAL";

		  IF (DEBUG_PASSED_RX)
		  {
			SWITCH (STATUS_REAL)
			{
			  CASE $F8: CALL 'SEND ITOHEX TO DEBUG' (dvDebug,"sHeader,'TC: '","STATUS_REAL",0,0)
			  CASE $FA: CALL 'SEND ITOHEX TO DEBUG' (dvDebug,"sHeader,'START: '","STATUS_REAL",0,0)
			  CASE $FB: CALL 'SEND ITOHEX TO DEBUG' (dvDebug,"sHeader,'CONTINUE: '","STATUS_REAL",0,0)
			  CASE $FC: CALL 'SEND ITOHEX TO DEBUG' (dvDebug,"sHeader,'STOP: '","STATUS_REAL",0,0)
			  CASE $FF: CALL 'SEND ITOHEX TO DEBUG' (dvDebug,"sHeader,'RESET: '","STATUS_REAL",0,0)
			  DEFAULT: CALL 'SEND ITOHEX TO DEBUG' (dvDebug,"sHeader,'UNDEFINED: '","STATUS_REAL",0,0)
			}
		  }
        }
      }
	  // end MIDI Realtime Data
      ELSE                          // Running Status affecting data
      {
        STATUS = midiSEPERATE(DATA.TEXT,sData);
		lengthData = LENGTH_ARRAY(sData);
		DO_PUSH_TIMED(vdvMIDI,1,3);

		SWITCH (STATUS & $F0)       // hi nybble
		{
		  CASE $F0:                 // System Common Data
		  {
			SWITCH (STATUS & $0F)   // lo nybble
			{
			  CASE $0:              // Sysx - many bytes data ($F7 will be in sDATA)
			  {
				IF (statusPass[PASS_SYSX])
				{
				  IF (bStatus_Sent == FALSE)
				  {
					SEND_STRING vdvMIDI,"STATUS,sData";
					bStatus_Sent = TRUE;

					IF (DEBUG_PASSED_RX)
					  CALL 'SEND ITOHEX TO DEBUG' (dvDebug,"sHeader,'SysX: '","STATUS,sData",0,0)
				  }
				  ELSE
				  {
					SEND_STRING vdvMIDI,"sData";

					IF (DEBUG_PASSED_RX)
					  CALL 'SEND ITOHEX TO DEBUG' (dvDebug,"sHeader,'SysX: '","sData",0,0)
				  }
				}
				IF (lengthData)
				{
				  IF (sData[lengthData] = $F7)
					bStatus_Sent = FALSE;
				}
				sData = ""
			  }
			  CASE $2:              // SONG POSITION POINTER - 2 byte data
			  {
				IF (statusPass[PASS_CMMN])
				{
  		    						// keep 2-byte data framed "lsb,msb"
				  IF (lengthData >= 2) 
				  {
					sPacket = GET_BUFFER_STRING(sData,(lengthData & $FFFFFFFE));

					IF (bStatus_Sent == FALSE)
					{
					  SEND_STRING vdvMIDI,"STATUS,sPacket";
					  bStatus_Sent = TRUE;

					  IF (DEBUG_PASSED_RX)
						CALL 'SEND ITOHEX TO DEBUG' (dvDebug,"sHeader,'Song Pos: '","STATUS,sPacket",0,0)
					}
					ELSE
					{
					  SEND_STRING vdvMIDI,"sPacket";
					  bStatus_Sent = TRUE;

					  IF (DEBUG_PASSED_RX)
						CALL 'SEND ITOHEX TO DEBUG' (dvDebug,"sHeader,'Song Pos: '","sPacket",0,0)
					}
				  }
				}
				ELSE
				  sData = "";
			  }
			  CASE $3:              // SONG SELECT - 1 byte data
			  {
				IF (statusPass[PASS_CMMN])
				{
				  IF (bStatus_Sent == FALSE)
				  {
					SEND_STRING vdvMIDI,"STATUS,sData";
					bStatus_Sent = TRUE;

					IF (DEBUG_PASSED_RX)
					  CALL 'SEND ITOHEX TO DEBUG' (dvDebug,"sHeader,'Song Sel: '","STATUS,sData",0,0)
				  }
				  ELSE
				  {
					SEND_STRING vdvMIDI,"sData";
					bStatus_Sent = TRUE;

					IF (DEBUG_PASSED_RX)
					  CALL 'SEND ITOHEX TO DEBUG' (dvDebug,"sHeader,'Song Sel: '","sData",0,0)
				  }
				}
				sData = ""
			  }
			  CASE $6:				// TUNE REQUEST - NO sData
			  {
				IF (statusPass[PASS_CMMN])
				{
				  SEND_STRING vdvMIDI,"STATUS";

				  IF (DEBUG_PASSED_RX)
					CALL 'SEND ITOHEX TO DEBUG' (dvDebug,"sHeader,'TUNE REQUEST: '","STATUS,sData",0,0)
				}
				sData = ""
			  }
			  DEFAULT:              // UNDEFINED - NO sData?
			  {
				IF (statusPass[PASS_CMMN])
				{
				  IF (bStatus_Sent == FALSE)
				  {
					SEND_STRING vdvMIDI,"STATUS,sData";

					IF (DEBUG_PASSED_RX)
					  CALL 'SEND ITOHEX TO DEBUG' (dvDebug,"sHeader,'UNDEFINED: '","STATUS,sData",0,0)
				  }
				  ELSE
				  {
					SEND_STRING vdvMIDI,"sData";

					IF (DEBUG_PASSED_RX)
					  CALL 'SEND ITOHEX TO DEBUG' (dvDebug,"sHeader,'UNDEFINED: '","sData",0,0)
				  }
				}
				sData = ""
			  }
			}
		  }
		  CASE $E0:                 // PITCH WHEEL - 2 byte data
		  {
		    IF (statusPass[PASS_BEND[(STATUS & $0F)+1]])
			{
  		    						// keep 2-byte data framed "lsb,msb"
			  IF (lengthData >= 2)
			  {
				sPacket = GET_BUFFER_STRING(sData,(lengthData & $FFFFFFFE));

				IF (!bStatus_Sent)
				{
				  SEND_STRING vdvMIDI,"STATUS,sPacket";
				  bStatus_Sent = TRUE;

				  IF (DEBUG_PASSED_RX)
					CALL 'SEND MIDI TO DEBUG' (dvDebug,"sHeader,'P BEND: '","STATUS,sPacket",0,0,STATUS)
				}
				ELSE
				{
				  SEND_STRING vdvMIDI,"sPacket";

				  IF (DEBUG_PASSED_RX)
					CALL 'SEND MIDI TO DEBUG' (dvDebug,"sHeader,'P BEND: '","sPacket",0,0,STATUS)
				}
			  }
			}
			ELSE
			  sData = "";
		  }
		  CASE $D0:                 // CHANNEL AFTERTOUCh - 1 byte data
		  {
   		    IF (statusPass[PASS_AFTT[(STATUS & $0F)+1]])
			{
			  IF (!bStatus_Sent)
			  {
				SEND_STRING vdvMIDI,"STATUS,sData";
				bStatus_Sent = TRUE;

				IF (DEBUG_PASSED_RX)
				  CALL 'SEND MIDI TO DEBUG' (dvDebug,"sHeader,'C AFTT: '","STATUS,sData",0,0,STATUS)
			  }
			  ELSE
			  {
				SEND_STRING vdvMIDI,"sData";

				IF (DEBUG_PASSED_RX)
				  CALL 'SEND MIDI TO DEBUG' (dvDebug,"sHeader,'C AFTT: '","sData",0,0,STATUS)
			  }
		 	}
			sData = ""
		  }
		  CASE $C0:                 // PROGRAM CHANGE - 1 byte data
		  {
  		    IF (statusPass[PASS_PROG[(STATUS & $0F)+1]])
			{
			  IF (!bStatus_Sent)
			  {
				SEND_STRING vdvMIDI,"STATUS,sData";
				bStatus_Sent = TRUE;

				IF (DEBUG_PASSED_RX)
				  CALL 'SEND MIDI TO DEBUG' (dvDebug,"sHeader,'PROG: '","STATUS,sData",0,0,STATUS)
			  }
			  ELSE
			  {
				SEND_STRING vdvMIDI,"sData";

				IF (DEBUG_PASSED_RX)
				  CALL 'SEND MIDI TO DEBUG' (dvDebug,"sHeader,'PROG: '","sData",0,0,STATUS)
			  }
			}
			sData = "";
		  }
		  CASE $B0:                 // CONTROL CHANGE - 2 byte data
		  {
  		    IF (statusPass[PASS_CTRL[(STATUS & $0F)+1]])
			{
									// keep 2-byte data framed "ctrl#,data"
			  IF (lengthData >= 2)
			  {
				sPacket = GET_BUFFER_STRING(sData,(lengthData & $FFFFFFFE));

				IF (!bStatus_Sent)
				{
				  SEND_STRING vdvMIDI,"STATUS,sPacket";
				  bStatus_Sent = TRUE;

				  IF (DEBUG_PASSED_RX)
					CALL 'SEND MIDI TO DEBUG' (dvDebug,"sHeader,'CTRL: '","STATUS,sPacket",0,0,STATUS)
				}
				ELSE
				{
				  SEND_STRING vdvMIDI,"sPacket";

				  IF (DEBUG_PASSED_RX)
					CALL 'SEND MIDI TO DEBUG' (dvDebug,"sHeader,'CTRL: '","sPacket",0,0,STATUS)
				}
			  }
			}
			ELSE
			  sData = "";
		  }
		  CASE $A0:                 // POLY AFTERTOUCH - 2 byte data
		  {
		    IF (statusPass[PASS_AFTT[(STATUS & $0F)+1]])
			{
									// keep 2-byte data framed "note#,pressure"
			  IF (lengthData >= 2)
			  {
				sPacket = GET_BUFFER_STRING(sData,(lengthData & $FFFFFFFE));

				IF (!bStatus_Sent)
				{
				  SEND_STRING vdvMIDI,"STATUS,sPacket";
				  bStatus_Sent = TRUE;

				  IF (DEBUG_PASSED_RX)
					CALL 'SEND MIDI TO DEBUG' (dvDebug,"sHeader,'P AFTT: '","STATUS,sPacket",0,0,STATUS)
				}
				ELSE
				{
				  SEND_STRING vdvMIDI,"sPacket";

				  IF (DEBUG_PASSED_RX)
					CALL 'SEND MIDI TO DEBUG' (dvDebug,"sHeader,'P AFTT: '","sPacket",0,0,STATUS)
				}
			  }
			}
			ELSE
			  sData = "";
		  }
		  CASE $90:                 // NOTE ON - 2 byte data
		  {
		    IF (statusPass[PASS_NOTE[(STATUS & $0F)+1]])
			{
									// keep 2-byte data framed "note#,velocity"
			  IF (lengthData >= 2)
			  {
				sPacket = GET_BUFFER_STRING(sData,(lengthData & $FFFFFFFE));

				IF (!bStatus_Sent)
				{
				  SEND_STRING vdvMIDI,"STATUS,sPacket";
				  bStatus_Sent = TRUE;

				  IF (DEBUG_PASSED_RX)
					CALL 'SEND MIDI TO DEBUG' (dvDebug,"sHeader,'NOTE N: '","STATUS,sPacket",0,0,STATUS)
				}
				ELSE
				{
				  SEND_STRING vdvMIDI,"sPacket";

				  IF (DEBUG_PASSED_RX)
					CALL 'SEND MIDI TO DEBUG' (dvDebug,"sHeader,'NOTE N: '","sPacket",0,0,STATUS)
				}
			  }
			}
			ELSE
			  sData = "";
		  }
		  CASE $80:                 // NOTE OFF - 2 byte data
		  {
		    IF (statusPass[PASS_NOTE[(STATUS & $0F)+1]])
			{
									// keep 2-byte data framed "note#,velocity"
			  IF (lengthData >= 2)
			  {
				sPacket = GET_BUFFER_STRING(sData,(lengthData & $FFFFFFFE));

				IF (!bStatus_Sent)
				{
				  SEND_STRING vdvMIDI,"STATUS,sPacket";
				  bStatus_Sent = TRUE;

				  IF (DEBUG_PASSED_RX)
					CALL 'SEND MIDI TO DEBUG' (dvDebug,"sHeader,'NOTE F: '","STATUS,sPacket",0,0,STATUS)
				}
				ELSE
				{
				  SEND_STRING vdvMIDI,"sPacket";

				  IF (DEBUG_PASSED_RX)
					CALL 'SEND MIDI TO DEBUG' (dvDebug,"sHeader,'NOTE F: '","sPacket",0,0,STATUS)
				}
			  }
			}
			ELSE
			  sData = "";
		  }
		}// end Switch (Nybble.Hi)
      }// end ELSE                          // Running Status affecting data
    }// end WHILE (LENGTH_ARRAY(DATA.TEXT))
  }// end STRING:
}// end DATA_EVENT[dvMIDI]

DATA_EVENT[vdvMIDI]
{
  ONLINE:
  {
	SET_VIRTUAL_CHANNEL_COUNT (DATA.DEVICE,511);

                                    // return module info on startup
    SEND_STRING dvDEBUG,__MODULE_INFO__();

    // main program will turn on necessary channels
  }
  COMMAND:
  {
	IF (DATA.TEXT[1] & $80)         // if the first byte is a MIDI status byte...
	{
	  midiTransmit()                // ...send to MIDI device
	}
	ELSE
	{
	  STACK_VAR CMD[32];

	  IF (FIND_STRING(DATA.TEXT, '-', 1)) { CMD = UPPER_STRING(REMOVE_STRING(DATA.TEXT, '-', 1)); }
	  ELSE                                { CMD = UPPER_STRING(LEFT_STRING(DATA.TEXT,32)); }
	  
	  SELECT
	  {
		ACTIVE (CMD == '?'):         CALL 'MODULE HELP';
		ACTIVE (CMD == 'HELP'):      CALL 'MODULE HELP';
		ACTIVE (CMD == '?STATUS'):   CALL 'MODULE STATUS';
		ACTIVE (CMD == '?VERSION'):  SEND_STRING dvDEBUG,__MODULE_INFO__();

		ACTIVE (CMD == 'DVDEBUG-'):
		{
		  CALL 'STRING TO DEV' (DATA.TEXT,dvDebug);
		  SEND_STRING dvDebug,"sHeader,'dvDebug-',DEV_TO_STRING(dvDebug)"
		}
		ACTIVE (CMD == '?DVDEBUG'):
		{
		  SEND_STRING dvDebug,"sHeader,'dvDebug-',DEV_TO_STRING(dvDebug)"
		}

		ACTIVE (CMD == 'HEADER-'):
		{
		  SEND_STRING dvDebug,"sVDV,// display vdvControl D:P:S
							  ' HEADER ''',sHeader,'''',  // display old HEADER in single quotes
							  ' changed to ',
														  // display new HEADER in single quotes
							  '''',DATA.TEXT,''''";
		  sHeader = DATA.TEXT;      // update HEADER
		}
		ACTIVE (CMD == '?HEADER'): SEND_STRING dvDebug,"sVDV,' HEADER-''',sHeader,''''";
		
        // MIDI routing commands
		ACTIVE (CMD == 'INA-'):
		{
		  cMASK_A = DATA.TEXT[1];
		  SEND_COMMAND dvMIDI,"'INA-',cMASK_A";
		  IF (DEBUG_TX) SEND_STRING dvDEBUG,"sHeader,sDV,' INA-',cMASK_A";
		}
		ACTIVE (CMD == 'INB-'):
		{
		  cMASK_B = DATA.TEXT[1];
		  SEND_COMMAND dvMIDI,"'INB-',cMASK_B";
		  IF (DEBUG_TX) SEND_STRING dvDEBUG,"sHeader,sDV,' INB-',cMASK_B";
		}
		ACTIVE (CMD == 'INM-'):
		{
		  cMASK_M = DATA.TEXT[1];
		  SEND_COMMAND dvMIDI,"'INM-',cMASK_M";
		  SWITCH (cMASK_M & (MASK_OUT | MASK_THRU))
		  {
			CASE $03: sMASK_M = 'THRU & OUT';
			CASE $02: sMASK_M = 'OUT';
			CASE $01: sMASK_M = 'THRU';
			CASE $00: sMASK_M = 'NULL';
		  }
		  IF (DEBUG_TX) SEND_STRING dvDEBUG,"sHeader,sDV,' INM-',cMASK_M";
		}

		// pass MIDI voice channel on/off macros
		// SEND C vdvMidi,'NOTE-<comma-delimited list of MIDI channels>'
		ACTIVE (CMD == 'NOTE-'):
		{
		  STACK_VAR CHAR ch_2dim[16][2];
		  STACK_VAR CHAR ch[16];
		  STACK_VAR INTEGER i;
		  STACK_VAR INTEGER j;

		  // split the list of MIDI channels into seperate tokens
		  StringTokenizer(DATA.TEXT,',',ch_2dim);

		  //  populate the ch array from the tokens
		  FOR (i = 1; i <= LENGTH_ARRAY(ch_2dim); i++)
		  {
									  // assume ch_2dim contains a MIDI channel#)
			IF (LENGTH_ARRAY(ch_2dim[i]) > 0)
			{
			  j++;
			  ch[j] = ATOI(ch_2dim[i]) - 1;
			}
			SET_LENGTH_ARRAY(ch,j);
		  }
		  // update PASS_NOTE status from CH
		  FOR (i = 1; i <= 16; i++) OFF[statusPass[PASS_NOTE[i]]];
		  FOR (i = 1; i <= 16; i++)
		  {
			IF (LENGTH_ARRAY(ch) >= i)
			{
			  // [(ch[i] & $0F) + 1] allows use of MIDI CH# or PASS_<whatever>#
			  IF (ch[i] < 80)
				ON[statusPass[PASS_NOTE[(ch[i] & $0F) + 1]]];
			}
		  }

		  // send results to terminal
		  SEND_STRING dvDEBUG,"textStatus (PASS_NOTE,"sHeader,'NOTES'")";
		}

		// SEND C vdvMidi,'AFTT-<comma-delimited list of MIDI channels>'
		ACTIVE (CMD == 'AFTT-'):
		{
		  STACK_VAR CHAR ch_2dim[16][2];
		  STACK_VAR CHAR ch[16];
		  STACK_VAR INTEGER i;
		  STACK_VAR INTEGER j;

		  // split the list of MIDI channels into seperate tokens
		  StringTokenizer(DATA.TEXT,',',ch_2dim);

		  //  populate the ch array from the tokens
		  FOR (i = 1; i <= LENGTH_ARRAY(ch_2dim); i++)
		  {
									  // assume ch_2dim contains a MIDI channel#)
			IF (LENGTH_ARRAY(ch_2dim[i]) > 0)
			{
			  j++;
			  ch[j] = ATOI(ch_2dim[i]) - 1;
			}
			SET_LENGTH_ARRAY(ch,j);
		  }
		  // update PASS_AFTT status from CH
		  FOR (i = 1; i <= 16; i++) OFF[statusPass[PASS_AFTT[i]]];
		  FOR (i = 1; i <= 16; i++)
		  {
			IF (LENGTH_ARRAY(ch) >= i)
			{
			  // [(ch[i] & $0F) + 1] allows use of MIDI CH# or PASS_<whatever>#
			  IF (ch[i] < 80)
				ON[statusPass[PASS_AFTT[(ch[i] & $0F) + 1]]];
			}
		  }

		  // send results to terminal
		  SEND_STRING dvDEBUG,"textStatus (PASS_AFTT,"sHeader,'AFTERTOUCH'")";
		}

		// SEND C vdvMidi,'CTRL-<comma-delimited list of MIDI channels>'
		ACTIVE (CMD == 'CTRL-'):
		{
		  STACK_VAR CHAR ch_2dim[16][2];
		  STACK_VAR CHAR ch[16];
		  STACK_VAR INTEGER i;
		  STACK_VAR INTEGER j;

		  // split the list of MIDI channels into seperate tokens
		  StringTokenizer(DATA.TEXT,',',ch_2dim);

		  //  populate the ch array from the tokens
		  FOR (i = 1; i <= LENGTH_ARRAY(ch_2dim); i++)
		  {
									  // assume ch_2dim contains a MIDI channel#)
			IF (LENGTH_ARRAY(ch_2dim[i]) > 0)
			{
			  j++;
			  ch[j] = ATOI(ch_2dim[i]) - 1;
			}
			SET_LENGTH_ARRAY(ch,j);
		  }
		  // update PASS_CTRL status from CH
		  FOR (i = 1; i <= 16; i++) OFF[statusPass[PASS_CTRL[i]]];
		  FOR (i = 1; i <= 16; i++)
		  {
			IF (LENGTH_ARRAY(ch) >= i)
			{
			  // [(ch[i] & $0F) + 1] allows use of MIDI CH# or PASS_<whatever>#
			  IF (ch[i] < 80)
				ON[statusPass[PASS_CTRL[(ch[i] & $0F) + 1]]];
			}
		  }

		  // send results to terminal
		  SEND_STRING dvDEBUG,"textStatus (PASS_CTRL,"sHeader,'CONTROLLERS'")";
		}

		// SEND C vdvMidi,'PROG-<comma-delimited list of MIDI channels>'
		ACTIVE (CMD == 'PROG-'):
		{
		  STACK_VAR CHAR ch_2dim[16][2];
		  STACK_VAR CHAR ch[16];
		  STACK_VAR INTEGER i;
		  STACK_VAR INTEGER j;

		  // split the list of MIDI channels into seperate tokens
		  StringTokenizer(DATA.TEXT,',',ch_2dim);

		  //  populate the ch array from the tokens
		  FOR (i = 1; i <= LENGTH_ARRAY(ch_2dim); i++)
		  {
									  // assume ch_2dim contains a MIDI channel#)
			IF (LENGTH_ARRAY(ch_2dim[i]) > 0)
			{
			  j++;
			  ch[j] = ATOI(ch_2dim[i]) - 1;
			}
			SET_LENGTH_ARRAY(ch,j);
		  }
		  // update PASS_PROG status from CH
		  FOR (i = 1; i <= 16; i++) OFF[statusPass[PASS_PROG[i]]];
		  FOR (i = 1; i <= 16; i++)
		  {
			IF (LENGTH_ARRAY(ch) >= i)
			{
			  // [(ch[i] & $0F) + 1] allows use of MIDI CH# or PASS_<whatever>#
			  IF (ch[i] < 80)
				ON[statusPass[PASS_PROG[(ch[i] & $0F) + 1]]];
			}
		  }

		  // send results to terminal
		  SEND_STRING dvDEBUG,"textStatus (PASS_PROG,"sHeader,'PROGRAM CHANGES'")";
		}

		// SEND C vdvMidi,'BEND-<comma-delimited list of MIDI channels>'
		ACTIVE (CMD == 'BEND-'):
		{
		  STACK_VAR CHAR ch_2dim[16][2];
		  STACK_VAR CHAR ch[16];
		  STACK_VAR INTEGER i;
		  STACK_VAR INTEGER j;

		  // split the list of MIDI channels into seperate tokens
		  StringTokenizer(DATA.TEXT,',',ch_2dim);

		  //  populate the ch array from the tokens
		  FOR (i = 1; i <= LENGTH_ARRAY(ch_2dim); i++)
		  {
									  // assume ch_2dim contains a MIDI channel#)
			IF (LENGTH_ARRAY(ch_2dim[i]) > 0)
			{
			  j++;
			  ch[j] = ATOI(ch_2dim[i]) - 1;
			}
			SET_LENGTH_ARRAY(ch,j);
		  }
		  // update PASS_BEND status from CH
		  FOR (i = 1; i <= 16; i++) OFF[statusPass[PASS_BEND[i]]];
		  FOR (i = 1; i <= 16; i++)
		  {
			IF (LENGTH_ARRAY(ch) >= i)
			{
			  // [(ch[i] & $0F) + 1] allows use of MIDI CH# or PASS_<whatever>#
			  IF (ch[i] < 80)
				ON[statusPass[PASS_BEND[(ch[i] & $0F) + 1]]];
			}
		  }

		  // send results to terminal
		  SEND_STRING dvDEBUG,"textStatus (PASS_BEND,"sHeader,'PITCH BENDS'")";
		}
		// pass MIDI system on/off macros
		// SEND C vdvMidi,'<MIDI system type>-<pass system ch#>'
		ACTIVE (CMD == 'SYSX-'):
		{
		  IF (ATOI(DATA.TEXT) >= 1)
			ON[statusPass[PASS_SYSX]];
		  ELSE
			OFF[statusPass[PASS_SYSX]];

		  // send results to terminal
		  SEND_STRING dvDEBUG,"textStatus ("PASS_SYSX","sHeader,'SYSTEM EXCLUSIVE'")";
		}

		ACTIVE (CMD == 'CMMN-'):
		{
		  IF (ATOI(DATA.TEXT) >= 1)
			ON[statusPass[PASS_CMMN]];
		  ELSE
			OFF[statusPass[PASS_CMMN]];

		  // send results to terminal
		  SEND_STRING dvDEBUG,"textStatus ("PASS_CMMN","sHeader,'SYSTEM COMMON'")";
		}

		ACTIVE (CMD == 'REAL-'):
		{
		  IF (ATOI(DATA.TEXT) >= 1)
			ON[statusPass[PASS_REAL]];
		  ELSE
			OFF[statusPass[PASS_REAL]];

		  // send results to terminal
		  SEND_STRING dvDEBUG,"textStatus ("PASS_REAL","sHeader,'SYSTEM REALTIME'")";
		}

		ACTIVE (CMD == 'ACSN-'):
		{
		  IF (ATOI(DATA.TEXT) >= 1)
			ON[statusPass[PASS_ACSN]];
		  ELSE
			OFF[statusPass[PASS_ACSN]];

		  // send results to terminal
		  SEND_STRING dvDEBUG,"textStatus ("PASS_ACSN","sHeader,'ACTIVE SENSE'")";
		}

		// terse status queries
		ACTIVE (CMD == '?INA'): { SEND_STRING dvDEBUG,"'INA-',cMASK_A"; }
		ACTIVE (CMD == '?INB'): { SEND_STRING dvDEBUG,"'INB-',cMASK_B"; }
		ACTIVE (CMD == '?INM'): { SEND_STRING dvDEBUG,"'INM-',cMASK_M"; }
		ACTIVE (CMD == '?NOTE'): { SEND_STRING dvDEBUG,"'NOTE-',textMidiChannels(PASS_NOTE)" }
		ACTIVE (CMD == '?AFTT'): { SEND_STRING dvDEBUG,"'AFTT-',textMidiChannels(PASS_AFTT)" }
		ACTIVE (CMD == '?CTRL'): { SEND_STRING dvDEBUG,"'CTRL-',textMidiChannels(PASS_CTRL)" }
		ACTIVE (CMD == '?PROG'): { SEND_STRING dvDEBUG,"'PROG-',textMidiChannels(PASS_PROG)" }
		ACTIVE (CMD == '?BEND'): { SEND_STRING dvDEBUG,"'BEND-',textMidiChannels(PASS_BEND)" }
		ACTIVE (CMD == '?SYSX'): { SEND_STRING dvDEBUG,"'SYSX-',ITOA(statusPass[PASS_SYSX])" }
		ACTIVE (CMD == '?CMMN'): { SEND_STRING dvDEBUG,"'CMMN-',ITOA(statusPass[PASS_CMMN])" }
		ACTIVE (CMD == '?REAL'): { SEND_STRING dvDEBUG,"'REAL-',ITOA(statusPass[PASS_REAL])" }
		ACTIVE (CMD == '?ACSN'): { SEND_STRING dvDEBUG,"'ACSN-',ITOA(statusPass[PASS_ACSN])" }

		// add more COMMANDS as necessary...

		ACTIVE(TRUE):               // any cmd not handled above, send DATA.TEXT to midi device
		{
		  midiTransmit();
		}
	  }
    }
  }
}

BUTTON_EVENT[vdvMIDI,0]
{
  PUSH:
  {
	IF(DEBUG_CH)
	{
	  SEND_STRING dvDebug,"sHeader,
					  	   sDV,' ',
						  'PUSH ',
						   ITOA(BUTTON.INPUT.CHANNEL)"
	}
  }
  RELEASE:
  {
	IF(DEBUG_CH)
	{
	  SEND_STRING dvDebug,"sHeader,
					  	   sDV,' ',
						  'RELEASE ',
						   ITOA(BUTTON.INPUT.CHANNEL)"
	}
  }
}

(***********************************************************)
(*            THE ACTUAL PROGRAM GOES BELOW                *)
(***********************************************************)
DEFINE_PROGRAM

// debug on/off via channel
DEBUG_PASSED_RX = [vdvMIDI,508]
DEBUG_RX = [vdvMIDI,509]
DEBUG_TX = [vdvMIDI,510]
DEBUG_CH = [vdvMIDI,511]

(***********************************************************)
(*                     END OF PROGRAM                      *)
(*        DO NOT PUT ANY CODE BELOW THIS COMMENT           *)
(***********************************************************)