PROGRAM_NAME='DEV_TO_STRING'
(***********************************************************)
(*  FILE CREATED ON: 09/08/2008  AT: 17:06:51              *)
(***********************************************************)
(*  FILE_LAST_MODIFIED_ON: 11/08/2008  AT: 13:02:55        *)
(***********************************************************)
(*  FILE REVISION: 2.55                                    *)
(*  REVISION DATE: 11/08/2008  AT: 13:02:55                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Speed Tweaks:                                        *)
(*    * Use ITOA() instead of FORMAT() if 0 padding isn't  *)
(*      necessary.                                         *)
(*    * In control expressions use IF (var) instead of     *)
(*      IF (var > 0) or IF (var < value).                  *)
(*      For example, reverse FOR loops.                    *)
(*  - Moved function descriptions below DEFINE_FUNCTION    *)
(*    so they will be visible when code is folded above    *)
(*    the function.                                        *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.54                                    *)
(*  REVISION DATE: 10/06/2008  AT: 07:17:07                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Added __display_lib_version__ variable.              *)
(*                                                         *)
(***********************************************************)
(*!!FILE REVISION: 2.53 GLM                                *)
(*  REVISION DATE: 09/08/2008                              *)
(*  COMMENTS:                                              *)
(*  - DEV_TO_STRING subroutines now in seperate lib from   *)
(*    debug subroutines. Debug.axi includes this lib -     *)
(*    but now the DEV_TO_STRING subroutines can be used    *)
(*    seperately, if desired.                              *)
(*                                                         *)
(* DEV_TO_STRING FUNCTIONS AND CALLS                       *)
/*  1   FUNCTION CHAR[17] DEV_TO_STRING (DEV dvDEV)                           */
/*  2   FUNCTION CHAR[2303] DEV_ARRAY_TO_STRING (DEV dvDA[])                  */
/*  3   FUNCTION CHAR[25] DEVCHAN_TO_STRING (DEVCHAN dcDC)                    */
/*  4   FUNCTION CHAR[25] DEV_CHAN_TO_STRING (DEV dvDEV, INTEGER nCH)         */
/*  5   FUNCTION CHAR[25] DEVLEV_TO_STRING (DEVLEV dlDL)                      */
/*  6   FUNCTION CHAR[25] DEV_LEV_TO_STRING (DEV dvDEV, INTEGER nLV)          */
/*  7   FUNCTION CHAR[17] DPS_TO_STRING (DEV dvDEV)                           */
/*  8   CALL 'STRING TO DEV' (sDEV[],dvDEV)                                   */
(***********************************************************)
#IF_NOT_DEFINED __DEV_TO_STRING_LIB__
#DEFINE __DEV_TO_STRING_LIB__

(***********************************************************)
(*          DEVICE NUMBER DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_DEVICE

(***********************************************************)
(*               CONSTANT DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_CONSTANT

// VERSION FOR CODE

CHAR __DEV_TO_STRING_LIB_NAME__[20]  = '   DEV_TO_STRING.axi';
CHAR __DEV_TO_STRING_LIB_VERSION__[] = '2.55';

(***********************************************************)
(*               VARIABLE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_VARIABLE

#IF_NOT_DEFINED __display_lib_version__
VOLATILE __display_lib_version__ = TRUE
#END_IF
(***********************************************************)
(*           SUBROUTINE DEFINITIONS GO BELOW               *)
(***********************************************************)
(* EXAMPLE: DEFINE_FUNCTION <RETURN_TYPE> <NAME> (<PARAMETERS>) *)
(* EXAMPLE: DEFINE_CALL '<NAME>' (<PARAMETERS>) *)
(***********************************************************)
(*           SUBROUTINE DEFINITIONS GO BELOW               *)
(***********************************************************)
(* EXAMPLE: DEFINE_FUNCTION <RETURN_TYPE> <NAME> (<PARAMETERS>) *)
(* EXAMPLE: DEFINE_CALL '<NAME>' (<PARAMETERS>) *)

DEFINE_FUNCTION CHAR[17] DEV_TO_STRING (DEV dvDEV)
// DEFINE_FUNCTION DEV_TO_STRING
// PARAMETER:
// dvDEV - DEVICE USED
// RETURNS:
// dvDEV AS A STRING
{
  IF (dvDEV.SYSTEM)
	RETURN "ITOA(dvDEV.NUMBER),':',ITOA(dvDEV.PORT),':',ITOA(dvDEV.SYSTEM)";
  ELSE                    // if device is defined as local, i.e. system is 0
    RETURN "ITOA(dvDEV.NUMBER),':',ITOA(dvDEV.PORT),':',ITOA(SYSTEM_NUMBER)";
}

DEFINE_FUNCTION CHAR[2303] DEV_ARRAY_TO_STRING (DEV dvDA[])
// DEFINE_FUNCTION DEV_ARRAY_TO_STRING
// PARAMETER:
// dvDA - DEVICE ARRAY USED
// RETURNS:
// dvDA AS A STRING, comma seperated
{
  STACK_VAR INTEGER i;
  STACK_VAR INTEGER len;
  STACK_VAR CHAR    ASCII[2303];     // 2303 = ((17 + 1) * 128) - 1

  len = LENGTH_ARRAY(dvDA);

  IF (len > 0)
  {
	FOR (i = len; i; i--)
	{
	  IF (i = len)
	  {
		ASCII = "DEV_TO_STRING(dvDA[i])";
	  }
	  ELSE
	  {
		ASCII = "DEV_TO_STRING(dvDA[i]),',',ASCII";
	  }
	}
  }

  RETURN ASCII;
}

DEFINE_FUNCTION CHAR[25] DEVCHAN_TO_STRING (DEVCHAN dcDC)
// DEFINE_FUNCTION DEVCHAN_TO_STRING
// PARAMETER:
// dcDC - DEVCHAN USED
// RETURNS:
// DEVCHAN AS A STRING, IN BRACKETS
{
  IF (dcDC.DEVICE.SYSTEM = 0)       // if device is defined as local, i.e. system is 0
  {
	RETURN "'[',
			ITOA(dcDC.DEVICE.NUMBER),':',
			ITOA(dcDC.DEVICE.PORT),':',
			ITOA(SYSTEM_NUMBER),
			',',
			ITOA(dcDC.CHANNEL),
			']'";
  }
  ELSE
  {
	RETURN "'[',
			ITOA(dcDC.DEVICE.NUMBER),':',
			ITOA(dcDC.DEVICE.PORT),':',
			ITOA(dcDC.DEVICE.SYSTEM),
			',',
			ITOA(dcDC.CHANNEL),
			']'";
  }
}
DEFINE_FUNCTION CHAR[25] DEV_CHAN_TO_STRING (DEV dvDEV, INTEGER nCH)
// DEFINE_FUNCTION DEV_CHAN_TO_STRING
// PARAMETER:
// dvDEV - DEV USED
// nCH   - CHAN USED
// RETURNS:
// DEVCHAN AS A STRING, IN BRACKETS
{
  IF (dvDEV.SYSTEM = 0)             // if device is defined as local, i.e. system is 0
  {
	RETURN "'[',
			ITOA(dvDEV.NUMBER),':',
			ITOA(dvDEV.PORT),':',
			ITOA(SYSTEM_NUMBER),
			',',
			ITOA(nCH),
			']'";
  }
  ELSE
  {
  	RETURN "'[',
			ITOA(dvDEV.NUMBER),':',
			ITOA(dvDEV.PORT),':',
			ITOA(dvDEV.SYSTEM),
			',',
			ITOA(nCH),
			']'";
  }
}

DEFINE_FUNCTION CHAR[25] DEVLEV_TO_STRING (DEVLEV dlDL)
// DEFINE_FUNCTION DEVLEV_TO_STRING
// PARAMETER:
// dlDL - DEVLEV USED
// RETURNS:
// DEVLEV AS A STRING
{
  IF (dlDL.DEVICE.SYSTEM = 0)       // if device is defined as local, i.e. system is 0
  {
	RETURN "ITOA(dlDL.DEVICE.NUMBER),':',
			ITOA(dlDL.DEVICE.PORT),':',
			ITOA(SYSTEM_NUMBER),
			' L',
			FORMAT('%03d',dlDL.LEVEL)";
  }
  ELSE
  {
	RETURN "ITOA(dlDL.DEVICE.NUMBER),':',
			ITOA(dlDL.DEVICE.PORT),':',
			ITOA(dlDL.DEVICE.SYSTEM),
			' L',
			FORMAT('%03d',dlDL.LEVEL)";
  }
}
DEFINE_FUNCTION CHAR[25] DEV_LEV_TO_STRING (DEV dvDEV, INTEGER nLV)
// DEFINE_FUNCTION DEV_LEV_TO_STRING
// PARAMETER:
// dvDEV - DEV USED
// nLV   - LEV USED
// RETURNS:
// DEVLEV AS A STRING
{
  IF (dvDEV.SYSTEM = 0)             // if device is defined as local, i.e. system is 0
  {
	RETURN "ITOA(dvDEV.NUMBER),':',
			ITOA(dvDEV.PORT),':',
			ITOA(SYSTEM_NUMBER),
			' L',
			FORMAT('%03d',nLV)";
  }
  ELSE
  {
	RETURN "ITOA(dvDEV.NUMBER),':',
			ITOA(dvDEV.PORT),':',
			ITOA(dvDEV.SYSTEM),
			' L',
			FORMAT('%03d',nLV)";
  }
}

DEFINE_FUNCTION CHAR[17] DPS_TO_STRING (DEV dvDEV)
// DEFINE_FUNCTION DPS_TO_STRING
// PARAMETER:
// dvDEV - DEVICE USED
// RETURNS:
// dvDEV AS A STRING
// this function differs from the original DEV_TO_STRING in that it always
// returns the actual system number, never system 0
// - As of debug.axi v2.03, DPS_TO_STRING and DEV_TO_STRING are the same
{
  IF (dvDEV.SYSTEM = 0)             // if device is defined as local, i.e. system is 0
    RETURN "ITOA(dvDEV.NUMBER),':',ITOA(dvDEV.PORT),':',ITOA(SYSTEM_NUMBER)";
  ELSE
	RETURN "ITOA(dvDEV.NUMBER),':',ITOA(dvDEV.PORT),':',ITOA(dvDEV.SYSTEM)";
}

DEFINE_CALL 'STRING TO DEV' (char Txt[],dev dvDEV)
/* CALL 'STRING TO DEV'                                    */
/* FUNCTION: Takes ASCII string of the form '...<dev>' or  */
/*           '...<dev>:<prt>:<sys>' and puts those values  */
/*           into dvDEV.                                   */
/* PARAMETERS: Txt   - String that contains the dv:prt:sys */
/*                     as ASCII text.                      */
/*             dvDEV - storage for the returned value      */
{
  STACK_VAR INTEGER DV
  STACK_VAR INTEGER PRT
  STACK_VAR INTEGER SYS
  STACK_VAR INTEGER WherePrt
  STACK_VAR INTEGER WhereSys

									/*********************/
                                    /* get device number */
									/*********************/
  DV = ATOI(Txt)                    // if Txt does not contain a number,
		                            // DV will be 0 (the master)

									/*******************/
                                    /* get port number */
									/*******************/
  IF (FIND_STRING(Txt,':',2))       // if Txt is of form '...<dv>:<prt>...'
  {
    WherePrt = FIND_STRING(Txt,':',1)
    IF (WherePrt)                   // if there might be a port number
    {
									// if port 0 was entered and
									// Txt if of form '...<dv>:<prt>:<sys>'
	  IF (FIND_STRING(Txt,':0:',1) = WherePrt)
	    PRT = 0
      ELSE
      {
	    PRT = ATOI(RIGHT_STRING(Txt,LENGTH_ARRAY(Txt)-WherePrt))
	    IF (PRT = 0)                // if Txt does not contain a number past the
		  PRT = 1                   // first ':', PRT will be 1
	  }
    }
  }
  ELSE								// if Txt does not contain a number past the
    PRT = 1                         // first ':', PRT will be 1

									/*********************/
                                    /* get system number */
									/*********************/
									// if Txt is of form '...<dv>:<prt>:<sys>'
									// note: SYS 0 means local system
  IF (FIND_STRING(Txt,':',WherePrt + 1))
  {
	WhereSys = FIND_STRING(Txt,':',WherePrt + 1)
	SYS = ATOI(RIGHT_STRING(Txt,LENGTH_ARRAY(Txt)-WhereSys))
  }
  dvDEV = DV:PRT:SYS
}

(***********************************************************)
(*                THE EVENTS GOES BELOW                    *)
(***********************************************************)
DEFINE_EVENT

DATA_EVENT[0:1:0]
{
  ONLINE:
  {
	IF (__display_lib_version__ == TRUE)
	{
	  wait 50                         // WAIT FOR DEVICE HOLDOFF
	  {
		// WHAT VERSION?
		SEND_STRING 0:0:0,"'Using lib ',__DEV_TO_STRING_LIB_NAME__,
								  ', v',__DEV_TO_STRING_LIB_VERSION__";
	  }
	}
  }
}

#END_IF
(***********************************************************)
(*                     END OF PROGRAM                      *)
(*        DO NOT PUT ANY CODE BELOW THIS COMMENT           *)
(***********************************************************)