PROGRAM_NAME='MIDI small program example'
(***********************************************************)
(*  FILE CREATED ON: 08/14/2006  AT: 04:36:08              *)
(***********************************************************)
(*  FILE_LAST_MODIFIED_ON: 06/21/2010  AT: 08:23:35        *)
(***********************************************************)
(* The following code block is provided as a guide to      *)
(* programming the device(s) listed above. This is a       *)
(* sample only, and will most likely require modification  *)
(* to be integrated into a master program.                 *)
(*                                                         *)
(* Device-specific protocols should be obtained directly   *)
(* from the equipment manufacturer to ensure compliance    *)
(* with the most current versions. Within the limits       *)
(* imposed by any non-disclosure agreements, AMX will      *)
(* provide protocols upon request.                         *)
(*                                                         *)
(* If further programming assistance is required, please   *)
(* contact your AMX customer support team.                 *)
(***********************************************************)
/**************************/
/* small program example: */
/**************************/
#INCLUDE 'DEV_TO_STRING.axi'
#INCLUDE 'debug.axi'
#INCLUDE 'MIDI_API.axi'

(***********************************************************)
(*          DEVICE NUMBER DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_DEVICE

dvMIDI_1 = 96:1:0
//vdvMIDI_1 = 33096:1:0

(***********************************************************)
(*               CONSTANT DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_CONSTANT

(***********************************************************)
(*              STRUCTURE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_TYPE

(***********************************************************)
(*               VARIABLE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_VARIABLE

_sMidi midi1                        // storage for midi data

INTEGER MIDI1_DEBUG_RX = 1

(***********************************************************)
(*           SUBROUTINE DEFINITIONS GO BELOW               *)
(***********************************************************)
(* EXAMPLE: DEFINE_FUNCTION <RETURN_TYPE> <NAME> (<PARAMETERS>) *)
(* EXAMPLE: DEFINE_CALL '<NAME>' (<PARAMETERS>) *)

(***********************************************************)
(*                STARTUP CODE GOES BELOW                  *)
(***********************************************************)
DEFINE_START

(***********************************************************)
(*               MODULE DEFINITIONS GO BELOW               *)
(***********************************************************)
//DEFINE_MODULE 'AMX_AXB_MIDI' mdlMIDI_1(vdvMIDI_1,dvMIDI_1)

(***********************************************************)
(*                 THE EVENTS GO BELOW                     *)
(***********************************************************)
DEFINE_EVENT

DATA_EVENT[dvMIDI_1]
//DATA_EVENT[vdvMIDI_1]
{
  ONLINE:
  {
									/**************************/
                                    /* AXB-MIDI IO mask setup */
									/**************************/
    #warn 'Initialize AXB-MIDI I/O masks here'
									// INA TO MASTER, OUT, and THRU
	midiSetIoMask (DATA.DEVICE,'A',MIDI_MASK_MASTER|MIDI_MASK_OUT|MIDI_MASK_THRU,midi1.maskInput.A)
									// INB TO MASTER only
	midiSetIoMask (DATA.DEVICE,'B',MIDI_MASK_MASTER,midi1.maskInput.B)
									// MASTER TO THRU AND OUT
	midiSetIoMask (DATA.DEVICE,'M',MIDI_MASK_THRU|MIDI_MASK_OUT,midi1.maskInput.M)

//									/*****************************/
//									/* AMX_AXB_MIDI module setup */
//									/*****************************/
//	SEND_COMMAND vdvMIDI_1,'HEADER-MIDI_1 '
//									// Use PROGRAM (preset) messages on MIDI CH 1 & 6
//	SEND_COMMAND vdvMIDI_1,"'PROG-1,6'"
//									// Use CONTROLLER messages on MIDI CH 1 & 6 
//	SEND_COMMAND vdvMIDI_1,"'CTRL-1,6'"
//// If need to parse all the channels of data coming in,
//// you'll be better off NOT using the module
//									// Use PROGRAM (preset) messages on MIDI CH 1-16
////	SEND_COMMAND vdvMIDI_1,"'PROG-1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16'"
////									// Use CONTROLLER messages on MIDI CH 1-16
////	SEND_COMMAND vdvMIDI_1,"'CTRL-1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16'"
////									// Use NOTE messages on MIDI CH 1-16
////	SEND_COMMAND vdvMIDI_1,"'NOTE-1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16'"
////									// Use PITCH BEND messages on MIDI CH 1-16
////	SEND_COMMAND vdvMIDI_1,"'BEND-1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16'"
////									// Use AFTERTOUCH messages on MIDI CH 1-16
////	SEND_COMMAND vdvMIDI_1,"'AFTT-1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16'"
//	SEND_COMMAND vdvMIDI_1,"'SYSX-1'" // Use SYSTEM EXCLUSIVE MESSAGES
// 	SEND_COMMAND vdvMIDI_1,"'CMMN-1'" // Use other SYSTEM COMMON messages
//	SEND_COMMAND vdvMIDI_1,"'ACSN-1'" // Use Active Sense
//	SEND_COMMAND vdvMIDI_1,"'REAL-1'" // Use other SYSTEM REALTIME messages

									/********************/
									/* MIDI status init */
									/********************/
	OFF[midi1.RunningStatus]        // clear MIDI Running Status and Buffers
	midi1.DataBuffer = ""
	midi1.DataLength = 0
  }
  STRING:
  {
	IF (MIDI1_DEBUG_RX > 2)
	  CALL 'SEND ITOHEX TO DEBUG' (0:1:0,
								  "DEV_TO_STRING(DATA.DEVICE),' str: '",
								  "DATA.TEXT",
								   0,0)

	WHILE (LENGTH_ARRAY(DATA.TEXT))
	{
									// SYSTEM REAL TIME messages
									// should not affect Running Status per MIDI spec.
	  IF (DATA.TEXT[1] >= $F8)      // IF IT IS A REAL TIME MESSAGE
	  {
		SWITCH (GET_BUFFER_CHAR(DATA.TEXT))
		{
		  CASE $F8:                 // AXB-MIDI won't be able to keep up with this...
		  {
			IF (MIDI1_DEBUG_RX) SEND_STRING 0,"'TIMING CLOCK:$F8'"
		  }

		  CASE $F9:
		  {
			IF (MIDI1_DEBUG_RX) SEND_STRING 0,"'UNDEFINED:$F9'"
		  }

		  CASE $FA:
		  {
			IF (MIDI1_DEBUG_RX) SEND_STRING 0,"'START:$FA'"
		  }

		  CASE $FB:
		  {
			IF (MIDI1_DEBUG_RX) SEND_STRING 0,"'CONTINUE:$FB'"
		  }

		  CASE $FC:
		  {
			IF (MIDI1_DEBUG_RX) SEND_STRING 0,"'STOP:$FC'"
		  }

		  CASE $FD:
		  {
			IF (MIDI1_DEBUG_RX) SEND_STRING 0,"'UNDEFINED:$FD'"
		  }

		  CASE $FE:                 // MIDI controller heartbeat
		  {
			IF (MIDI1_DEBUG_RX > 1) SEND_STRING 0,"'ACTIVE SENSE:$FE'"
		  }

		  CASE $FF:
		  {
			IF (MIDI1_DEBUG_RX) SEND_STRING 0,"'RESET:$FF'"
		  }
		}
	  }
	  ELSE                          // SYSTEM COMMON and CHANNEL VOICE MESSAGEs
	  {
		midi1.RunningStatus = midiSEPERATE(1,DATA.TEXT,midi1.DataBuffer)
		midi1.DataLength = LENGTH_ARRAY(midi1.DataBuffer)

		SWITCH (midi1.RunningStatus & $F0)
		{
		  CASE $80:                 // Note Off: 2-byte data
		  {
			LOCAL_VAR INTEGER i

			midi1.MC = midi1.RunningStatus & $0F

			#warn 'add MIDI Note Off parsing in this block'
			// for i = note# byte; buffer contains a velocity byte;
			//         set i to next note# byte
			FOR (i = 1; i < midi1.DataLength; i = i + 2)
			{
									// store note status off
			  midiSetNoteStatus("midi1.DataBuffer[i],0",midi1.VD[midi1.mc+1].Note)
			  // Add more code to NOTE OFF section as necessary...
			}

			IF (MIDI1_DEBUG_RX)
			{
			                        // print note status on/off
			  midiPrintNoteStatusHex(0:1:0,
									"'Note Status CH ',ITOA(midi1.mc+1),' '",
									midi1.VD[midi1.mc+1].Note.hex)
			}
			// Add more code to NOTE OFF section as necessary...
          }
		  CASE $90:                 // Note On: 2-byte data
		  {
			LOCAL_VAR INTEGER i

			midi1.MC = midi1.RunningStatus & $0F

			#warn 'add MIDI Note On parsing in this block'
			// NOTE on with velocity 0 = NOTE off

			// for i = note# byte; buffer contains a velocity byte;
			//         set i to next note# byte
			FOR (i = 1; i < midi1.DataLength; i = i + 2)
			{
								  // store note status on/off
			  midiSetNoteStatus(MID_STRING(midi1.DataBuffer,i,2),
								midi1.VD[midi1.mc+1].Note)
			  // Add more code to NOTE ON section as necessary...
 			}

			IF (MIDI1_DEBUG_RX)
			{
									// print note status on/off
			  midiPrintNoteStatusHex(0:1:0,
									"'Note Status CH ',ITOA(midi1.MC+1),' '",
									midi1.VD[midi1.MC+1].Note.hex)
			}
			// Add more code to NOTE ON section as necessary...
		  }
		  CASE $A0:                 // Poly AfterTouch: 2-byte data
		  {
			midi1.MC = midi1.RunningStatus & $0F

			CALL 'SEND MIDI TO DEBUG' (0:1:0,
									  "'POLY AFTERTOUCH CH ',ITOA(midi1.MC+1),':'",
									  "midi1.DataBuffer",
									  0,0,midi1.RunningStatus)
            // Add more code to POLY AT section as necessary...
		  }
		  CASE $B0:                 // Controller: 2-byte data
		  {
			LOCAL_VAR INTEGER i
			LOCAL_VAR INTEGER ctrl_ndx

			midi1.MC = midi1.RunningStatus & $0F

			#warn 'add MIDI Controller parsing in this block'
			// for i = controller byte; buffer contains a value byte;
			//         set i to next controller byte
			FOR (i = 1; i < midi1.DataLength; i = i + 2)
			{
								  // store controller data for later use
			  ctrl_ndx = midi1.Databuffer[i]+1
			  midi1.VD[midi1.MC+1].CTRL[ctrl_ndx] = midi1.Databuffer[i+1]
			  
			  IF (MIDI1_DEBUG_RX)
			  {
				CALL 'SEND MIDI TO DEBUG' (0:1:0,
										"'CONTROLLER CH ',ITOA(midi1.MC+1),':'",
										"ctrl_ndx - 1,midi1.VD[midi1.MC+1].CTRL[ctrl_ndx]",
										0,0,midi1.RunningStatus)
			  }
			  // Add more code to CTRL section as necessary...
			}
		  }
		  CASE $C0:                 // Program Change: 1-byte data
		  {
			LOCAL_VAR INTEGER ndxMC // midi1.MC + 1 is much used in this section, 
			                        // so better to store it in a LOCAL_VAR
		  
			midi1.MC = midi1.RunningStatus & $0F
			ndxMC = midi1.MC + 1

			IF (midi1.DataLength)   // last program change received is the current program change
			{
			  midi1.VD[ndxMC].PROG = midi1.DataBuffer[midi1.DataLength]

			  #warn 'add MIDI Program (preset) parsing in this block'
			  // If you're tracking MIDI program banks:
			  // Probably better to do this here, not under very busy controller parsing.
			  midi1.VD[ndxMC].BANK.LSB = midi1.VD[ndxMC].CTRL[M_NDX_BANK_LSB]
			  midi1.VD[ndxMC].BANK.MSB = midi1.VD[ndxMC].CTRL[M_NDX_BANK_MSB]
			  midi1.VD[ndxMC].BANK.VALUE =
				midiUnpack_14_Bits("midi1.VD[ndxMC].BANK.LSB,midi1.VD[ndxMC].BANK.MSB",MIDI_DATA_LE)

			  IF (MIDI1_DEBUG_RX)
			  {
									// Program # displayed is 1-128, not 0-127
				SEND_STRING 0,"'Midi Channel ',ITOA(ndxMC),
								' Bank ',ITOA(midi1.VD[ndxMC].BANK.VALUE),
								' Program ',ITOA(midi1.VD[ndxMC].PROG+1)"
			  }
			  // Add more code to PROG section as necessary...
			}
		  }
		  CASE $D0:                 // Channel Aftertouch: 1-byte data
		  {
			midi1.MC = midi1.RunningStatus & $0F

			IF (MIDI1_DEBUG_RX)
			{
			  CALL 'SEND MIDI TO DEBUG' (0:1:0,
									  "'AFTERTOUCH CH ',ITOA(midi1.MC+1),':'",
									  "midi1.DataBuffer",
									  0,0,midi1.RunningStatus)
			}
			// Add more code to AT section as necessary...
		  }
		  CASE $E0:                 // Pitch Wheel: 2-byte data
		  {
			LOCAL_VAR INTEGER i
			LOCAL_VAR INTEGER bendValue

			midi1.MC = midi1.RunningStatus & $0F

			IF (MIDI1_DEBUG_RX)
			{
			  CALL 'SEND MIDI TO DEBUG' (0:1:0,
										"'PITCH BEND CH ',ITOA(midi1.MC+1),':'",
										"midi1.DataBuffer",
										0,0,midi1.RunningStatus)
			}

			// for i = bend LSB; buffer contains a bend MSB;
			//         set i to next bend LSB
			FOR (i = 1; i < midi1.DataLength; i = i + 2)
			{
								  // get and display pitch bend value
			  bendValue = midiUnpack_14_Bits(MID_STRING(midi1.DataBuffer,i,2),MIDI_DATA_LE)
			  SEND_STRING 0,"'BEND VALUE:$',ITOHEX(bendValue)"
			  // Add more code to BEND section as necessary...
			}
								  // optional: store bend value for later use...
			midi1.VD[midi1.MC+1].BEND.VALUE = bendValue
			// Add more code to BEND section as necessary...
		  }
		  CASE $F0:
		  {
			SWITCH (midi1.RunningStatus & $0F)
			{
			  CASE $0:
			  {
				IF (MIDI1_DEBUG_RX)
				{
				  CALL 'SEND ITOHEX TO DEBUG' (0:1:0,
											   "'SYSTEM EXCLUSIVE:'",
											   "midi1.DataBuffer",
											   0,0)
				}
				#warn 'add MIDI Sysex parsing in this block'
			    // Add more code to SYSX section as necessary...for example:
						            // if End-of-SYSX byte is found...
				IF (FIND_STRING(midi1.DataBuffer,"$F7",1)) 
				{
				  // parse completed message here...or EOM if it's too long.
				}
				ELSE
				{
				  // if desired, parse partial message here...
				}
			  }
			  CASE $1:
			  CASE $4:
			  CASE $5:
			  {
				IF (MIDI1_DEBUG_RX)
				{
				  CALL 'SEND MIDI TO DEBUG' (0:1:0,
											"'UNDEFINED:'",
											"midi1.DataBuffer",
											0,0,midi1.RunningStatus)
				}
			  }
			  CASE $2:              // AXB-MIDI probably won't be able to keep up with this, either
			  {
				IF (MIDI1_DEBUG_RX)
				{
				  CALL 'SEND MIDI TO DEBUG' (0:1:0,
											"'SONG POSITION POINTER:'",
											"midi1.DataBuffer",
											0,0,midi1.RunningStatus)
				}
			  }
			  CASE $3:
			  {
			  	IF (MIDI1_DEBUG_RX)
				{
				  CALL 'SEND MIDI TO DEBUG' (0:1:0,
											"'SONG SELECT:'",
											"midi1.DataBuffer",
											0,0,midi1.RunningStatus)
				}
			  }
			  CASE $6:
			  {
				IF (MIDI1_DEBUG_RX)
				{
				  CALL 'SEND MIDI TO DEBUG' (0:1:0,
											"'TUNE REQUEST:'",
											"midi1.DataBuffer",
											0,0,midi1.RunningStatus)
				}
			  }
			  CASE $7: // this will not happen when using midiSeperate()
			  {
				IF (MIDI1_DEBUG_RX)
				{
				  CALL 'SEND MIDI TO DEBUG' (0:1:0,
											"'END-OF-SYSEX:'",
											"midi1.DataBuffer",
											0,0,midi1.RunningStatus)
				}
			  }
			}
		  }
		}
	  }
	}
  }
}

(***********************************************************)
(*            THE ACTUAL PROGRAM GOES BELOW                *)
(***********************************************************)


(***********************************************************)
(*                     END OF PROGRAM                      *)
(*        DO NOT PUT ANY CODE BELOW THIS COMMENT           *)
(***********************************************************)
