PROGRAM_NAME='MIDI_API'
(***********************************************************)
(*  FILE CREATED ON: 08/14/2006  AT: 04:36:08              *)
(***********************************************************)
(*  FILE_LAST_MODIFIED_ON: 06/21/2010  AT: 08:24:55        *)
(***********************************************************)
(*  FILE REVISION: 2.14                                    *)
(*  REVISION DATE: 06/21/2010  AT: 08:24:55                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Updated midiSEPERATE() so it no longer needs to be   *)
(*    instanced per device. Instead an index must be       *)
(*    passed in as a parameter.                            *)
(*    The old version of the subroutine has been renamed   *)
(*    midiSEPERATE_single().                               *)
(*  - Updated MIDI small program example.                  *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.13                                    *)
(*  REVISION DATE: 08/02/2009  AT: 10:37:02                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - midiNoteText parameter is an integer, not a char.    *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.12                                    *)
(*  REVISION DATE: 04/30/2009  AT: 18:37:47                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Added CONSTANTs for Controller 123 All Notes Off.    *)
(*  - midiStatusByteTable warning: Dereferencing an        *)
(*    array of structs is noticeably slower than just      *)
(*    using the byte value, especially when doing 6 at     *)
(*    once.                                                *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.11                                    *)
(*  REVISION DATE: 01/28/2009  AT: 08:10:23                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Added CONSTANTs for Controller 88, Note Vel LSB.     *)
(*  - Corrected NDX's for Controllers 65-84 to the values  *)
(*    66-85.                                               *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.10                                    *)
(*  REVISION DATE: 11/10/2008  AT: 20:05:37                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Minor code and comment tweaks in Small Program       *)
(*    Example.                                             *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.09                                    *)
(*  REVISION DATE: 11/09/2008  AT: 20:39:09                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Tested both with and without AMX_AXB_MIDI module.    *)
(*  - More Speed Tweaks:                                   *)
(*    * Avoid storing little used data, or data offsets    *)
(*      that have a high traffic impact during storage.    *)
(*      For example, avoid storing MIDI_CH indexes 1-16,   *)
(*      unless in a section of code where the indexes are  *)
(*      highly used - then use a LOCAL_VAR.                *)
(*  - Code Cleanup:                                        *)
(*    * Simplified MIDI structs.                           *)
(*    * Deleted unused MIDI structs and functions.         *)
(*  - Added velocity storage to midiSetNoteStatus().       *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.08                                    *)
(*  REVISION DATE: 11/03/2008  AT: 10:49:08                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Speed tweaks                                         *)
(*    * Use LOCAL_VAR <var> instead of STACK_VAR <var>     *)
(*      inside of FOR and WHILE loops.                     *)
(*    * Reduce FOR and WHILE loop control expressions to   *)
(*      only the variable name where possible.             *)
(*    * Avoid use of LENGTH_ARRAY or LENGTH_STRING() in    *)
(*      FOR or WHILE loop control expressions.             *)
(*    For example:                                         *)
(*    Bad  -   FOR (i = 1; i <= LENGTH_STRING(str); i++)   *)
(*    Good -   len = LENGTH_STRING(str)                    *)
(*             FOR (i = 1; i <= len; i++)                  *)
(*    Better - FOR (i = LENGTH_STRING(str); i > 0; i--)    *)
(*    Best -   FOR (i = LENGTH_STRING(str); i; i--)        *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.07                                    *)
(*  REVISION DATE: 10/06/2008  AT: 08:11:48                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Added __display_lib_version__ variable.              *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.06                                    *)
(*  REVISION DATE: 09/15/2008  AT: 07:11:44                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - midiSetNoteStatus() will return $FE as error if the  *)
(*    data passed in does not have a data[2] byte.         *)
(*  - Tweaked output of midiPrintNoteStatusHex().          *)
(*  - Added comments to clarify _sMidi_note_status.hex     *)
(*    data is little endian.                               *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.05                                    *)
(*  REVISION DATE: 08/25/2008  AT: 12:24:49                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Added struct member _sMidi_14_bits .BANK to structs  *)
(*    that contain a member _sMidi_prog .PROG.             *)
(*  - Shortened some struct member names to keep lines     *)
(*    of code indexing the structs from getting extremely  *)
(*    long.                                                *)
(*    .MidiChannel is now .MC                              *)
(*    .VoiceData is now .VD                                *)
(*    .INDEX is now .i                                     *)
(*  - Added midiPrintNoteStatusHex() for debugging MIDI    *)
(*    note on/off.                                         *)
(*  - Added midiSetNoteStatus() and _sMidi_note_status for *)
(*    tracking MIDI note on/off.                           *)
(*  - Added midiSetControllerData().                       *)
(*  - Added .value member to _sMidi_14_bits.               *)
(*  - Added "order" parameter to midiUnpack_14_Bits().     *)
(*  - All function names now start with "midi...":         *)
(*    SEPERATE_MIDI() is now midiSEPERATE().               *)
(*    setMidiIoMask() is now midiSetIoMask().              *)
(*  - Made sure "MIDI_..." is prepended  to all constants, *)
(*    except controllers.                                  *)
(*  - Controller numbers are prepended with "M_CTL...".    *)
(*  - Controller indexes are prepended with "M_NDX...".    *)
(*  - Moved AMX_AXB_MIDI constants from DEFINE_VARIABLE to *)
(*    DEFINE_CONSTANT.                                     *)
(*  - Removed unused ACSN_TX constant.                     *)
(*  - Added more controller number constants.              *)
(*  - Tweaked comments, code cleanup.                      *)
(*  - Added note status parsing to example code.           *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.04                                    *)
(*  REVISION DATE: 08/18/2008  AT: 06:14:00                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Prepended "_s" to all struct names.                  *)
(*  - Added status byte upper nybble constants.            *)
(*  - midiUnpack_14_Bits() now requires a single 2-byte    *)
(*    data packet, "lsb,msb".                              *)
(*    This should improve speed of parsing pitch bends as  *)
(*    the NetLinx interpreter no longer has to split the   *)
(*    data packet before passing it to the function.       *)
(*  - setMidiIoMask() no longer requires DEV_TO_STRING()   *)
(*    function from debug.axi.                             *)
(*  - Tweaked comments.                                    *)
(*  - Updated small program example for use with           *)
(*    AMX_AXB_MIDI.tko v1.14.                              *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: v2.03                                   *)
(*  REVISION DATE: 01/12/2008  AT: 09:06:19                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Removed features designed for a channel/level based  *)
(*    MIDI module, it was a failed experiment.             *)
(*  - Changed indexes from CHAR to INTEGER as necessary.   *)
(*  - Updated small program example for use with           *)
(*    AMX_AXB_MIDI.tko v1.11.                              *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: Rev 2.02                                *)
(*  REVISION DATE: 01/06/2008  AT: 21:11:36                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - tweaked SEPERATE_MIDI                                *)
(*                                                         *)
(***********************************************************)
(* midi functions                                          *)
/* 1  char[8] midiNoteText (char noteNumber)                                    */
/* 2          midiPrintNoteStatusHex (DEV dvD, CHAR hdr[], long hex[4])         */
/* 3  char[2] midiPack_14_Bits (integer nValue, integer order)                  */
/* 4  integer midiUnpack_14_Bits (char data[2], integer order)                  */
/* 5  char    midiSEPERATE(char INCOMING[], char DATA[])                        */
/* 6          midiSetIoMask (dvMODULE, cINPUT, nMASK_NEW, nMASK_CUR)            */
/* 7  integer midiSetNoteStatus (CHAR data[2], _sMidi_note_status status)       */
#IF_NOT_DEFINED __MIDI_API__
#DEFINE __MIDI_API__
/*
 * tab-width: 4
 * indent-level: 2
 *
 */
(***********************************************************)
(*          DEVICE NUMBER DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_DEVICE

#IF_NOT_DEFINED __dvMidiDebug__
__dvMidiDebug__ = 0:1:0
#END_IF

(***********************************************************)
(*               CONSTANT DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_CONSTANT                     // version

// VERSION FOR CODE

CHAR __MIDI_API_NAME__[12]     = 'MIDI_API.axi';
CHAR __MIDI_API_VERSION__[]    = '2.14';

DEFINE_CONSTANT                     // MIDI Status bytes
/*
 * STATUS BYTES:
 * FOR CHANNEL VOICE MESSAGES
 * UPPER NYBBLE IS MESSAGE TYPE
 * LOWER NYBBLE IS MIDI CHANNEL
 * FOR SYSTEM MESSAGES
 * UPPER NYBBLE IS ALWAYS $F
 * LOWER NYBBLE IS MESSAGE TYPE
 */
DEFINE_CONSTANT                     // MIDI Status byte - upper nybbles

MIDI_NOTE_OFF   = $80               // "$80 & <midi_ch>,<note #>, <velocity>"
MIDI_NOTE_ON    = $90               // "$90 & <midi_ch>,<note #>, <velocity>"
MIDI_POLY_AT    = $A0               // "$A0 & <midi_ch>,<note #>, <pressure>"
MIDI_CONTROL    = $B0               // "$B0 & <midi_ch>,<controller #>, <value>"
MIDI_PROGRAM    = $C0               // "$C0 & <midi_ch>,<patch # 0-127>"
MIDI_AT         = $D0               // "$D0 & <midi_ch>,<pressure>"
MIDI_BEND       = $E0               // "$E0 & <midi_ch>,<lower 7 bits>, <upper 7 bits>"
MIDI_SYSTEM     = $F0               // "$F0 & <msg type>[,<data bytes>]"

DEFINE_CONSTANT                     // MIDI channel values
                                    // Status byte lower nybble of voice channel messages
MIDI_CH_01      = $00
MIDI_CH_02      = $01
MIDI_CH_03      = $02
MIDI_CH_04      = $03
MIDI_CH_05      = $04
MIDI_CH_06      = $05
MIDI_CH_07      = $06
MIDI_CH_08      = $07
MIDI_CH_09      = $08
MIDI_CH_10      = $09
MIDI_CH_11      = $0A
MIDI_CH_12      = $0B
MIDI_CH_13      = $0C
MIDI_CH_14      = $0D
MIDI_CH_15      = $0E
MIDI_CH_16      = $0F

DEFINE_CONSTANT                     // MIDI controller numbers
// "M_CTL_..." values are the data values 0-127
// "M_NDX_..." values are "CTL_..." + 1.
// "M_NDX_..." values are used for indexing arrays that store controller values.
                                    // SELECTED MIDI CONTROLLER # BYTES
/* Controller Group 1 Basic MSB 0-31 */
M_CTL_BANK_MSB     = 0;             // BANK SELECT MSB
M_CTL_MOD          = 1;             // MODULATION WHEEL
M_CTL_BRE          = 2;             // BREATH CONTROLLER
M_CTL_FOO          = 4;             // FOOT CONTROLLER
M_CTL_POR_TIME     = 5;             // PORTAMENTO TIME
M_CTL_DATA         = 6;             // DATA ENTRY
M_CTL_VOL          = 7;             // VOLUME
M_CTL_BAL          = 8;             // BALANCE
M_CTL_PAN          = 10;            // PAN
M_CTL_EXP          = 11;            // EXPRESSION
M_CTL_FX1          = 12;            // EFFECT CONTROL 1
M_CTL_FX2          = 13;            // EFFECT CONTROL 2
M_CTL_GP1          = 16;            // GENERAL PURPOSE CONTROLLER # 1
M_CTL_GP2          = 17;            // GENERAL PURPOSE CONTROLLER # 2
M_CTL_GP3          = 18;            // GENERAL PURPOSE CONTROLLER # 3
M_CTL_GP4          = 19;            // GENERAL PURPOSE CONTROLLER # 4

M_NDX_BANK_MSB     = 1;             // BANK SELECT MSB
M_NDX_MOD          = 2;             // MODULATION WHEEL
M_NDX_BRE          = 3;             // BREATH CONTROLLER
M_NDX_FOO          = 5;             // FOOT CONTROLLER
M_NDX_POR_TIME     = 6;             // PORTAMENTO TIME
M_NDX_DATA         = 7;             // DATA ENTRY
M_NDX_VOL          = 8;             // VOLUME
M_NDX_BAL          = 9;             // BALANCE
M_NDX_PAN          = 11;            // PAN
M_NDX_EXP          = 12;            // EXPRESSION
M_NDX_FX1          = 13;            // EFFECT CONTROL 1
M_NDX_FX2          = 14;            // EFFECT CONTROL 2
M_NDX_GP1          = 17;            // GENERAL PURPOSE CONTROLLER # 1
M_NDX_GP2          = 18;            // GENERAL PURPOSE CONTROLLER # 2
M_NDX_GP3          = 19;            // GENERAL PURPOSE CONTROLLER # 3
M_NDX_GP4          = 20;            // GENERAL PURPOSE CONTROLLER # 4

/* Controller Group 2 Basic LSB 32-63 */
M_CTL_BANK_LSB     = 32;            // BANK SELECT lsb
M_CTL_MOD_LSB      = 33;            // MODULATION WHEEL lsb
M_CTL_BRE_LSB      = 34;            // BREATH CONTROLLER lsb
M_CTL_FOO_LSB      = 36;            // FOOT CONTROLLER lsb
M_CTL_POR_TIME_LSB = 37;            // PORTAMENTO TIME lsb
M_CTL_DATA_LSB     = 38;            // DATA ENTRY lsb
M_CTL_VOL_LSB      = 39;            // VOLUME lsb
M_CTL_BAL_LSB      = 40;            // BALANCE lsb
M_CTL_PAN_LSB      = 42;            // PAN lsb
M_CTL_EXP_LSB      = 43;            // EXPRESSION lsb
M_CTL_FX1_LSB      = 44;            // EFFECT CONTROL 1 lsb
M_CTL_FX2_LSB      = 45;            // EFFECT CONTROL 2 lsb
M_CTL_GP1_LSB      = 48;            // GENERAL PURPOSE CONTROLLER # 1 lsb
M_CTL_GP2_LSB      = 49;            // GENERAL PURPOSE CONTROLLER # 2 lsb
M_CTL_GP3_LSB      = 50;            // GENERAL PURPOSE CONTROLLER # 3 lsb
M_CTL_GP4_LSB      = 51;            // GENERAL PURPOSE CONTROLLER # 4 lsb

M_NDX_BANK_LSB     = 33;            // BANK SELECT lsb
M_NDX_MOD_LSB      = 34;            // MODULATION WHEEL lsb
M_NDX_BRE_LSB      = 35;            // BREATH CONTROLLER lsb
M_NDX_FOO_LSB      = 37;            // FOOT CONTROLLER lsb
M_NDX_POR_TIME_LSB = 38;            // PORTAMENTO TIME lsb
M_NDX_DAT_LSB      = 39;            // DATA ENTRY lsb
M_NDX_VOL_LSB      = 40;            // VOLUME lsb
M_NDX_BAL_LSB      = 41;            // BALANCE lsb
M_NDX_PAN_LSB      = 43;            // PAN lsb
M_NDX_EXP_LSB      = 44;            // EXPRESSION lsb
M_NDX_FX1_LSB      = 45;            // EFFECT CONTROL 1 lsb
M_NDX_FX2_LSB      = 46;            // EFFECT CONTROL 2 lsb
M_NDX_GP1_LSB      = 49;            // GENERAL PURPOSE CONTROLLER # 1 lsb
M_NDX_GP2_LSB      = 50;            // GENERAL PURPOSE CONTROLLER # 2 lsb
M_NDX_GP3_LSB      = 51;            // GENERAL PURPOSE CONTROLLER # 3 lsb
M_NDX_GP4_LSB      = 52;            // GENERAL PURPOSE CONTROLLER # 4 lsb

/* Controller Group 3 Additional Switches and Controllers 64 - 95 */
M_CTL_HOLD         = 64;            // HOLD:       63 or less = OFF, 64 or greater = ON
M_CTL_POR          = 65;            // PORTAMENTO: 63 or less = OFF, 64 or greater = ON
M_CTL_SOS          = 66;            // SOSTENUTO:  63 or less = OFF, 64 or greater = ON
M_CTL_SOFT         = 67;            // SOFT PEDAL: 63 or less = OFF, 64 or greater = ON
M_CTL_LEGATO       = 68;            // LEGATO:     63 or less = OFF, 64 or greater = ON
M_CTL_HOLD_2       = 69;            // HOLD 2:     63 or less = OFF, 64 or greater = ON
M_CTL_SC1          = 70;            // SOUND CONTROLLER 1 (Sound Variation)
M_CTL_SC2          = 71;            // SOUND CONTROLLER 2 (Timbre)
M_CTL_SC3          = 72;            // SOUND CONTROLLER 3 (Release Time)
M_CTL_SC4          = 73;            // SOUND CONTROLLER 4 (Attack Time)
M_CTL_SC5          = 74;            // SOUND CONTROLLER 5 (Brightness)
M_CTL_SC6          = 75;            // SOUND CONTROLLER 6 (GM2 Decay Time)
M_CTL_SC7          = 76;            // SOUND CONTROLLER 7 (GM2 Vibrato Rate)
M_CTL_SC8          = 77;            // SOUND CONTROLLER 8 (GM2 Vibrato Depth)
M_CTL_SC9          = 78;            // SOUND CONTROLLER 9 (GM2 Vibrato Delay)
M_CTL_SC10         = 79;            // SOUND CONTROLLER 10 (GM2 Undefined)
M_CTL_GP5          = 80;            // GENERAL PURPOSE CONTROLLER # 5
M_CTL_GP6          = 81;            // GENERAL PURPOSE CONTROLLER # 6
M_CTL_GP7          = 82;            // GENERAL PURPOSE CONTROLLER # 7
M_CTL_GP8          = 83;            // GENERAL PURPOSE CONTROLLER # 8
M_CTL_POR_CTL      = 84;            // PORTAMENTO CONTROL: 0-127 lsb
M_CTL_VEL_LSB      = 88;            // Note Velocity lsb
M_CTL_REV          = 91;            // REVERB SEND
M_CTL_TREM         = 92;            // TREMELO DEPTH
M_CTL_CHO          = 93;            // CHORUS SEND
M_CTL_CEL          = 94;            // CELESTE (DETUNE) DEPTH
M_CTL_PHAS         = 95;            // PHASER DEPTH

M_NDX_HOLD         = 65;            // HOLD:  63 or less = OFF, 64 or greater = ON
M_NDX_POR          = 66;            // PORTAMENTO: 63 or less = OFF, 64 or greater = ON
M_NDX_SOS          = 67;            // SOSTENUTO:  63 or less = OFF, 64 or greater = ON
M_NDX_SOFT         = 68;            // SOFT PEDAL: 63 or less = OFF, 64 or greater = ON
M_NDX_LEGATO       = 69;            // LEGATO:     63 or less = OFF, 64 or greater = ON
M_NDX_HOLD_2       = 70;            // HOLD 2:     63 or less = OFF, 64 or greater = ON
M_NDX_SC1          = 71;            // SOUND CONTROLLER 1 (Sound Variation)
M_NDX_SC2          = 72;            // SOUND CONTROLLER 2 (Timbre)
M_NDX_SC3          = 73;            // SOUND CONTROLLER 3 (Release Time)
M_NDX_SC4          = 74;            // SOUND CONTROLLER 4 (Attack Time)
M_NDX_SC5          = 75;            // SOUND CONTROLLER 5 (Brightness)
M_NDX_SC6          = 76;            // SOUND CONTROLLER 6 (GM2 Decay Time)
M_NDX_SC7          = 77;            // SOUND CONTROLLER 7 (GM2 Vibrato Rate)
M_NDX_SC8          = 78;            // SOUND CONTROLLER 8 (GM2 Vibrato Depth)
M_NDX_SC9          = 79;            // SOUND CONTROLLER 9 (GM2 Vibrato Delay)
M_NDX_SC10         = 80;            // SOUND CONTROLLER 10 (GM2 Undefined)
M_NDX_GP5          = 81;            // GENERAL PURPOSE CONTROLLER # 5
M_NDX_GP6          = 82;            // GENERAL PURPOSE CONTROLLER # 6
M_NDX_GP7          = 83;            // GENERAL PURPOSE CONTROLLER # 7
M_NDX_GP8          = 84;            // GENERAL PURPOSE CONTROLLER # 8
M_NDX_VEL_LSB      = 89;            // Note Velocity lsb
M_NDX_REV          = 92;            // REVERB SEND
M_NDX_TREM         = 93;            // TREMELO DEPTH
M_NDX_CHO          = 94;            // CHORUS SEND
M_NDX_CEL          = 95;            // CELESTE (DETUNE) DEPTH
M_NDX_PHAS         = 96;            // PHASER DEPTH

/* Controller Group 4: 96 - 119 */
M_CTL_DATA_INC     = 96;            // DATA INCREMENT
M_CTL_DATA_DEC     = 97;            // DATA DECREMENT
M_CTL_NRPN_LSB     = 98;            // NON-REGISTERED PARAMETER NUM LSB
M_CTL_NRPN_MSB     = 99;            // NON-REGISTERED PARAMETER NUM MSB
M_CTL_RPN_LSB      = 100;           // REGISTERED PARAMETER NUM LSB
M_CTL_RPN_MSB      = 101;           // REGISTERED PARAMETER NUM MSB

M_NDX_DATA_INC     = 97;            // DATA INCREMENT
M_NDX_DATA_DEC     = 98;            // DATA DECREMENT
M_NDX_NRPN_LSB     = 99;            // NON-REGISTERED PARAMETER NUM LSB
M_NDX_NRPN_MSB     = 100;           // NON-REGISTERED PARAMETER NUM MSB
M_NDX_RPN_LSB      = 101;           // REGISTERED PARAMETER NUM LSB
M_NDX_RPN_MSB      = 102;           // REGISTERED PARAMETER NUM MSB

/* Controller Group 5: Channel Mode messages */
M_CTL_SOUND_OFF    = 120;           // ALL SOUND OFF, 2nd byte is always "0"
M_CTL_RESET_ALL    = 121;           // RESET ALL CONTROLLERS, 2nd byte is always "0"
M_CTL_LOCAL_CTL    = 122;           // LOCAL CONTROL: 0 = OFF, 127 = ON
M_CTL_NOTE_OFF     = 123;           // ALL NOTES OFF, 2nd byte is always "0"
M_CTL_OMNI_OFF     = 124;           // OMNI MODE OFF, 2nd byte is always "0"
M_CTL_OMNI_ON      = 125;           // OMNI MODE ON, 2nd byte is always "0"
M_CTL_MONO_ON      = 126;           // MONO MODE ON, 2nd byte is number of channels, or
                                    // 0 the number of channels = the number available
M_CTL_POLY_ON      = 127;           // POLY MODE ON, 2nd byte is always "0"

M_NDX_SOUND_OFF    = 121;           // ALL SOUND OFF, 2nd byte is always "0"
M_NDX_RESET_ALL    = 122;           // RESET ALL CONTROLLERS, 2nd byte is always "0"
M_NDX_LOCAL_CTL    = 123;           // LOCAL CONTROL: 0 = OFF, 127 = ON
M_NDX_NOTE_OFF     = 124;           // ALL NOTES OFF, 2nd byte is always "0"
M_NDX_OMNI_OFF     = 125;           // OMNI MODE OFF, 2nd byte is always "0"
M_NDX_OMNI_ON      = 126;           // OMNI MODE ON, 2nd byte is always "0"
M_NDX_MONO_ON      = 127;           // MONO MODE ON, 2nd byte is number of channels, or
                                    // 0 the number of channels = the number available
M_NDX_POLY_ON      = 128;           // POLY MODE ON, 2nd byte is always "0"

DEFINE_CONSTANT                     // MIDI byte order

MIDI_DATA_BE = 1;                   // Big-Endian:    MSB first
MIDI_DATA_LE = 0;                   // Little-Endian: LSB first

DEFINE_CONSTANT                     // AMX_AXB_MIDI module IO mask values

// MASK VALUES FOR 'UPDATE AXB-MIDI IO MASK'
MIDI_MASK_OFF    = $00;
MIDI_MASK_THRU   = $01;             // this value means "send data to midi thru"
MIDI_MASK_OUT    = $02;             // this value means "send data to midi out"
MIDI_MASK_MASTER = $04;             // this value means "send data to amx master"

(***********************************************************)
(*              STRUCTURE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_TYPE                         // AMX AXB-MIDI input mask

STRUCT _sAxb_midi_input_mask
{
  integer a;                        // IN-A mask
  integer b;                        // IN-B mask
  integer m;                        // IN-M mask (data from Master to AXB-MIDI)
}

DEFINE_TYPE                         // basic MIDI structures

STRUCT _sMidi_14_bits
{
  char    LSB;                      // lower 7 bits of data
  char    MSB;                      // upper 7 bits of data
  integer VALUE;                    // (msb<<7) | lsb
}

DEFINE_TYPE                         // MIDI Channel Voice structures

STRUCT _sMidi_status_bytes          // storage for channel voice status bytes
{
  char note_off;                    // $80 + MIDI channel number
  char note_on;                     // $90 + MIDI channel number
  char poly_at;                     // $A0 + MIDI channel number
  char ctrl;                        // $B0 + MIDI channel number
  char prog;                        // $C0 + MIDI channel number
  char at;                          // $D0 + MIDI channel number
  char bend;                        // $E0 + MIDI channel number
}

STRUCT _sMidi_note_status           // storage for 1 channel of note on/off status
                                    // If note velocity is 0, note is off
                                    // If note velocity > 0, note is on
{
  LONG    hex[4];                   // 128 bit note on/off, for printing to terminal
                                    // The subroutines in this axi set/get the data
                                    // as little-endian.
  INTEGER boolean[128];             // [<NOTE index>] = on/off

  CHAR    velocity[128];            // [<NOTE index>] = velocity
}

STRUCT _sMidi_voice_data            // CHANNEL VOICE DATA
{
  _sMidi_note_status NOTE;
  CHAR           POLY_AT[128];      // [<NOTE index>] = PRESSURE
  CHAR           CTRL[128];         // [<CTRL index>] = VALUE
  _sMidi_14_bits BANK;              // CTRL[1] is MSB, CTRL[33] is LSB
  CHAR           PROG;              // PROGRAM DATA
  char           AT;                // CHANNEL AFTERTOUCH PRESSURE
  _sMidi_14_bits BEND;              // PITCH WHEEL (BEND) DATA
}

DEFINE_TYPE                         // other compound MIDI structures

STRUCT _sMidi                       // GENERIC MIDI DATA
{                                   // voice data on all 16 MIDI channels
                                    // individual status for controllers
                                    // buffer large enough for most sysx dumps
  _sAxb_midi_input_mask maskInput;

  char    DataBuffer[16000];        // DATA BYTE(S) OF MESSAGE
  integer DataLength;               // LENGTH OF BUFFER

  char    RunningStatus;            // STATUS BYTE OF MESSAGE: RUNNING STATUS
  char    RealtimeStatus;           // STATUS BYTE OF MESSAGE: SYSTEM REAL TIME

  char    MC;                       // Current MIDI CHANNEL $0 to $F
  _sMidi_voice_data   VD[16];       // CHANNEL VOICE DATA[MC+1]

  char              sysxID[5];      // SYSTEM EXCLUSIVE ID:
                                    // [MANUFACTURER ID (1 or 3 BYTES),
                                    // DEVICE ID (USUALLY MIDI CHANNEL NUMBER),
                                    // MODEL ID]
}

(***********************************************************)
(*               VARIABLE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_VARIABLE                     // __display_lib_version__

#IF_NOT_DEFINED __display_lib_version__
VOLATILE __display_lib_version__ = TRUE
#END_IF

DEFINE_VARIABLE                     // Lookup Tables

                                    // MIDI Channel Voice Status bytes
VOLATILE _sMidi_status_bytes midiStatusByteTable[16];// this table populated in DEFINE_START

                                    // 32 bits
VOLATILE LONG __BIT_TABLE_32__[32] = 
{
  $00000001,$00000002,$00000004,$0000008,$00000010,$00000020,$00000040,$00000080,
  $00000100,$00000200,$00000400,$0000800,$00001000,$00002000,$00004000,$00008000,
  $00010000,$00020000,$00040000,$0080000,$00100000,$00200000,$00400000,$00800000,
  $01000000,$02000000,$04000000,$8000000,$10000000,$20000000,$40000000,$80000000
}
                                    // ~32 bits
VOLATILE LONG __UNBIT_TABLE_32__[32] = 
{
  $FFFFFFFe,$FFFFFFFd,$FFFFFFFb,$FFFFFF7,$FFFFFFeF,$FFFFFFdF,$FFFFFFbF,$FFFFFF7F,
  $FFFFFeFF,$FFFFFdFF,$FFFFFbFF,$FFFF7FF,$FFFFefff,$FFFFdFFF,$FFFFbFFF,$FFFF7FFF,
  $FFFeFFFF,$FFFdFFFF,$FFFbFFFF,$FF7FFFF,$FFeFFFFF,$FFdFFFFF,$FFbFFFFF,$FF7FFFFF,
  $FeFFFFFF,$FdFFFFFF,$FbFFFFFF,$7FFFFFF,$eFFFFFFF,$dFFFFFFF,$bFFFFFFF,$7FFFFFFF
}

(***********************************************************)
(*        SUBROUTINE/FUNCTION DEFINITIONS GO BELOW         *)
(***********************************************************)
(* EXAMPLE: DEFINE_FUNCTION <RETURN_TYPE> <NAME> (<PARAMETERS>) *)
(* EXAMPLE: DEFINE_CALL '<NAME>' (<PARAMETERS>) *)

DEFINE_FUNCTION char[8] midiNoteText (integer noteNumber)
// FUNCTION: midiNoteText
// RETURNS: Text designating which note. Based on Middle C = C4 = noteNumber 60
//          This makes the MIDI octave range -1 to 9
{
  LOCAL_VAR SINTEGER snOctave;
  LOCAL_VAR CHAR noteName[2];

  snOctave = TYPE_CAST(noteNumber/12) - 1;

  SWITCH (noteNumber%12)
  {
    CASE 0: noteName = 'C';
    CASE 1: noteName = 'Db';
    CASE 2: noteName = 'D';
    CASE 3: noteName = 'Eb';
    CASE 4: noteName = 'E';
    CASE 5: noteName = 'F';
    CASE 6: noteName = 'F#';
    CASE 7: noteName = 'G';
    CASE 8: noteName = 'Ab';
    CASE 9: noteName = 'A';
    CASE 10: noteName = 'Bb';
    CASE 11: noteName = 'B';
  }

  RETURN "noteName, ITOA(snOctave)";
}

(*------------------------------------------------------------------------------*------------------------*)
DEFINE_FUNCTION midiPrintNoteStatusHex (DEV dvD, CHAR hdr[], long hex[4])
// function: midiPrintNoteStatusHex()
// purpose: print 1 MIDI channel of _sMidi_note_status.hex for debugging
// parameters:
//      dvD	   - destination for function output
//      hdr    - header prepended to result, if desired
//      hex[4] - data to printed, little endian
{
  SEND_STRING dvD,"hdr,'0x ',
				  FORMAT('%08X',hex[4]),' ', // notes 96-127
				  FORMAT('%08X',hex[3]),' ', // notes 64-95
				  FORMAT('%08X',hex[2]),' ', // notes 32-63
				  FORMAT('%08X',hex[1])"     // notes 0-32
}

(*------------------------------------------------------------------------------*------------------------*)
DEFINE_FUNCTION char[2] midiPack_14_Bits (integer nValue, integer order)
// FUNCTION: midiPack_14_Bits
// PURPOSE: Packs integer value to two bytes MIDI data
// PARAMETERS:
//      nValue - integer value
//      Order  - MIDI_DATA_BE = Big-Endian (MSB first),
//               MIDI_DATA_LE = Little-Endian (LSB first) - default for MIDI_BEND data
{
  IF (order = MIDI_DATA_BE)
	RETURN "TYPE_CAST((nValue >> 7) & $7F),TYPE_CAST(nValue & $7F)";
  ELSE
	RETURN "TYPE_CAST(nValue & $7F),TYPE_CAST((nValue >> 7) & $7F)";
}

DEFINE_FUNCTION integer midiUnpack_14_Bits (char data[2], integer order)
// FUNCTION: midiUnpack_14_Bits
// PURPOSE: unpacks 2 bytes midi data to integer
// PARAMETERS:
//      data   - 2 byte data packet
//      Order  - MIDI_DATA_BE = Big-Endian (MSB first),
//               MIDI_DATA_LE = Little-Endian (LSB first) - default for MIDI_BEND data
{
  IF (order == MIDI_DATA_BE) RETURN TYPE_CAST((data[1]<<7) | data[2]);
  ELSE                       RETURN TYPE_CAST((data[2]<<7) | data[1]);
}
(*------------------------------------------------------------------------------*------------------------*)
DEFINE_FUNCTION char midiSEPERATE(integer ndx, char INCOMING[], char DATA[])
// FUNCTION: midiSEPERATE
// - REMOVES DATA FROM INCOMING buffer, that is, INCOMING is parsed destructively
// - SEPERATES MIDI DATA BYTES (IF ANY) FROM MIDI STATUS BYTE
// - BUFFERS STATUS BYTE (MIDI DATA BYTES DON'T CHANGE RUNNING STATUS)
// - Buffers Status Byte for up to 128 devices.
// Parameters:
//      ndx     : index of MIDI stream 1 to 128
//                Each NetLinx device receiving MIDI must use a unique index
//      INCOMING: data buffer from device
//      DATA    : MIDI data seperated from INCOMING
// Returns:
//      MIDI Status Byte associated with DATA

// Note/Warning 2:
//    If DATA buffer passed in is a STACK_VAR, some bytes can be lost if not used immediately.
//    Use a LOCAL_VAR or global for DATA if necessary.

// Note/Warning 1:
//    The End-of-Sysx byte ($F7) is not treated as a Status
//    byte, but as a Data byte.
//    This is to ensure SYSX messages being parsed will
//    contain the start byte AND end byte (if received in the same event).
//    Before this tweak, SYSX end bytes ($F7) were always being
//    parsed seperately from the rest of the SYSX message.
{
  LOCAL_VAR INTEGER INDEX;
  LOCAL_VAR char    MIDI_BYTE[128]; // 128x Running Status buffers
  LOCAL_VAR INTEGER LENGTH_INCOMING;

  if(INCOMING[1] == $F7)            // IF IT'S A LONE "END OF SYSX" BYTE
  {
    DATA = "DATA,GET_BUFFER_CHAR(INCOMING)";
  }
  else if(INCOMING[1] >= $80)       // if INCOMING starts with MIDI status byte
  {
    MIDI_BYTE[ndx] = GET_BUFFER_char(INCOMING);
    LENGTH_INCOMING = LENGTH_ARRAY(INCOMING);
    IF(!LENGTH_INCOMING)
    {
      //This is a one byte command
      DATA = '';
    }
    else if(INCOMING[1] >= $80)
    {
      //This is a one byte command
      DATA = '';
    }
    else
    {
      // Get bytes from this command
      for(INDEX=1; INDEX <= LENGTH_INCOMING; INDEX++)
      {
        IF(INCOMING[INDEX] == $F7)  // IF IT'S "END OF SYSX" BYTE (i WILL BE > 1)
        {
                                    // GET THE DATA AND STATUS BYTE $F7 AND BREAK
          DATA = "GET_BUFFER_STRING(INCOMING,INDEX)";
          BREAK;
        }
                                    // IF YOU FIND A NEW STATUS BYTE (i WILL BE > 1)
        ELSE IF(INCOMING[INDEX] >= $80)
        {
          DATA = "GET_BUFFER_STRING(INCOMING,INDEX-1)";
          BREAK;
        }
                                    // ELSE IF THE END OF THE STRING IS REACHED
        ELSE IF(INDEX == LENGTH_INCOMING)
        {
          DATA = "GET_BUFFER_STRING(INCOMING,INDEX)";
        }
      }
    }
  }
  else                              // if INCOMING starts with MIDI data < $80
  {
    //some number of bytes from old command
    LENGTH_INCOMING = LENGTH_ARRAY(INCOMING);
    for(INDEX=1; INDEX <= LENGTH_INCOMING; INDEX++)
    {
      IF(INCOMING[INDEX] == $F7)    // IF IT'S "END OF SYSX" BYTE (i WILL BE > 1)
      {
                                    // GET THE DATA AND STATUS BYTE $F7 AND BREAK
        DATA = "DATA,GET_BUFFER_STRING(INCOMING,INDEX)";
        BREAK;
      }
                                    // IF YOU FIND A NEW STATUS BYTE (i WILL BE > 1)
      else if(INCOMING[INDEX] >= $80)
      {
                                    // GET THE DATA AND BREAK
        DATA = "DATA,GET_BUFFER_STRING(INCOMING,INDEX-1)";
        BREAK;
      }
                                    // ELSE IF THE END OF THE STRING IS REACHED
      else if(INDEX == LENGTH_INCOMING)
      {
        DATA = "DATA,GET_BUFFER_STRING(INCOMING,INDEX)";
      }
    }
  }
  return MIDI_BYTE[ndx];
}

DEFINE_FUNCTION char midiSEPERATE_single(char INCOMING[], char DATA[])
// NOTE: this is the old version of midiSEPERATE, renamed
// FUNCTION: midiSEPERATE_single
// REMOVES DATA FROM INCOMING, that is, INCOMING is parsed destructively
// SEPERATES MIDI DATA BYTES (IF ANY) FROM MIDI STATUS BYTE
// BUFFERS STATUS BYTE (MIDI DATA BYTES DON'T CHANGE RUNNING STATUS)
// Parameters:
//      INCOMING: data from device
//      DATA    : MIDI data seperated from INCOMING
// Returns:
//      MIDI Status Byte associated with DATA

// #WARN '!!!MUST have seperate instance of midiSeperate() for each device!!!'
/* !!!IMPORTANT - because the Running Status is buffered by a
 * LOCAL_VAR, you MUST instance (i.e. make a seperate copy of)
 * this function for each MIDI data stream you will be parsing!!!
 */

// Note/Warning 2:
//    The DATA buffer passed in should not be a STACK_VAR, as some bytes can be lost.
//    Use a LOCAL_VAR for DATA if the buffer is not global.

// Note/Warning 1:
//    The End-of-Sysx byte ($F7) is not treated as a Status
//    byte, but as a Data byte.
//    This is to ensure SYSX messages being parsed will
//    contain the start byte AND end byte.
//    Before this tweak, SYSX end bytes ($F7) were always being
//    parsed seperately from the rest of the SYSX message.
{
  LOCAL_VAR INTEGER INDEX;
  LOCAL_VAR char    MIDI_BYTE;      // Running Status buffer
  LOCAL_VAR INTEGER LENGTH_INCOMING;

  if(INCOMING[1] == $F7)            // IF IT'S A LONE "END OF SYSX" BYTE
  {
    DATA = "DATA,GET_BUFFER_CHAR(INCOMING)";
  }
  else if(INCOMING[1] >= $80)
  {
    MIDI_BYTE = GET_BUFFER_char(INCOMING);
    LENGTH_INCOMING = LENGTH_ARRAY(INCOMING);
    IF(!LENGTH_INCOMING)
    {
      //This is a one byte command
      DATA = '';
    }
    else if(INCOMING[1] >= $80)
    {
      //This is a one byte command
      DATA = '';
    }
    else
    {
      // Get bytes from this command
      for(INDEX=1; INDEX <= LENGTH_INCOMING; INDEX++)
      {
        IF(INCOMING[INDEX] == $F7)  // IF IT'S "END OF SYSX" BYTE (i WILL BE > 1)
        {
                                    // GET THE DATA AND STATUS BYTE $F7 AND BREAK
          DATA = GET_BUFFER_STRING(INCOMING,INDEX);
          BREAK;
        }
                                    // IF YOU FIND A NEW STATUS BYTE (i WILL BE > 1)
        ELSE IF(INCOMING[INDEX] >= $80)
        {
          DATA = GET_BUFFER_STRING(INCOMING,INDEX-1);
          BREAK;
        }
                                    // ELSE IF THE END OF THE STRING IS REACHED
        ELSE IF(INDEX == LENGTH_INCOMING)
        {
          DATA = GET_BUFFER_STRING(INCOMING,INDEX);
        }
      }
    }
  }
  else
  {
    //some number of bytes from old command
    LENGTH_INCOMING = LENGTH_ARRAY(INCOMING);
    for(INDEX=1; INDEX <= LENGTH_INCOMING; INDEX++)
    {
      IF(INCOMING[INDEX] == $F7)    // IF IT'S "END OF SYSX" BYTE (i WILL BE > 1)
      {
                                    // GET THE DATA AND STATUS BYTE $F7 AND BREAK
        DATA = "DATA,GET_BUFFER_STRING(INCOMING,INDEX)";
        BREAK;
      }
                                    // IF YOU FIND A NEW STATUS BYTE (i WILL BE > 1)
      else if(INCOMING[INDEX] >= $80)
      {
                                    // GET THE DATA AND BREAK
        DATA = "DATA,GET_BUFFER_STRING(INCOMING,INDEX-1)";
        BREAK;
      }
                                    // ELSE IF THE END OF THE STRING IS REACHED
      else if(INDEX == LENGTH_INCOMING)
      {
        DATA = "DATA,GET_BUFFER_STRING(INCOMING,INDEX)";
      }
    }
  }
  return MIDI_BYTE;
}

(*------------------------------------------------------------------------------*------------------------*)
DEFINE_FUNCTION midiSetIoMask (DEV dvMIDI, char cINPUT, INTEGER nMASK_NEW, INTEGER nMASK_CUR)
// FUNCTION midiSetIoMask()
// BITMAPS AXB-MIDI INPUT TO OUTPUT BASED ON nMASK_NEW
// STORES NEW VALUE IN nMASK_CUR
// PARAMETERS:
// dvMIDI   - DEVICE to receive command
// cINPUT     - A, B, or M
// nMASK_NEW  - NEW VALUE FOR MASK
//              1 = THRU, 2 = OUT, 4 = MASTER
// nMASK_CUR  - STORAGE VARIABLE FOR MASK
{
  SEND_COMMAND dvMIDI,FORMAT("'IN',cINPUT,'-%d'",nMASK_NEW);
  nMASK_CUR = nMASK_NEW;
}

(*------------------------------------------------------------------------------*------------------------*)
DEFINE_FUNCTION integer midiSetNoteStatus (CHAR data[2], _sMidi_note_status note)
// function: midiSetNoteStatus()
// purpose: set note on/off status in _sMidiNoteStatus struct
// parameters:
//      data   - 2 byte data packet "<note#>,<velocity>"
//               if velocity is 0, status is 0/FALSE
//               if velocity is > 0, status is 1/TRUE
//      status - destination struct for 1 MIDI channel
//             - status.hex data is little-endian:
//               [1]         ,[2]          ,[3]          ,[4]
//               <notes 0-31>,<notes 32-63>,<notes 64-95>,<notes 96-127>
// return:     - note status FALSE/TRUE/error (not enough data)
{
  IF (LENGTH_ARRAY(data) > 1)       // check for data bytes 1 and 2
  {
	LOCAL_VAR LONG offset;  // status.hex index 1 - 4
	LOCAL_VAR LONG bit;     // 1 - 32
	
	bit = (data[1]%32) + 1
	offset = (data[1]/32) + 1;
	
	IF (!data[2])
	{
	  note.hex[offset] = note.hex[offset] & __unbit_table_32__[bit];
	  note.boolean[data[1] + 1] = FALSE;
	  note.velocity[data[1] + 1] = 0;
	  RETURN FALSE;
	}
	ELSE
	{
	  note.hex[offset] = note.hex[offset] | __bit_table_32__[bit];
	  note.boolean[data[1] + 1] = TRUE;
	  note.velocity[data[1] + 1] = data[2];
	  RETURN TRUE;
	}
  }
  ELSE
	RETURN $FE;                     // return $FE to signify error
}

(***********************************************************)
(*                 STARTUP CODE GOES BELOW                 *)
(***********************************************************)
DEFINE_START                        // populate midiStatusByteTable[<MIDI ch>].<status bytes>
{
  STACK_VAR INTEGER i;
  FOR (i = 16; i; i--)
  {
    midiStatusByteTable[i].note_off = TYPE_CAST($7F + i);
    midiStatusByteTable[i].note_on  = TYPE_CAST($8F + i);
    midiStatusByteTable[i].poly_at  = TYPE_CAST($9F + i);
    midiStatusByteTable[i].ctrl     = TYPE_CAST($AF + i);
    midiStatusByteTable[i].prog     = TYPE_CAST($BF + i);
    midiStatusByteTable[i].at       = TYPE_CAST($CF + i);
    midiStatusByteTable[i].bend     = TYPE_CAST($DF + i);
  }
  SET_LENGTH_ARRAY(midiStatusByteTable,MAX_LENGTH_ARRAY(midiStatusByteTable));
}

(***********************************************************)
(*                THE EVENTS GOES BELOW                    *)
(***********************************************************)
DEFINE_EVENT

DATA_EVENT[0:1:0]
{
  ONLINE:
  {
    wait 50                         // WAIT FOR DEVICE HOLDOFF
    {
	  IF (__display_lib_version__ == TRUE)
	  {
		// WHAT VERSION?
		SEND_STRING __dvMidiDebug__,"'Using lib ',__MIDI_API_NAME__,
											', v',__MIDI_API_VERSION__";
	  }
    }
  }
}

#END_IF __MIDI_API__

DEFINE_PROGRAM                      // small program example (commented out)
//PROGRAM_NAME='MIDI small program example'
//(***********************************************************)
//(*  FILE CREATED ON: 08/14/2006  AT: 04:36:08              *)
//(***********************************************************)
//(*  FILE_LAST_MODIFIED_ON: 06/21/2010  AT: 08:23:35        *)
//(***********************************************************)
//(* The following code block is provided as a guide to      *)
//(* programming the device(s) listed above. This is a       *)
//(* sample only, and will most likely require modification  *)
//(* to be integrated into a master program.                 *)
//(*                                                         *)
//(* Device-specific protocols should be obtained directly   *)
//(* from the equipment manufacturer to ensure compliance    *)
//(* with the most current versions. Within the limits       *)
//(* imposed by any non-disclosure agreements, AMX will      *)
//(* provide protocols upon request.                         *)
//(*                                                         *)
//(* If further programming assistance is required, please   *)
//(* contact your AMX customer support team.                 *)
//(***********************************************************)
///**************************/
///* small program example: */
///**************************/
//#INCLUDE 'DEV_TO_STRING.axi'
//#INCLUDE 'debug.axi'
//#INCLUDE 'MIDI_API.axi'
//
//(***********************************************************)
//(*          DEVICE NUMBER DEFINITIONS GO BELOW             *)
//(***********************************************************)
//DEFINE_DEVICE
//
//dvMIDI_1 = 96:1:0
////vdvMIDI_1 = 33096:1:0
//
//(***********************************************************)
//(*               CONSTANT DEFINITIONS GO BELOW             *)
//(***********************************************************)
//DEFINE_CONSTANT
//
//(***********************************************************)
//(*              STRUCTURE DEFINITIONS GO BELOW             *)
//(***********************************************************)
//DEFINE_TYPE
//
//(***********************************************************)
//(*               VARIABLE DEFINITIONS GO BELOW             *)
//(***********************************************************)
//DEFINE_VARIABLE
//
//_sMidi midi1                        // storage for midi data
//
//INTEGER MIDI1_DEBUG_RX = 1
//
//(***********************************************************)
//(*           SUBROUTINE DEFINITIONS GO BELOW               *)
//(***********************************************************)
//(* EXAMPLE: DEFINE_FUNCTION <RETURN_TYPE> <NAME> (<PARAMETERS>) *)
//(* EXAMPLE: DEFINE_CALL '<NAME>' (<PARAMETERS>) *)
//
//(***********************************************************)
//(*                STARTUP CODE GOES BELOW                  *)
//(***********************************************************)
//DEFINE_START
//
//(***********************************************************)
//(*               MODULE DEFINITIONS GO BELOW               *)
//(***********************************************************)
////DEFINE_MODULE 'AMX_AXB_MIDI' mdlMIDI_1(vdvMIDI_1,dvMIDI_1)
//
//(***********************************************************)
//(*                 THE EVENTS GO BELOW                     *)
//(***********************************************************)
//DEFINE_EVENT
//
//DATA_EVENT[dvMIDI_1]
////DATA_EVENT[vdvMIDI_1]
//{
//  ONLINE:
//  {
//									/**************************/
//                                    /* AXB-MIDI IO mask setup */
//									/**************************/
//    #warn 'Initialize AXB-MIDI I/O masks here'
//									// INA TO MASTER, OUT, and THRU
//	midiSetIoMask (DATA.DEVICE,'A',MIDI_MASK_MASTER|MIDI_MASK_OUT|MIDI_MASK_THRU,midi1.maskInput.A)
//									// INB TO MASTER only
//	midiSetIoMask (DATA.DEVICE,'B',MIDI_MASK_MASTER,midi1.maskInput.B)
//									// MASTER TO THRU AND OUT
//	midiSetIoMask (DATA.DEVICE,'M',MIDI_MASK_THRU|MIDI_MASK_OUT,midi1.maskInput.M)
//
////									/*****************************/
////									/* AMX_AXB_MIDI module setup */
////									/*****************************/
////	SEND_COMMAND vdvMIDI_1,'HEADER-MIDI_1 '
////									// Use PROGRAM (preset) messages on MIDI CH 1 & 6
////	SEND_COMMAND vdvMIDI_1,"'PROG-1,6'"
////									// Use CONTROLLER messages on MIDI CH 1 & 6 
////	SEND_COMMAND vdvMIDI_1,"'CTRL-1,6'"
////// If need to parse all the channels of data coming in,
////// you'll be better off NOT using the module
////									// Use PROGRAM (preset) messages on MIDI CH 1-16
//////	SEND_COMMAND vdvMIDI_1,"'PROG-1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16'"
//////									// Use CONTROLLER messages on MIDI CH 1-16
//////	SEND_COMMAND vdvMIDI_1,"'CTRL-1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16'"
//////									// Use NOTE messages on MIDI CH 1-16
//////	SEND_COMMAND vdvMIDI_1,"'NOTE-1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16'"
//////									// Use PITCH BEND messages on MIDI CH 1-16
//////	SEND_COMMAND vdvMIDI_1,"'BEND-1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16'"
//////									// Use AFTERTOUCH messages on MIDI CH 1-16
//////	SEND_COMMAND vdvMIDI_1,"'AFTT-1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16'"
////	SEND_COMMAND vdvMIDI_1,"'SYSX-1'" // Use SYSTEM EXCLUSIVE MESSAGES
//// 	SEND_COMMAND vdvMIDI_1,"'CMMN-1'" // Use other SYSTEM COMMON messages
////	SEND_COMMAND vdvMIDI_1,"'ACSN-1'" // Use Active Sense
////	SEND_COMMAND vdvMIDI_1,"'REAL-1'" // Use other SYSTEM REALTIME messages
//
//									/********************/
//									/* MIDI status init */
//									/********************/
//	OFF[midi1.RunningStatus]        // clear MIDI Running Status and Buffers
//	midi1.DataBuffer = ""
//	midi1.DataLength = 0
//  }
//  STRING:
//  {
//	IF (MIDI1_DEBUG_RX > 2)
//	  CALL 'SEND ITOHEX TO DEBUG' (0:1:0,
//								  "DEV_TO_STRING(DATA.DEVICE),' str: '",
//								  "DATA.TEXT",
//								   0,0)
//
//	WHILE (LENGTH_ARRAY(DATA.TEXT))
//	{
//									// SYSTEM REAL TIME messages
//									// should not affect Running Status per MIDI spec.
//	  IF (DATA.TEXT[1] >= $F8)      // IF IT IS A REAL TIME MESSAGE
//	  {
//		SWITCH (GET_BUFFER_CHAR(DATA.TEXT))
//		{
//		  CASE $F8:                 // AXB-MIDI won't be able to keep up with this...
//		  {
//			IF (MIDI1_DEBUG_RX) SEND_STRING 0,"'TIMING CLOCK:$F8'"
//		  }
//
//		  CASE $F9:
//		  {
//			IF (MIDI1_DEBUG_RX) SEND_STRING 0,"'UNDEFINED:$F9'"
//		  }
//
//		  CASE $FA:
//		  {
//			IF (MIDI1_DEBUG_RX) SEND_STRING 0,"'START:$FA'"
//		  }
//
//		  CASE $FB:
//		  {
//			IF (MIDI1_DEBUG_RX) SEND_STRING 0,"'CONTINUE:$FB'"
//		  }
//
//		  CASE $FC:
//		  {
//			IF (MIDI1_DEBUG_RX) SEND_STRING 0,"'STOP:$FC'"
//		  }
//
//		  CASE $FD:
//		  {
//			IF (MIDI1_DEBUG_RX) SEND_STRING 0,"'UNDEFINED:$FD'"
//		  }
//
//		  CASE $FE:                 // MIDI controller heartbeat
//		  {
//			IF (MIDI1_DEBUG_RX > 1) SEND_STRING 0,"'ACTIVE SENSE:$FE'"
//		  }
//
//		  CASE $FF:
//		  {
//			IF (MIDI1_DEBUG_RX) SEND_STRING 0,"'RESET:$FF'"
//		  }
//		}
//	  }
//	  ELSE                          // SYSTEM COMMON and CHANNEL VOICE MESSAGEs
//	  {
//		midi1.RunningStatus = midiSEPERATE(1,DATA.TEXT,midi1.DataBuffer)
//		midi1.DataLength = LENGTH_ARRAY(midi1.DataBuffer)
//
//		SWITCH (midi1.RunningStatus & $F0)
//		{
//		  CASE $80:                 // Note Off: 2-byte data
//		  {
//			LOCAL_VAR INTEGER i
//
//			midi1.MC = midi1.RunningStatus & $0F
//
//			#warn 'add MIDI Note Off parsing in this block'
//			// for i = note# byte; buffer contains a velocity byte;
//			//         set i to next note# byte
//			FOR (i = 1; i < midi1.DataLength; i = i + 2)
//			{
//									// store note status off
//			  midiSetNoteStatus("midi1.DataBuffer[i],0",midi1.VD[midi1.mc+1].Note)
//			  // Add more code to NOTE OFF section as necessary...
//			}
//
//			IF (MIDI1_DEBUG_RX)
//			{
//			                        // print note status on/off
//			  midiPrintNoteStatusHex(0:1:0,
//									"'Note Status CH ',ITOA(midi1.mc+1),' '",
//									midi1.VD[midi1.mc+1].Note.hex)
//			}
//			// Add more code to NOTE OFF section as necessary...
//          }
//		  CASE $90:                 // Note On: 2-byte data
//		  {
//			LOCAL_VAR INTEGER i
//
//			midi1.MC = midi1.RunningStatus & $0F
//
//			#warn 'add MIDI Note On parsing in this block'
//			// NOTE on with velocity 0 = NOTE off
//
//			// for i = note# byte; buffer contains a velocity byte;
//			//         set i to next note# byte
//			FOR (i = 1; i < midi1.DataLength; i = i + 2)
//			{
//								  // store note status on/off
//			  midiSetNoteStatus(MID_STRING(midi1.DataBuffer,i,2),
//								midi1.VD[midi1.mc+1].Note)
//			  // Add more code to NOTE ON section as necessary...
// 			}
//
//			IF (MIDI1_DEBUG_RX)
//			{
//									// print note status on/off
//			  midiPrintNoteStatusHex(0:1:0,
//									"'Note Status CH ',ITOA(midi1.MC+1),' '",
//									midi1.VD[midi1.MC+1].Note.hex)
//			}
//			// Add more code to NOTE ON section as necessary...
//		  }
//		  CASE $A0:                 // Poly AfterTouch: 2-byte data
//		  {
//			midi1.MC = midi1.RunningStatus & $0F
//
//			CALL 'SEND MIDI TO DEBUG' (0:1:0,
//									  "'POLY AFTERTOUCH CH ',ITOA(midi1.MC+1),':'",
//									  "midi1.DataBuffer",
//									  0,0,midi1.RunningStatus)
//            // Add more code to POLY AT section as necessary...
//		  }
//		  CASE $B0:                 // Controller: 2-byte data
//		  {
//			LOCAL_VAR INTEGER i
//			LOCAL_VAR INTEGER ctrl_ndx
//
//			midi1.MC = midi1.RunningStatus & $0F
//
//			#warn 'add MIDI Controller parsing in this block'
//			// for i = controller byte; buffer contains a value byte;
//			//         set i to next controller byte
//			FOR (i = 1; i < midi1.DataLength; i = i + 2)
//			{
//								  // store controller data for later use
//			  ctrl_ndx = midi1.Databuffer[i]+1
//			  midi1.VD[midi1.MC+1].CTRL[ctrl_ndx] = midi1.Databuffer[i+1]
//			  
//			  IF (MIDI1_DEBUG_RX)
//			  {
//				CALL 'SEND MIDI TO DEBUG' (0:1:0,
//										"'CONTROLLER CH ',ITOA(midi1.MC+1),':'",
//										"ctrl_ndx - 1,midi1.VD[midi1.MC+1].CTRL[ctrl_ndx]",
//										0,0,midi1.RunningStatus)
//			  }
//			  // Add more code to CTRL section as necessary...
//			}
//		  }
//		  CASE $C0:                 // Program Change: 1-byte data
//		  {
//			LOCAL_VAR INTEGER ndxMC // midi1.MC + 1 is much used in this section, 
//			                        // so better to store it in a LOCAL_VAR
//		  
//			midi1.MC = midi1.RunningStatus & $0F
//			ndxMC = midi1.MC + 1
//
//			IF (midi1.DataLength)   // last program change received is the current program change
//			{
//			  midi1.VD[ndxMC].PROG = midi1.DataBuffer[midi1.DataLength]
//
//			  #warn 'add MIDI Program (preset) parsing in this block'
//			  // If you're tracking MIDI program banks:
//			  // Probably better to do this here, not under very busy controller parsing.
//			  midi1.VD[ndxMC].BANK.LSB = midi1.VD[ndxMC].CTRL[M_NDX_BANK_LSB]
//			  midi1.VD[ndxMC].BANK.MSB = midi1.VD[ndxMC].CTRL[M_NDX_BANK_MSB]
//			  midi1.VD[ndxMC].BANK.VALUE =
//				midiUnpack_14_Bits("midi1.VD[ndxMC].BANK.LSB,midi1.VD[ndxMC].BANK.MSB",MIDI_DATA_LE)
//
//			  IF (MIDI1_DEBUG_RX)
//			  {
//									// Program # displayed is 1-128, not 0-127
//				SEND_STRING 0,"'Midi Channel ',ITOA(ndxMC),
//								' Bank ',ITOA(midi1.VD[ndxMC].BANK.VALUE),
//								' Program ',ITOA(midi1.VD[ndxMC].PROG+1)"
//			  }
//			  // Add more code to PROG section as necessary...
//			}
//		  }
//		  CASE $D0:                 // Channel Aftertouch: 1-byte data
//		  {
//			midi1.MC = midi1.RunningStatus & $0F
//
//			IF (MIDI1_DEBUG_RX)
//			{
//			  CALL 'SEND MIDI TO DEBUG' (0:1:0,
//									  "'AFTERTOUCH CH ',ITOA(midi1.MC+1),':'",
//									  "midi1.DataBuffer",
//									  0,0,midi1.RunningStatus)
//			}
//			// Add more code to AT section as necessary...
//		  }
//		  CASE $E0:                 // Pitch Wheel: 2-byte data
//		  {
//			LOCAL_VAR INTEGER i
//			LOCAL_VAR INTEGER bendValue
//
//			midi1.MC = midi1.RunningStatus & $0F
//
//			IF (MIDI1_DEBUG_RX)
//			{
//			  CALL 'SEND MIDI TO DEBUG' (0:1:0,
//										"'PITCH BEND CH ',ITOA(midi1.MC+1),':'",
//										"midi1.DataBuffer",
//										0,0,midi1.RunningStatus)
//			}
//
//			// for i = bend LSB; buffer contains a bend MSB;
//			//         set i to next bend LSB
//			FOR (i = 1; i < midi1.DataLength; i = i + 2)
//			{
//								  // get and display pitch bend value
//			  bendValue = midiUnpack_14_Bits(MID_STRING(midi1.DataBuffer,i,2),MIDI_DATA_LE)
//			  SEND_STRING 0,"'BEND VALUE:$',ITOHEX(bendValue)"
//			  // Add more code to BEND section as necessary...
//			}
//								  // optional: store bend value for later use...
//			midi1.VD[midi1.MC+1].BEND.VALUE = bendValue
//			// Add more code to BEND section as necessary...
//		  }
//		  CASE $F0:
//		  {
//			SWITCH (midi1.RunningStatus & $0F)
//			{
//			  CASE $0:
//			  {
//				IF (MIDI1_DEBUG_RX)
//				{
//				  CALL 'SEND ITOHEX TO DEBUG' (0:1:0,
//											   "'SYSTEM EXCLUSIVE:'",
//											   "midi1.DataBuffer",
//											   0,0)
//				}
//				#warn 'add MIDI Sysex parsing in this block'
//			    // Add more code to SYSX section as necessary...for example:
//						            // if End-of-SYSX byte is found...
//				IF (FIND_STRING(midi1.DataBuffer,"$F7",1)) 
//				{
//				  // parse completed message here...or EOM if it's too long.
//				}
//				ELSE
//				{
//				  // if desired, parse partial message here...
//				}
//			  }
//			  CASE $1:
//			  CASE $4:
//			  CASE $5:
//			  {
//				IF (MIDI1_DEBUG_RX)
//				{
//				  CALL 'SEND MIDI TO DEBUG' (0:1:0,
//											"'UNDEFINED:'",
//											"midi1.DataBuffer",
//											0,0,midi1.RunningStatus)
//				}
//			  }
//			  CASE $2:              // AXB-MIDI probably won't be able to keep up with this, either
//			  {
//				IF (MIDI1_DEBUG_RX)
//				{
//				  CALL 'SEND MIDI TO DEBUG' (0:1:0,
//											"'SONG POSITION POINTER:'",
//											"midi1.DataBuffer",
//											0,0,midi1.RunningStatus)
//				}
//			  }
//			  CASE $3:
//			  {
//			  	IF (MIDI1_DEBUG_RX)
//				{
//				  CALL 'SEND MIDI TO DEBUG' (0:1:0,
//											"'SONG SELECT:'",
//											"midi1.DataBuffer",
//											0,0,midi1.RunningStatus)
//				}
//			  }
//			  CASE $6:
//			  {
//				IF (MIDI1_DEBUG_RX)
//				{
//				  CALL 'SEND MIDI TO DEBUG' (0:1:0,
//											"'TUNE REQUEST:'",
//											"midi1.DataBuffer",
//											0,0,midi1.RunningStatus)
//				}
//			  }
//			  CASE $7: // this will not happen when using midiSeperate()
//			  {
//				IF (MIDI1_DEBUG_RX)
//				{
//				  CALL 'SEND MIDI TO DEBUG' (0:1:0,
//											"'END-OF-SYSEX:'",
//											"midi1.DataBuffer",
//											0,0,midi1.RunningStatus)
//				}
//			  }
//			}
//		  }
//		}
//	  }
//	}
//  }
//}
//
//(***********************************************************)
//(*            THE ACTUAL PROGRAM GOES BELOW                *)
//(***********************************************************)
//
//
//(***********************************************************)
//(*                     END OF PROGRAM                      *)
//(*        DO NOT PUT ANY CODE BELOW THIS COMMENT           *)
//(***********************************************************)
(***********************************************************)
(*                     END OF PROGRAM                      *)
(*        DO NOT PUT ANY CODE BELOW THIS COMMENT           *)
(***********************************************************)
