PROGRAM_NAME='debug'
(***********************************************************)
(*  FILE CREATED ON: 12/28/2001 AT: 00:00:00               *)
(***********************************************************)
(*  FILE_LAST_MODIFIED_ON: 06/09/2010  AT: 18:40:08        *)
(***********************************************************)
(*  FILE REVISION: 2.60                                    *)
(*  REVISION DATE: 06/09/2010  AT: 18:40:08                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Tweaked RETURN code for new functions.               *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.59                                    *)
(*  REVISION DATE: 05/16/2010  AT: 10:37:53                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*   New functions: formatAscii(), formatAsciiHex(),       *)
(*   formatDecimal(), formatHex(), formatMIDI().           *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.58                                    *)
(*  REVISION DATE: 05/13/2010  AT: 12:34:39                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Removed 131 character line limit in subroutines with *)
(*    an "nLine" parameter.                                *)
(*    If  nLine > 131, longer messages may be truncated,   *)
(*    e.g. when dvDebug is the master (send_string 0).     *)
(*  - Updated ethernet connection types returned by        *)
(*    printDEVICE_INFO() and                               *)
(*   'SEND DEVICE_INFO TO DEBUG'().                        *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.57                                    *)
(*  REVISION DATE: 11/08/2008  AT: 13:17:15                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Speed Tweak:                                         *)
(*    * Change FORMAT('$%0 2X',STR2[X]) to                 *)
(*             FORMAT('$%02X',STR2[X])                     *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.56                                    *)
(*  REVISION DATE: 10/10/2008  AT: 18:52:56                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - added not-in-terminal parameter to                   *)
(*    printCustom_Event().                                 *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.55                                    *)
(*  REVISION DATE: 10/08/2008  AT: 09:04:35                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Added functions printCUSTOM_EVENT() and              *)
(*    printDEVICE_INFO().                                  *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.54                                    *)
(*  REVISION DATE: 10/06/2008  AT: 08:01:21                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Added __display_lib_version__ variable.              *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.53                                    *)
(*  REVISION DATE: 09/08/2008  AT: 17:24:28                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Moved DEV_TO_STRING and its variations to            *)
(*    DEV_TO_STRING.axi.                                   *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.52                                    *)
(*  REVISION DATE: 08/18/2008  AT: 05:49:49                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - Added call 'SEND MIDI TO DEBUG'.                     *)
(*  - Made __SPACES__ a volatile variable instead of a     *)
(*    constant.                                            *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.51, 2.50                              *)
(*  REVISION DATE: 08/13/2008  AT: 14:28:35                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - 'SEND...' calls were rewritten to handle very large  *)
(*    string arrays and string expressions that can output *)
(*    more than 15999 bytes of processed data.             *)
(*  - Large string parameters of 'SEND...' calls were set  *)
(*    to 15999 to workaround a feature that truncates      *)
(*    string expressions to 2048 bytes if passed in via an *)
(*    unbounded array.                                     *)
(*    With the parameter size set to 15999 they are not    *)
(*    truncated below this "normal" NetLinx interpreter    *)
(*    limit.                                               *)
(*    String arrays, for example the output of FILE_READ() *)
(*    or GET_BUFFER_STRING(), will still pass through      *)
(*    unaffected at 16000 bytes or greater.                *)
(*  - 'SEND ASCII TO MASTER' and 'SEND ITOHEX TO MASTER'   *)
(*    are now just wrappers that call                      *)
(*   'SEND ASCII TO DEBUG' and 'SEND ITOHEX TO DEBUG'      *)
(*    using the default settings.                          *)
(*  - CALL 'SEND 131 BYTE PACKETS TO MASTER' no longer     *)
(*    parses destructively, that is, it does not modify    *)
(*    the array passed in.                                 *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.14                                    *)
(*  REVISION DATE: 04/27/2008  AT: 11:14:35                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - renamed 'SEND 7BIT ASCII TO DEBUG' to                *)
(*    'SEND ASCII/HEX TO DEBUG'.                           *)
(*    For backwards compatibility, the old name is now a   *)
(*    wrapper that calls the new name.                     *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: 2.13                                    *)
(*  REVISION DATE: 11/10/2007  AT: 17:18:44                *)
(*                                                         *)
(*  COMMENTS:                                              *)
(*  - tweaked 'SEND CUSTOM_EVENT DATA TO DEBUG' messages   *)
(*    to be more organized and more consistent with        *)
(*    'SEND DEVICE_INFO TO DEBUG'.                         *)
(*                                                         *)
(***********************************************************)
(*  FILE REVISION: Rev 2.12                                *)
(*  REVISION DATE: 10/18/2007  AT: 22:00:11                *)
(*      - Fixed issue with '...ASCII...' calls where a     *)
(*        string  starting with single printable character *)
(*        would not print with a comma following that      *)
(*        character.                                       *)
(***********************************************************)
(*  FILE REVISION: 2.11                                    *)
(*  REVISION DATE: 09/26/2007  AT: 13:24:34                *)
(*      - Increased max size of string returned by         *)
(*        DEV_ARRAY_TO_STRING.                             *)
(***********************************************************)
(*  FILE REVISION: 2.10                                    *)
(*  REVISION DATE: 09/24/2007  AT: 14:59:42                *)
(*      - STR1, passed into 'SEND TO...' subroutines, is   *)
(*        no longer overwritten with spaces if the debug   *)
(*        message takes more than 1 line.                  *)
(*      - Changed SOURCE_TYPE_NO_ADDRESS message from      *)
(*        '...not available' to '...no address'.           *)
(*      - Code cleanup.                                    *)
(***********************************************************)
(*  FILE REVISION: Rev 2.09                                *)
(*  REVISION DATE: 01/09/2007  AT: 10:05:25                *)
(*      - Fixed incorrect use of 0:1:0 in                  *)
(*       'SEND CUSTOM_EVENT DATA TO DEBUG'.                *)
(***********************************************************)
(*  FILE REVISION: Rev 2.08                                *)
(*  REVISION DATE: 10/13/2006  AT: 17:21:44                *)
(*      - Use SOURCE_TYPE_INTERNAL ($14) instead of $20 in *)
(*        the call 'SEND DEVICE_INFO TO DEBUG'. The value  *)
(*        was corrected in NetLinx.axi v1.25.              *)
(***********************************************************)
(*  FILE REVISION: Rev 2.07                                *)
(*  REVISION DATE: 09/26/2006  AT: 18:22:52                *)
(*      - added subroutines for debugging CUSTOM_EVENT's.  *)
(***********************************************************)
(*  FILE REVISION: 2.06                                    *)
(*  REVISION DATE: 09/21/2006  AT: 13:20:35                *)
(*      - added DEV_ARRAY_TO_STRING().                     *)
(***********************************************************)
(*  FILE REVISION: 2.05                                    *)
(*  REVISION DATE: 08/10/2006  AT: 14:59:37                *)
(*      - remove comma, add 'L' prefix to level number     *)
(*        returned by DEVLEV_TO_STRING and                 *)
(*        DEV_LEV_TO_STRING. In addition, the level number *)
(*        is padded with 0's so it's always 3 digits.      *)
(***********************************************************)
(*  FILE REVISION: Rev 2.04 GLM                            *)
(*  REVISION DATE: 06/30/2006  AT: 16:06:33                *)
(*      - 'SEND DEVICE_INFO TO DEBUG' now trims garbage    *)
(*        from SN's less than 16 characters long.          *)
(***********************************************************)
(*  FILE REVISION: Rev 2.03 GLM                            *)
(*  REVISION DATE: 07/08/2005  AT: 15:12:54                *)
(*      - All "DEV..." functions now return the actual     *)
(*        system number, never 0...                        *)
(***********************************************************)
(*!!FILE REVISION: 2.02 GLM                                *)
(*  REVISION DATE: 05/31/2005                              *)
(*      - In the 'SEND...' calls, replace STR1 with an     *)
(*        equal number of spaces if a single reply takes   *)
(*        more than one line, after the first line.        *)
(*        So it's easier to distiguish between multiple    *)
(*        responses. Starting in v2.10 a copy of STR1 is   *)
(*        used so the passed-in STR1 is not changed.       *)
(***********************************************************)
(*!!FILE REVISION: 2.01 GLM                                *)
(*  REVISION DATE: 02/01/2005                              *)
(*      - added DPS_TO_STRING, similar to DEV_TO_STRING    *)
(*        but always returns actual system number.         *)
(*      - added 'STRING TO DEV' call. A call because       *)
(*        functions cannot return complex data types such  *)
(*        as DEVs.                                         *)
(*      - fixed problem where sometimes an extra ' would   *)
(*        sneak into 'SEND 7BIT ASCII TO DEBUG' responses. *)
(*      - 'SEND ASCII TO DEBUG' and 'SEND ASCII TO MASTER' *)
(*        now format DEL as $7F.                           *)
(*      - changed the 'SEND...' calls so they no longer    *)
(*        add a space between the header and data.         *)
(*      - added 'SEND DEVICE_INFO TO DEBUG'.               *)
(*      - rearranged calls alphabetically.                 *)
(***********************************************************)
(*!!FILE REVISION: 2.00 GLM                                *)
(*  REVISION DATE: 11/22/2004                              *)
(*      - use STACK_VAR instead of LOCAL_VAR.              *)
(*      - no longer has virtual device for control, it was *)
(*        never used in practice.                          *)
(***********************************************************)
(*!!FILE REVISION: 1.07 GLM                                *)
(*  REVISION DATE: 04/22/2004                              *)
(*      - added DEV_CHAN_TO_STRING, DEV_LEV_TO_STRING      *)
(*        these functions take a seperate dev,chan or      *)
(*        dev,lev as parameters.                           *)
(***********************************************************)
(*!!FILE REVISION: 1.06 GLM                                *)
(*  REVISION DATE: 09/15/2003                              *)
(*      - modified 'SEND INTEGER TO DEBUG' to only format  *)
(*        line length. It had been a misnamed version of   *)
(*        'SEND DECIMAL TO DEBUG'.                         *)
(***********************************************************)
(*!!FILE REVISION: 1.05 GLM                                *)
(*  REVISION DATE: 05/29/2003                              *)
(*      - added 'SEND 7BIT ASCII TO DEBUG'                 *)
(*        characters  >= $7F are formated as hex           *)
(*      - remove ; from #DEFINE __DEBUG_LIB__              *)
(*      - added explicit parameter types where missing     *)
(*      - added 'SEND DECIMAL TO DEBUG'. This is actually  *)
(*        what 'SEND INTEGER TO DEBUG' should have been    *)
(*        named.                                           *)
(***********************************************************)
(*!!FILE REVISION: 1.04 GLM                                *)
(*  REVISION DATE: 01/21/2003                              *)
(*      - added 'SEND INTEGER TO DEBUG'                    *)
(***********************************************************)
(*!!FILE REVISION: 1.03 GLM                                *)
(*  REVISION DATE: 08/28/2002                              *)
(*      Code cleanup:                                      *)
(*      - added line ends ";" for Studio debug mode.       *)
(*      - changed remaining LENGTH_STRING's to             *)
(*        LENGTH_ARRAY's.                                  *)
(*      - explicitly declare all variable types.           *)
(***********************************************************)
(*!!FILE REVISION: 1.02 GLM                                *)
(*  REVISION DATE: 04/27/2002                              *)
(*      - Added 'SEND ASCII TO DEBUG' and 'SEND ITOHEX TO  *)
(*        DEBUG'.                                          *)
(*        These calls have:                                *)
(*        - Device parameter to allow sending info to      *)
(*        devices other than 0:0:0, for example to virtual *)
(*        device for multi master debugging.               *)
(*        - Line length parameter to allow different line  *)
(*        lengths. If this is 0, line length defaults to   *)
(*        80 for mode  2, 67 for all other modes           *)
(*        - Mode parameter, if set to 2 appends <cr>,<lf>  *)
(*        to each line for master 'MSG ON 2'               *)
(*!!FILE REVISION: 1.01 GLM                                *)
(*  REVISION DATE: 04/27/2002                              *)
(*      - First line returned by the 'SEND ASCII...' and   *)
(*        'SEND ITOHEX...' calls is one less character     *)
(*        than subsequent lines to account for Netlinx '>' *)
(*        prompt. Removed in version 1.02.                 *)
(***********************************************************)
(*!!FILE REVISION: 1.00 GLM                                *)
(*  REVISION DATE: 03/26/2002                              *)
(*                                                         *)
(*  COMMENTS:                                              *)
(* DEBUG FUNCTIONS AND CALLS                               *)
/*  1   FUNCTION CHAR[16] CUSTOM_EVENT_JUSTIFY_TO_STRING (SLONG slJustify)    */
/*  2   FUNCTION CHAR[32] CUSTOM_EVENT_TYPE_TO_STRING (LONG lCeType)          */
/*  3   FUNCTION CHAR[32] CUSTOM_EVENT_VALUE2_TO_STRING (LONG lCeType)        */
/*  4   FUNCTION CHAR[32] CUSTOM_EVENT_VALUE3_TO_STRING (LONG lCeType)        */
/*  5   FUNCTION LONG formatAsciiHex (STR[], lenSTR, sTemp[], lenPacket, i)   */
/*  6   FUNCTION LONG formatAscii (STR[], lenSTR, sTemp[], lenPacket, i)      */
/*  7   FUNCTION LONG formatDecimal (STR[], lenSTR, sTemp[], lenPacket, i)    */
/*  8   FUNCTION LONG formatHex (STR[], lenSTR, sTemp[], lenPacket, i)        */
/*  9   FUNCTION LONG formatMIDI (STR[], lenSTR, sTemp[], lenPacket, s, i)    */
/* 10   FUNCTION printCUSTOM_EVENT (DEV dvDebug, CHAR hdr[], INTEGER flagNIT) */
/* 11   FUNCTION printDEVICE_INFO (DEV dvDEBUG, CHAR hdr[], DEV dvDEV)        */
/* 12   CALL 'SEND 131 BYTE PACKETS TO MASTER' (sSTRING[])                    */
/* 13   CALL 'SEND ASCII/HEX TO DEBUG' (DEV dvDEBUG,STR1[],STR2[],nLine,nMode)*/
/* 14   CALL 'SEND ASCII TO DEBUG' (DEV dvDEBUG,STR1[],STR2[],nLine,nMode)    */
/* 15   CALL 'SEND ASCII TO MASTER' (STR1[],STR2[])                           */
/* 16   CALL 'SEND CUSTOM_EVENT DATA TO DEBUG' (DEV dvDebug)                  */
/* 17   CALL 'SEND DECIMAL TO DEBUG' (DEV dvDEBUG,STR1[],STR2[],nLine,nMode)  */
/* 18   CALL 'SEND DEVICE_INFO TO DEBUG' (DEV dvDEBUG, DEV dvDEV)             */
/* 19   CALL 'SEND INTEGER TO DEBUG' (DEV dvDEBUG,STR1[],STR2[],nLine,nMode)  */
/* 20   CALL 'SEND ITOHEX TO DEBUG' (DEV dvDEBUG,STR1[],STR2[],nLine,nMode)   */
/* 21   CALL 'SEND ITOHEX TO MASTER' (STR1[],STR2[])                          */
/* 22   CALL 'SEND MIDI TO DEBUG' (dvDEBUG,STR1[],STR2[],nLINE,nMODE,nStat)   */
(***********************************************************)
#IF_NOT_DEFINED __DEBUG_LIB__
#DEFINE __DEBUG_LIB__

#INCLUDE 'DEV_TO_STRING'
(***********************************************************)
(*          DEVICE NUMBER DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_DEVICE

(***********************************************************)
(*               CONSTANT DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_CONSTANT

// VERSION FOR CODE

CHAR __DEBUG_LIB_NAME__[12]     = '   debug.axi';
CHAR __DEBUG_LIB_VERSION__[]    = '2.60';

(***********************************************************)
(*               VARIABLE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_VARIABLE

VOLATILE CHAR __SPACES__[128] =
{
  $20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,
  $20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,
  $20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,
  $20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,
  $20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,
  $20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,
  $20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,
  $20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20
}

#IF_NOT_DEFINED __display_lib_version__
VOLATILE __display_lib_version__ = TRUE
#END_IF

(***********************************************************)
(*           SUBROUTINE DEFINITIONS GO BELOW               *)
(***********************************************************)
(* EXAMPLE: DEFINE_FUNCTION <RETURN_TYPE> <NAME> (<PARAMETERS>) *)
(* EXAMPLE: DEFINE_CALL '<NAME>' (<PARAMETERS>) *)

                                    ///////////////////////////
                                    // DEBUG FUNCTIONS/CALLS //
                                    ///////////////////////////

DEFINE_FUNCTION CHAR[16] CUSTOM_EVENT_JUSTIFY_TO_STRING (SLONG slJustify)
// FUNCTION CUSTOM_EVENT_JUSTIFY_TO_STRING (SLONG slJustify)
// PARAMETER:
// slJustify - CUSTOM.VALUE2 from CUSTOM.EVENT types 1004,1005,& 1006
// RETURNS:
// G4 JUSTIFICATION as a string
{
  SWITCH (slJustify)
  {
	CASE 1: RETURN 'TOP LEFT';
	CASE 2: RETURN 'TOP CENTER';
	CASE 3: RETURN 'TOP RIGHT';
	CASE 4: RETURN 'MIDDLE LEFT';
	CASE 5: RETURN 'MIDDLE CENTER';
	CASE 6: RETURN 'MIDDLE RIGHT';
	CASE 7: RETURN 'BOTTOM LEFT';
	CASE 8: RETURN 'BOTTOM CENTER';
	CASE 9: RETURN 'BOTTOM RIGHT';
	DEFAULT: RETURN 'UNKNOWN';
  }
}

DEFINE_FUNCTION CHAR[32] CUSTOM_EVENT_TYPE_TO_STRING (LONG lCeType)
// FUNCTION CUSTOM_EVENT_TYPE_TO_STRING (LONG lCeType)
// PARAMETER:
// lCeType - CUSTOM.TYPE from CUSTOM.EVENT
// RETURNS:
// CUSTOM.TYPE string
{
  SWITCH (lCeType)
  {
	CASE 1001: RETURN 'TEXT';
	CASE 1002: RETURN 'BITMAP';
	CASE 1003: RETURN 'ICON';
	CASE 1004: RETURN 'TEXT JUSTIFICATION';
	CASE 1005: RETURN 'BITMAP JUSTIFICATION';
	CASE 1006: RETURN 'ICON JUSTIFICATION';
	CASE 1007: RETURN 'FONT';
	CASE 1008: RETURN 'TEXT EFFECT NAME';
	CASE 1009: RETURN 'TEXT EFFECT COLOR';
	CASE 1010: RETURN 'WORD WRAP';
	CASE 1011: RETURN 'BORDER COLOR';
	CASE 1012: RETURN 'FILL COLOR';
	CASE 1013: RETURN 'TEXT COLOR';
	CASE 1014: RETURN 'BORDER NAME';
	CASE 1015: RETURN 'OPACITY';
	DEFAULT:   RETURN 'UNKNOWN';
  }
}

DEFINE_FUNCTION CHAR[32] CUSTOM_EVENT_VALUE2_TO_STRING (LONG lCeType)
// FUNCTION CUSTOM_EVENT_VALUE2_TO_STRING (LONG lCeType)
// PARAMETER:
// lCeType - CUSTOM.TYPE from CUSTOM.EVENT
// RETURNS:
// VALUE2 name string
{
  SWITCH (lCeType)
  {
	CASE 1001: RETURN 'MESSAGE LENGTH';
	CASE 1002: RETURN 'MESSAGE LENGTH';
	CASE 1003: RETURN 'ICON INDEX';
	CASE 1004: RETURN 'TEXT JUSTIFICATION';
	CASE 1005: RETURN 'BITMAP JUSTIFICATION';
	CASE 1006: RETURN 'ICON JUSTIFICATION';
	CASE 1007: RETURN 'FONT INDEX';
	CASE 1008: RETURN 'MESSAGE LENGTH';
	CASE 1009: RETURN 'MESSAGE LENGTH';
	CASE 1010: RETURN 'WORD WRAP';
	CASE 1011: RETURN 'MESSAGE LENGTH';
	CASE 1012: RETURN 'MESSAGE LENGTH';
	CASE 1013: RETURN 'MESSAGE LENGTH';
	CASE 1014: RETURN 'MESSAGE LENGTH';
	CASE 1015: RETURN 'OPACITY VALUE';
	DEFAULT:   RETURN 'UNKNOWN';
  }
}

DEFINE_FUNCTION CHAR[32] CUSTOM_EVENT_VALUE3_TO_STRING (LONG lCeType)
// FUNCTION CUSTOM_EVENT_VALUE3_TO_STRING (LONG lCeType)
// PARAMETER:
// lCeType - CUSTOM.TYPE from CUSTOM.EVENT
// RETURNS:
// VALUE3 name string
{
  SWITCH (lCeType)
  {
	CASE 1001: RETURN 'TEXT INDEX';
	CASE 1002: RETURN 'VALUE3';
	CASE 1003: RETURN 'VALUE3';
	CASE 1004: RETURN 'VALUE3';
	CASE 1005: RETURN 'VALUE3';
	CASE 1006: RETURN 'VALUE3';
	CASE 1007: RETURN 'VALUE3';
	CASE 1008: RETURN 'VALUE3';
	CASE 1009: RETURN 'VALUE3';
	CASE 1010: RETURN 'VALUE3';
	CASE 1011: RETURN 'VALUE3';
	CASE 1012: RETURN 'VALUE3';
	CASE 1013: RETURN 'VALUE3';
	CASE 1014: RETURN 'VALUE3';
	CASE 1015: RETURN 'VALUE3';
	DEFAULT:   RETURN 'UNKNOWN';
  }
}

DEFINE_FUNCTION LONG formatAsciiHex(char STR[], long lenSTR, char sTemp[], integer lenPacket, long i)
// FUNCTION: formatAsciiHex()
// Purpose:
//  - Create one ASCII packet of length lenPacket from STR, without destroying STR contents
//  - Can be called recursively if STR is too long to be formatted into one packet
//  - PASSES ASCII CHARACTERS UNFORMATTED, ASCII STRINGS INSIDE SINGLE-QUOTES
//  - FORMATS CHARS >= $7F (del) AS 2 DIGIT HEX, SEPERATED BY COMMAS
//  - FORMATS CONTROL CHARACTERS AS 2 DIGIT HEX, SEPERATED BY COMMAS
//  - ASCII STRINGS SEPERATED FROM HEX BY COMMAS
// PARAMETERS:
//  - STR:       data to be formatted
//  - lenSTR:    length of data to be formatted
//  - sTemp:     temp storage for formatted data
//  - lenPacket: maximum desired length of formatted data
//  - i:         current index into str[]
// RETURNS:
//  - i
{
  IF (i = 0) { i = 1 }              // NetLinx indexing starts at 1, not 0
  FOR (; i <= lenSTR; i++)
  {
									// IF IT'S A CONTROL CHARACTER
									// OR DEL OR EXTENDED ASCII
    IF (STR[i] < $20 || STR[i] >= $7F)
    {
      IF (i = lenSTR)               // IF IT'S THE LAST BYTE
      {
                                    // JUST PRINT $XX
        sTemp = "sTemp,FORMAT('$%02X',STR[i])";
      }
									// IF THE NEXT BYTE IS A CONTROL CHARACTER
									// OR DEL OR EXTENDED ASCII
      ELSE IF (STR[i+1] < $20 || STR[i+1] > $7F)
      {
                                    // END WITH COMMA
        sTemp = "sTemp,FORMAT('$%02X,',STR[i])";
      }
      ELSE
      {
                                    // END WITH COMMA/SINGLE QUOTE
        sTemp = "sTemp,FORMAT('$%02X,''',STR[i])";
      }
    }
    ELSE                            // IF IT'S PRINTABLE
    {
      SELECT
      {
        ACTIVE (i = 1):             // IF IT'S THE FIRST BYTE
        {
          IF (i = lenSTR)           // IF IT'S THE ONLY BYTE
          {
                                    // PUT IT IN SINGLE QUOTES
            sTemp = "'''',STR[i],''''";
          }
		  ELSE IF                   // ELSE IF THE NEXT BYTE IS
			   ((STR[i+1] < $20) || // A CONTROL CHARACTER OR
                (STR[i+1] >= $7F))  // THE NEXT BYTE IS DEL OR EXTENDED ASCII
		  {
								    // PUT IT IN SINGLE QUOTES FOLLOWED BY COMMA
            sTemp = "'''',STR[i],''','";
		  }
          ELSE
          {
            sTemp = "'''',STR[i]";  // LEADING SINGLE QUOTE ONLY
          }
        }
        ACTIVE (i = lenSTR):        // IF IT'S THE LAST BYTE
        {
                                    // TRAILING SINGLE QUOTE ONLY
          sTemp = "sTemp,STR[i],''''";
        }
        ACTIVE (STR[i+1] < $20):    // IF THE NEXT BYTE IS A CONTROL CHARACTER
        {
                                    // TRAILING SINGLE QUOTE/COMMA
          sTemp = "sTemp,STR[i],''','";
        }
        ACTIVE (STR[i+1] >= $7F):   // IF THE NEXT BYTE IS DEL OR EXTENDED ASCII
        {
                                    // TRAILING SINGLE QUOTE/COMMA
          sTemp = "sTemp,STR[i],''','";
        }
        ACTIVE (1):
        {
          sTemp = "sTemp,STR[i]";   // JUST PRINT THE CHARACTER
        }
      }
    }

									// if ASCII packet is long enough, break the loop
	IF (LENGTH_ARRAY(sTemp) >= lenPacket) 
	{ 
	  i++;                          // return next index into str[]
	  BREAK;
	}
  }
  RETURN i;                         // i = LENGTH_ARRAY(str)+1 or next index into str[]
}
DEFINE_FUNCTION LONG formatAscii(char STR[], long lenSTR, char sTemp[], integer lenPacket, long i)
// FUNCTION: formatAscii()
// Purpose:
//  - Create one ASCII packet of length lenPacket from STR, without destroying STR contents
//  - Can be called recursively if STR is too long to be formatted into one packet
//  - PASSES ASCII and EXTENDED ASCII CHARACTERS UNFORMATTED, ASCII STRINGS INSIDE SINGLE-QUOTES
//  - FORMATS CONTROL CHARACTERS AS 2 DIGIT HEX, SEPERATED BY COMMAS
//  - ASCII STRINGS SEPERATED FROM HEX BY COMMAS
// PARAMETERS:
//  - STR:       data to be formatted
//  - lenSTR:    length of data to be formatted
//  - sTemp:     temp storage for formatted data
//  - lenPacket: maximum desired length of formatted data
//  - i:         current index into str[]
// RETURNS:
//  - i
{
  IF (i = 0) { i = 1 }              // NetLinx indexing starts at 1, not 0
  FOR (; i <= lenSTR; i++)
  {
									// IF IT'S A CONTROL CHARACTER
    IF (STR[i] < $20 || STR[i] = $7F)
    {
      IF (i = lenSTR)               // IF IT'S THE LAST BYTE
      {
                                    // JUST PRINT $XX
        sTemp = "sTemp,FORMAT('$%02X',STR[i])";
      }
									// IF THE NEXT BYTE IS A CONTROL CHARACTER
      ELSE IF (STR[i+1] < $20 || STR[i+1] = $7F)
      {
                                    // END WITH COMMA
        sTemp = "sTemp,FORMAT('$%02X,',STR[i])";
      }
      ELSE
      {
                                    // END WITH COMMA/SINGLE QUOTE
        sTemp = "sTemp,FORMAT('$%02X,''',STR[i])";
      }
    }
    ELSE                            // IF IT'S PRINTABLE
    {
      SELECT
      {
        ACTIVE (i = 1):             // IF IT'S THE FIRST BYTE
        {
          IF (i = lenSTR)           // IF IT'S THE ONLY BYTE
          {
                                    // PUT IT IN SINGLE QUOTES
            sTemp = "'''',STR[i],''''";
          }
		                            // ELSE IF THE NEXT BYTE IS A CONTROL CHARACTER
		  ELSE IF ((STR[i+1] < $20) || (STR[i+1] = $7F))
		  {
								    // PUT IT IN SINGLE QUOTES FOLLOWED BY COMMA
            sTemp = "'''',STR[i],''','";
		  }
          ELSE
          {
            sTemp = "'''',STR[i]";  // LEADING SINGLE QUOTE ONLY
          }
        }
        ACTIVE (i = lenSTR):        // IF IT'S THE LAST BYTE
        {
                                    // TRAILING SINGLE QUOTE ONLY
          sTemp = "sTemp,STR[i],''''";
        }
									// IF THE NEXT BYTE IS A CONTROL CHARACTER
        ACTIVE (STR[i+1] < $20 || STR[i+1] = $7F):
        {
                                    // TRAILING SINGLE QUOTE/COMMA
          sTemp = "sTemp,STR[i],''','";
        }
        ACTIVE (1):
        {
          sTemp = "sTemp,STR[i]";   // JUST PRINT THE CHARACTER
        }
      }
    }
									// if ASCII packet is long enough, break the loop
	IF (LENGTH_ARRAY(sTemp) >= lenPacket) 
	{ 
	  i++;                          // return next index into str[]
	  BREAK;
	}
  }
  RETURN i;                         // i = LENGTH_ARRAY(str)+1 or next index into str[]
}
DEFINE_FUNCTION LONG formatDecimal(char STR[], long lenSTR, char sTemp[], integer lenPacket, long i)
// FUNCTION: formatDecimal()
// Purpose:
//  - Create one ASCII packet of length lenPacket from STR, without destroying STR contents
//  - Can be called recursively if STR is too long to be formatted into one packet
//  - FORMATS ALL CHARACTERS AS 3 DIGIT DECIMAL, SEPERATED BY COMMAS
// PARAMETERS:
//  - STR:       data to be formatted
//  - lenSTR:    length of data to be formatted
//  - sTemp:     temp storage for formatted data
//  - lenPacket: desired length of formatted data
//  - i:         current index into str[]
// RETURNS:
//  - i
{
  IF (i = 0) { i = 1 }              // NetLinx indexing starts at 1, not 0
  FOR (; i <= lenSTR; i++)
  {
    IF (i = lenSTR)                // IF IT'S THE LAST BYTE
    {
      sTemp = "sTemp,FORMAT('%03d',STR[i])";
    }
    ELSE
    {
      sTemp = "sTemp,FORMAT('%03d,',STR[i])";
    }

									// if ASCII packet is long enough, break the loop
	IF (LENGTH_ARRAY(sTemp) >= lenPacket) 
	{ 
	  i++;                          // return next index into str[]
	  BREAK;
	}
  }
  RETURN i;                         // i = LENGTH_ARRAY(str)+1 or next index into str[]
}
DEFINE_FUNCTION LONG formatHex(char STR[], long lenSTR, char sTemp[], integer lenPacket, long i)
// FUNCTION: formatHex()
// Purpose:
//  - Create one ASCII packet of length lenPacket from STR, without destroying STR contents
//  - Can be called recursively if STR is too long to be formatted into one packet
//  - FORMATS ALL CHARACTERS AS 2 DIGIT HEX, SEPERATED BY COMMAS
// ASCII STRINGS SEPERATED FROM HEX BY COMMAS
// PARAMETERS:
//  - STR:       data to be formatted
//  - lenSTR:    length of data to be formatted
//  - sTemp:     temp storage for formatted data
//  - lenPacket: desired length of formatted data
//  - i:         current index into str[]
// RETURNS:
//  - i
{
  IF (i = 0) { i = 1 }              // NetLinx indexing starts at 1, not 0
  FOR (; i <= lenSTR; i++)
  {
	IF (i = lenSTR)                // IF IT'S THE LAST BYTE
    {
      sTemp = "sTemp,FORMAT('$%02X',STR[i])";
    }
    ELSE
    {
      sTemp = "sTemp,FORMAT('$%02X,',STR[i])";
    }

									// if ASCII packet is long enough, break the loop
	IF (LENGTH_ARRAY(sTemp) >= lenPacket) 
	{ 
	  i++;                          // return next index into str[]
	  BREAK;
	}
  }
  RETURN i;                         // i = LENGTH_ARRAY(str)+1 or next index into str[]
}
DEFINE_FUNCTION LONG formatMIDI(char STR[], long lenSTR, char sTemp[], integer lenPacket, integer s, long i)
// FUNCTION: formatMIDI()
// Purpose:
//  - Create one ASCII packet of length lenPacket from STR, without destroying STR contents
//  - Can be called recursively if STR is too long to be formatted into one packet
//  - Formats MIDI channel voice messages with:
//              MIDI status bytes as 2 digit hex, seperated by commas.
//              MIDI data bytes as 3 digit decimal, seperated by commas.
//  - Formats MIDI system messages as 2 digit hex, seperated by commas.
// PARAMETERS:
//  - STR:       data to be formatted
//  - lenSTR:    length of data to be formatted
//  - sTemp:     temp storage for formatted data
//  - lenPacket: desired length of formatted data
//  - s:         current MIDI status byte:
//                $EF and lower = default = format status bytes as hex, data bytes as decimal.
//                $F0 and higher = format status and data as hex.
//  - i:        current index into str[]
// RETURNS:
//  - i
{
  FOR (; i <= lenSTR; i++)
  {
	IF (s & $F0 == $F0)		        // if it's a MIDI SYSTEM message
	{
	  IF (i = lenSTR)               // IF IT'S THE LAST BYTE
	  {
		sTemp = "sTemp,FORMAT('$%02X',STR[i])";
	  }
	  ELSE
	  {
		sTemp = "sTemp,FORMAT('$%02X,',STR[i])";
	  }
	}
	ELSE
	{
	  IF (i = lenSTR)               // IF IT'S THE LAST BYTE
	  {
		IF	(STR[i] & $80) sTemp = "sTemp,FORMAT('$%02X',STR[i])";
		ELSE			   sTemp = "sTemp,FORMAT('%03d',STR[i])";
	  }
	  ELSE
	  {
		IF	(STR[i] & $80) sTemp = "sTemp,FORMAT('$%02X,',STR[i])";
		ELSE			   sTemp = "sTemp,FORMAT('%03d,',STR[i])";
	  }
	}

									// if ASCII packet is long enough, break the loop
	IF (LENGTH_ARRAY(sTemp) >= lenPacket) 
	{ 
	  i++;                          // return next index into str[]
	  BREAK;
	}
  }
  RETURN i;                         // i = LENGTH_ARRAY(str)+1 or next index into str[]
}

DEFINE_FUNCTION printCUSTOM_EVENT (DEV dvDebug, CHAR hdr[], INTEGER flagNotInTerminal)
// FUNCTION printCUSTOM_EVENT (DEV dvDebug, CHAR hdr[])
// This subroutine must be called inside of a CUSTOM_EVENT
// Purpose: Print CUSTOM_EVENT data to dvDebug
//          Difference from CALL 'SEND CUSTOM_EVENT DATA TO DEBUG':
//          1. Header parameter for companion debug module v2.xx and higher.
//             Header is prepended to each line.
// PARAMETERS:
// dvDEBUG IS DESTINATION
// hdr IS HEADER
// flagNIT is FALSE if result is to be displayed in terminal
{
  STACK_VAR CHAR sVALUE2[32];
									/****************/
									/* DEVCHAN INFO */
									/****************/
  SEND_STRING dvDebug,"hdr,'D:P:S ',DEV_TO_STRING(CUSTOM.DEVICE)"
  SEND_STRING dvDebug,"hdr,'BUTTON ADDRESS ',ITOA(CUSTOM.ID)";
  SEND_STRING dvDebug,"hdr,'BUTTON STATE ',ITOA(CUSTOM.VALUE1)";

									/**********************/
								    /* BASIC MESSAGE INFO */
									/**********************/
  SEND_STRING dvDebug,"hdr,'MESSAGE TYPE ', CUSTOM_EVENT_TYPE_TO_STRING(CUSTOM.TYPE)";

								    // if this is a justification response
  IF (CUSTOM.TYPE = 1004 || CUSTOM.TYPE = 1005 || CUSTOM.TYPE = 1006)
  {
	sVALUE2 = CUSTOM_EVENT_JUSTIFY_TO_STRING(CUSTOM.VALUE2);
  }
  ELSE
  {
	sVALUE2 = ITOA(CUSTOM.VALUE2);
  }
  SEND_STRING dvDebug,"hdr,'',CUSTOM_EVENT_VALUE2_TO_STRING(CUSTOM.TYPE),' ',sVALUE2";

									/*************************/
								    /* OPTIONAL MESSAGE INFO */
									/*************************/
  SEND_STRING dvDebug,"hdr,'UNICODE ',ITOA(CUSTOM.FLAG)";

  SEND_STRING dvDebug,"hdr,'',CUSTOM_EVENT_VALUE3_TO_STRING(CUSTOM.TYPE),' ',ITOA(CUSTOM.VALUE3)";

  SEND_STRING dvDebug,"hdr,'TEXT LENGTH ',ITOA(LENGTH_STRING(CUSTOM.TEXT))";

  IF (flagNotInTerminal)            // if result may be parsed by code
  {
	SEND_STRING dvDebug,"hdr,'TEXT ',CUSTOM.TEXT"
  }
  ELSE                              // if result is to be read in a terminal
  {
	CALL 'SEND ASCII/HEX TO DEBUG' (dvDebug,"hdr,'TEXT '",CUSTOM.TEXT,0,0);
  }
}

DEFINE_FUNCTION printDEVICE_INFO (DEV dvDEBUG, CHAR hdr[], DEV dvDEV)
// FUNCTION printDEVICE_INFO (DEV dvDEBUG, DEV dvDEV)
// PURPOSE:    Get device info structure for a device,
//             then send the info to a debug device such
//             as the master's debug port.
// Difference from CALL 'SEND DEVICE_INFO TO DEBUG':
//          1. Header parameter for companion debug module v2.xx and higher.
//             Header is prepended to each line.
// PARAMETERS: dvDEBUG   - device to receive device info
//             hdr       - Header
//             dvDEV     - device from which to get info
{
  IF (DEVICE_ID(dvDEV))             // if device is online
  {
    STACK_VAR DEV_INFO_STRUCT devinfo;
	STACK_VAR CHAR sSTR[100];

	DEVICE_INFO(dvDEV,devinfo);

	send_string dvDebug,"hdr,'D:P:S ',DEV_TO_STRING(dvDEV)"

	if (length_array(devinfo.MANUFACTURER_STRING))
	  send_string dvDebug,"hdr,'Manufacturer ',devinfo.MANUFACTURER_STRING"
	else
	  send_string dvDebug,"hdr,'Manufacturer not available'"

	if (devinfo.MANUFACTURER)
	  send_string dvDebug,"hdr,'Manufacturer ID ',ITOA(devinfo.MANUFACTURER)"
	else
	  send_string dvDebug,"hdr,'Manufacturer ID not available'"

	if (length_array(devinfo.DEVICE_ID_STRING))
	  send_string dvDebug,"hdr,'Device ',devinfo.DEVICE_ID_STRING"
	else
	  send_string dvDebug,"hdr,'Device not available'"

	send_string dvDebug,"hdr,'Device ID $',ITOHEX(devinfo.DEVICE_ID)"

	if (length_array(devinfo.VERSION))
	  send_string dvDebug,"hdr,'Version ',devinfo.VERSION"
	else
	  send_string dvDebug,"hdr,'Version not available'"

	if (devinfo.FIRMWARE_ID)
	  send_string dvDebug,"hdr,'Firmware ID $',ITOHEX(devinfo.FIRMWARE_ID)"
	else
	  send_string dvDebug,"hdr,'Firmware ID not available'"

	if (length_array(devinfo.SERIAL_NUMBER))
	{
	                                // if serial_number is blank for Netlinx device,
									// first byte is $00
	  if (find_string(devinfo.SERIAL_NUMBER,"$00",1) = 1)
									// 16 0's
	    send_string dvDebug,"hdr,'Serial Number 0000000000000000'"

									// trim excess bytes if SN is not 16 chars long
	  else if (find_string(devinfo.SERIAL_NUMBER,"$00",1) > 1)
		send_string dvDebug,"hdr,'Serial Number ',
											LEFT_STRING(devinfo.SERIAL_NUMBER,
LENGTH_ARRAY(devinfo.SERIAL_NUMBER) - (17 - FIND_STRING(devinfo.serial_number,"$00",1))
													   )"

	  else                          // if no $00, just send the whole string
	    send_string dvDebug,"hdr,'Serial Number ',devinfo.SERIAL_NUMBER"
	}
	else                            // this should never happen
	  send_string dvDebug,"hdr,'Serial Number not available'"

	SWITCH (devinfo.SOURCE_TYPE)
	{
	  CASE SOURCE_TYPE_NO_ADDRESS:
		send_string dvDebug,FORMAT("hdr,'Source Type $%02X: no address'",devinfo.SOURCE_TYPE)
	  CASE SOURCE_TYPE_NEURON_ID:
		send_string dvDebug,FORMAT("hdr,'Source Type $%02X: connected via ICSNet'",devinfo.SOURCE_TYPE)
	  CASE SOURCE_TYPE_IP_ADDRESS:
		send_string dvDebug,FORMAT("hdr,'Source Type $%02X: connected via ethernet'",devinfo.SOURCE_TYPE)
	  CASE SOURCE_TYPE_AXLINK:
		send_string dvDebug,FORMAT("hdr,'Source Type $%02X: connected via AXLink'",devinfo.SOURCE_TYPE)
	  CASE SOURCE_TYPE_NEURON_SUBNODE_ICSP:
		send_string dvDebug,FORMAT("hdr,'Source Type $%02X: connected via ICSNet'",devinfo.SOURCE_TYPE)
	  CASE SOURCE_TYPE_NEURON_SUBNODE_PL:
		send_string dvDebug,FORMAT("hdr,'Source Type $%02X: connected via ICSNet'",devinfo.SOURCE_TYPE)
	  CASE SOURCE_TYPE_IP_SOCKET_ADDRESS:
		send_string dvDebug,FORMAT("hdr,'Source Type $%02X: connected via ethernet'",devinfo.SOURCE_TYPE)
	  CASE SOURCE_TYPE_RS232:
		send_string dvDebug,FORMAT("hdr,'Source Type $%02X: connected via RS232'",devinfo.SOURCE_TYPE)
	  CASE SOURCE_TYPE_INTERNAL:
		send_string dvDebug,FORMAT("hdr,'Source Type $%02X: internal to the NetLinx controller'",devinfo.SOURCE_TYPE)

		// next 3 source types added NetLinx.axi v1.25
	  CASE SOURCE_TYPE_MAC_ADDRESS:
		send_string dvDebug,FORMAT("hdr,'Source Type $%02X: connected via ethernet'",devinfo.SOURCE_TYPE)
	  CASE SOURCE_TYPE_IPv4_PORT:
		send_string dvDebug,FORMAT("hdr,'Source Type $%02X: connected via ethernet'",devinfo.SOURCE_TYPE)
	  CASE SOURCE_TYPE_IPv4_PORT_MAC_ADDRESS:
		send_string dvDebug,FORMAT("hdr,'Source Type $%02X: connected via ethernet'",devinfo.SOURCE_TYPE)

	  DEFAULT:
		send_string dvDebug,FORMAT("hdr,'Source Type $%02X: unknown'",devinfo.SOURCE_TYPE)
	}

	if (length_array(devinfo.SOURCE_STRING))
	  send_string dvDebug,"hdr,'Source String ',devinfo.SOURCE_STRING"
	else
	  send_string dvDebug,"hdr,'Source String not available'"
  }
  else
	send_string dvDebug,"hdr,'D:P:S ',DEV_TO_STRING(dvDEV),' not online'"
}

DEFINE_CALL 'SEND 131 BYTE PACKETS TO MASTER' (CHAR STR1[15999])
// DEFINE_CALL 'SEND 131 BYTE PACKETS TO MASTER'
// BREAKS STRINGS UP INTO THE MAXIMUM SIZE
// THAT CAN BE DISPLAYED IN A MASTER TERMINAL,
// SENDS THEM TO THE MASTER.
// PARAMETER:
// STR1  - STRING TO BE SENT TO MASTER
{
  STACK_VAR LONG X;
  STACK_VAR LONG STR1_LEN;

  STR1_LEN = LENGTH_ARRAY(STR1);

  FOR (X = 1; X < STR1_LEN; X = X + 131)
  {
	SEND_STRING 0:1:0,"MID_STRING(STR1,X,131)"
  }
}

DEFINE_CALL 'SEND 7BIT ASCII TO DEBUG' (DEV dvDEBUG,CHAR STR1[],CHAR STR2[15999],INTEGER nLINE,INTEGER nMODE)
// CALL 'SEND 7BIT ASCII TO DEBUG'
// v2.14 - now a wrapper for 'SEND ASCII/HEX TO DEBUG'
{
  CALL 'SEND ASCII/HEX TO DEBUG'  (dvDEBUG,STR1,STR2,nLINE,nMODE)
}
DEFINE_CALL 'SEND ASCII/HEX TO DEBUG' (DEV dvDEBUG,CHAR STR1[],CHAR STR2[15999],INTEGER nLINE,INTEGER nMODE)
// CALL 'SEND ASCII/HEX TO DEBUG'
// PASSES ASCII CHARACTERS UNFORMATTED, ASCII STRINGS INSIDE SINGLE-QUOTES
// FORMATS CHARS >= $7F (del) AS 2 DIGIT HEX, SEPERATED BY COMMAS
// FORMATS CONTROL CHARACTERS AS 2 DIGIT HEX, SEPERATED BY COMMAS
// ASCII STRINGS SEPERATED FROM HEX BY COMMAS
// PARAMETERS:
// dvDEBUG IS DESTINATION
// STR1 IS HEADER
// STR2 IS DATA TO BE FORMATTED
// nLine IS LINE LENGTH. 0 = default = 80 for nMode 2, 67 for all others.
// nMode is MSG ON mode. 2 = append <cr>,<lf> to lines.
{
  STACK_VAR	integer STR1_LEN;
  STACK_VAR LONG STR2_LEN;
  STACK_VAR LONG X;
  STACK_VAR char ASCII[2560];
  STACK_VAR integer ASCII_PACKET_LEN;
  STACK_VAR INTEGER FIRST_STRING_SENT;
  STACK_VAR CHAR HEADER[128];

  STR1_LEN = LENGTH_ARRAY(STR1);
  STR2_LEN = LENGTH_ARRAY(STR2);
  HEADER = STR1;

  SELECT                            // SET THE ASCII PACKET LENGTH
  {
//    If  nLine > 131, longer messages may be truncated
//    e.g. when dvDebug is the master (send_string 0).
    ACTIVE (nLINE > MAX_LENGTH_ARRAY(ASCII)):
    {
      ASCII_PACKET_LEN = MAX_LENGTH_ARRAY(ASCII) - STR1_LEN;
    }
    ACTIVE (nLINE > STR1_LEN):
    {
      ASCII_PACKET_LEN = nLINE - STR1_LEN;
    }
    ACTIVE (nMODE = 2):             // MODE 2 DEFAULT TO 80 CHARACTER LINE
    {
      ASCII_PACKET_LEN = 80 - STR1_LEN;
    }
    ACTIVE (1):                     // DEFAULT TO 67 CHARACTER LINE
    {
      ASCII_PACKET_LEN = 67 - STR1_LEN;
    }
  }

									// BUILD UP ASCII STRING FROM STR2
  FOR (X = 1; X <= STR2_LEN;/* x is incremented via formatAsciiHex() */ )
  {
	X = formatAsciiHex (STR2,STR2_LEN,ASCII,ASCII_PACKET_LEN,X)

									// if the line is ready, send it
	IF (LENGTH_ARRAY(ASCII) >= ASCII_PACKET_LEN)
	{
	  IF (nMODE = 2)
	  {
		SEND_STRING dvDEBUG,"HEADER,GET_BUFFER_STRING(ASCII,ASCII_PACKET_LEN),13,10";
	  }
	  ELSE
	  {
		SEND_STRING dvDEBUG,"HEADER,GET_BUFFER_STRING(ASCII,ASCII_PACKET_LEN)";
	  }

	  IF (!FIRST_STRING_SENT)		// replace HEADER with spaces after first pass
	  {
		ON[FIRST_STRING_SENT]
		HEADER = LEFT_STRING(__SPACES__,LENGTH_STRING(HEADER))
	  }
	}
  }

  IF (LENGTH_ARRAY(ASCII))          // IF THERE'S ANYTHING LEFT SEND IT
  {
    IF (nMODE = 2)
    {
      SEND_STRING dvDEBUG,"HEADER,ASCII,13,10";
    }
    ELSE
    {
      SEND_STRING dvDEBUG,"HEADER,ASCII";
    }
  }
}

DEFINE_CALL 'SEND ASCII TO DEBUG' (DEV dvDEBUG,CHAR STR1[],CHAR STR2[15999],INTEGER nLINE,INTEGER nMODE)
// CALL 'SEND ASCII TO DEBUG'
// PASSES ASCII and EXTENDED ASCII CHARACTERS UNFORMATTED, ASCII STRINGS INSIDE SINGLE-QUOTES
// FORMATS CONTROL CHARACTERS AS 2 DIGIT HEX, SEPERATED BY COMMAS
// ASCII STRINGS SEPERATED FROM HEX BY COMMAS
// PARAMETERS:
// dvDEBUG IS DESTINATION
// STR1 IS HEADER
// STR2 IS DATA TO BE FORMATTED
// nLine IS LINE LENGTH. 0 = default = 80 for nMode 2, 67 for all others.
// nMode is MSG ON mode. 2 = append <cr>,<lf> to lines.
{
  STACK_VAR	integer STR1_LEN;
  STACK_VAR LONG STR2_LEN;
  STACK_VAR LONG X;
  STACK_VAR char ASCII[2560];
  STACK_VAR integer ASCII_PACKET_LEN;
  STACK_VAR INTEGER FIRST_STRING_SENT;
  STACK_VAR CHAR HEADER[128];

  STR1_LEN = LENGTH_ARRAY(STR1);
  STR2_LEN = LENGTH_ARRAY(STR2);
  HEADER = STR1;

  SELECT                            // SET THE ASCII PACKET LENGTH
  {
//    If  nLine > 131, longer messages may be truncated
//    e.g. when dvDebug is the master (send_string 0).
    ACTIVE (nLINE > MAX_LENGTH_ARRAY(ASCII)):
    {
      ASCII_PACKET_LEN = MAX_LENGTH_ARRAY(ASCII) - STR1_LEN;
    }
    ACTIVE (nLINE > STR1_LEN):
    {
      ASCII_PACKET_LEN = nLINE - STR1_LEN;
    }
    ACTIVE (nMODE = 2):             // MODE 2 DEFAULT TO 80 CHARACTER LINE
    {
      ASCII_PACKET_LEN = 80 - STR1_LEN;
    }
    ACTIVE (1):                     // DEFAULT TO 67 CHARACTER LINE
    {
      ASCII_PACKET_LEN = 67 - STR1_LEN;
    }
  }

									// BUILD UP ASCII STRING FROM STR2
  FOR (X = 1; X <= STR2_LEN; /* x is incremented via formatAscii() */)
  {
	X = formatAscii(STR2,STR2_LEN,ASCII,ASCII_PACKET_LEN,X)

									// if the line is ready, send it
	IF (LENGTH_ARRAY(ASCII) >= ASCII_PACKET_LEN)
	{
	  IF (nMODE = 2)
	  {
		SEND_STRING dvDEBUG,"HEADER,GET_BUFFER_STRING(ASCII,ASCII_PACKET_LEN),13,10";
	  }
	  ELSE
	  {
		SEND_STRING dvDEBUG,"HEADER,GET_BUFFER_STRING(ASCII,ASCII_PACKET_LEN)";
	  }

	  IF (!FIRST_STRING_SENT)		// replace HEADER with spaces after first pass
	  {
		ON[FIRST_STRING_SENT]
		HEADER = LEFT_STRING(__SPACES__,LENGTH_STRING(HEADER))
	  }
	}
  }

  IF (LENGTH_ARRAY(ASCII))          // IF THERE'S ANYTHING LEFT SEND IT
  {
    IF (nMODE = 2)
    {
      SEND_STRING dvDEBUG,"HEADER,ASCII,13,10";
    }
    ELSE
    {
      SEND_STRING dvDEBUG,"HEADER,ASCII";
    }
  }
}

DEFINE_CALL 'SEND ASCII TO MASTER' (CHAR STR1[],CHAR STR2[15999])
// CALL 'SEND ASCII TO MASTER'
// v2.50 - now a wrapper for 'SEND ASCII TO DEBUG', using default settings
// PASSES ASCII CHARACTERS UNFORMATTED, ASCII STRINGS INSIDE SINGLE-QUOTES
// FORMATS CONTROL CHARACTERS AS 2 DIGIT HEX, SEPERATED BY COMMAS
// ASCII STRINGS SEPERATED FROM HEX BY COMMAS
// PARAMETERS:
// STR1 IS HEADER
// STR2 IS DATA TO BE FORMATTED
{
  CALL 'SEND ASCII TO DEBUG' (0:1:0,STR1,STR2,0,0)
}

DEFINE_CALL 'SEND CUSTOM_EVENT DATA TO DEBUG' (DEV dvDebug)
// CALL 'SEND CUSTOM_EVENT DATA TO DEBUG'
// This subroutine must be called inside of a CUSTOM_EVENT
// PARAMETERS:
// dvDEBUG IS DESTINATION
{
  STACK_VAR CHAR sVALUE2[32];
									/****************/
									/* DEVCHAN INFO */
									/****************/
  SEND_STRING dvDebug,"'D:P:S ',DEV_TO_STRING(CUSTOM.DEVICE)"
  SEND_STRING dvDebug,"'BUTTON ADDRESS ',ITOA(CUSTOM.ID)"
  SEND_STRING dvDebug,"'BUTTON STATE ',ITOA(CUSTOM.VALUE1)"

									/**********************/
								    /* BASIC MESSAGE INFO */
									/**********************/
  SEND_STRING dvDebug,"'MESSAGE TYPE ', CUSTOM_EVENT_TYPE_TO_STRING(CUSTOM.TYPE)"

								    // if this is a justification response
  IF (CUSTOM.TYPE = 1004 || CUSTOM.TYPE = 1005 || CUSTOM.TYPE = 1006)
  {
	sVALUE2 = CUSTOM_EVENT_JUSTIFY_TO_STRING(CUSTOM.VALUE2);
  }
  ELSE
  {
	sVALUE2 = ITOA(CUSTOM.VALUE2);
  }
  SEND_STRING dvDebug,"CUSTOM_EVENT_VALUE2_TO_STRING(CUSTOM.TYPE),' ',sVALUE2"

									/*************************/
								    /* OPTIONAL MESSAGE INFO */
									/*************************/
  SEND_STRING dvDebug,"'UNICODE ',ITOA(CUSTOM.FLAG)"

  SEND_STRING dvDebug,"CUSTOM_EVENT_VALUE3_TO_STRING(CUSTOM.TYPE),' ',ITOA(CUSTOM.VALUE3)"

  SEND_STRING dvDebug,"'TEXT LENGTH ',ITOA(LENGTH_STRING(CUSTOM.TEXT))"

  IF (dvDebug.NUMBER & $8000)       // IF output is being directed through a virtual device
                                    // that will add its own header, e.g. the companion debug module v1.xx...
	SEND_STRING dvDebug,"'TEXT ',CUSTOM.TEXT"
  ELSE                              // ELSE assume result is being displayed directly to a
                                    // master/ip/device 80 column terminal.
	CALL 'SEND ASCII/HEX TO DEBUG' (dvDebug,'TEXT ',CUSTOM.TEXT,0,0)
}

DEFINE_CALL 'SEND DECIMAL TO DEBUG' (DEV dvDEBUG,CHAR STR1[],CHAR STR2[15999],INTEGER nLine,INTEGER nMode)
// 'SEND DECIMAL TO DEBUG'
// FORMATS ALL CHARACTERS AS 3 DIGIT DECIMAL, SEPERATED BY COMMAS
// PARAMETERS:
// dvDEBUG IS DESTINATION
// STR1 IS HEADER
// STR2 IS DATA TO BE FORMATTED
// nLine IS LINE LENGTH. 0 = default = 80 for nMode 2, 67 for all others.
// nMode is MSG ON mode. 2 = append <cr>,<lf> to lines.
{
  STACK_VAR	integer STR1_LEN;
  STACK_VAR LONG STR2_LEN;
  STACK_VAR LONG X;
  STACK_VAR char ASCII[2560];
  STACK_VAR integer ASCII_PACKET_LEN;
  STACK_VAR INTEGER FIRST_STRING_SENT;
  STACK_VAR CHAR HEADER[128];

  STR1_LEN = LENGTH_ARRAY(STR1);
  STR2_LEN = LENGTH_ARRAY(STR2);
  HEADER = STR1;

  SELECT                            // SET THE ASCII PACKET LENGTH
  {
//    If  nLine > 131, longer messages may be truncated
//    e.g. when dvDebug is the master (send_string 0).
    ACTIVE (nLINE > MAX_LENGTH_ARRAY(ASCII)):
    {
      ASCII_PACKET_LEN = MAX_LENGTH_ARRAY(ASCII) - STR1_LEN;
    }
    ACTIVE (nLINE > STR1_LEN):
    {
      ASCII_PACKET_LEN = nLINE - STR1_LEN;
    }
    ACTIVE (nMODE = 2):             // MODE 2 DEFAULT TO 80 CHARACTER LINE
    {
      ASCII_PACKET_LEN = 80 - STR1_LEN;
    }
    ACTIVE (1):                     // DEFAULT TO 67 CHARACTER LINE
    {
      ASCII_PACKET_LEN = 67 - STR1_LEN;
    }
  }

									// BUILD UP ASCII STRING FROM STR2
  FOR (X = 1; X <= STR2_LEN;/* x is incremented via formatDecimal() */ )
  {
	X = formatDecimal(STR2,STR2_LEN,ASCII,ASCII_PACKET_LEN,X)

									// if the line is ready, send it
	IF (LENGTH_ARRAY(ASCII) >= ASCII_PACKET_LEN)
	{
	  IF (nMODE = 2)
	  {
		SEND_STRING dvDEBUG,"HEADER,GET_BUFFER_STRING(ASCII,ASCII_PACKET_LEN),13,10";
	  }
	  ELSE
	  {
		SEND_STRING dvDEBUG,"HEADER,GET_BUFFER_STRING(ASCII,ASCII_PACKET_LEN)";
	  }

	  IF (!FIRST_STRING_SENT)		// replace HEADER with spaces after first pass
	  {
		ON[FIRST_STRING_SENT]
		HEADER = LEFT_STRING(__SPACES__,LENGTH_STRING(HEADER))
	  }
	}
  }

  IF (LENGTH_ARRAY(ASCII))          // IF THERE'S ANYTHING LEFT SEND IT
  {
    IF (nMODE = 2)
    {
      SEND_STRING dvDEBUG,"HEADER,ASCII,13,10";
    }
    ELSE
    {
      SEND_STRING dvDEBUG,"HEADER,ASCII";
    }
  }
}

DEFINE_CALL 'SEND DEVICE_INFO TO DEBUG' (DEV dvDEBUG, DEV dvDEV)
/* CALL 'SEND DEVICE_INFO TO DEBUG'                        */
/* FUNCTION:   Get device info structure for a device,     */
/*             then send the info to a debug device such   */
/*             as the master's debug port.                 */
/* PARAMETERS: dvDEBUG   - device to receive device info   */
/*             dvDEV     - device from which to get info   */
{
  STACK_VAR DEV_INFO_STRUCT devinfo;
  STACK_VAR CHAR sSTR[100];

  IF (DEVICE_ID(dvDEV))             // if device is online
  {
	DEVICE_INFO(dvDEV,devinfo);

	send_string dvDebug,"'D:P:S ',DPS_TO_STRING(dvDEV)";

	if (length_array(devinfo.MANUFACTURER_STRING))
	  send_string dvDebug,"'Manufacturer ',devinfo.MANUFACTURER_STRING";
	else
	  send_string dvDebug,"'Manufacturer not available'";

	if (devinfo.MANUFACTURER)
	  send_string dvDebug,"'Manufacturer ID ',ITOA(devinfo.MANUFACTURER)";
	else
	  send_string dvDebug,"'Manufacturer ID not available'";

	if (length_array(devinfo.DEVICE_ID_STRING))
	  send_string dvDebug,"'Device ',devinfo.DEVICE_ID_STRING";
	else
	  send_string dvDebug,"'Device not available'";

	send_string dvDebug,"'Device ID $',ITOHEX(devinfo.DEVICE_ID)";

	if (length_array(devinfo.VERSION))
	  send_string dvDebug,"'Version ',devinfo.VERSION";
	else
	  send_string dvDebug,"'Version not available'";

	if (devinfo.FIRMWARE_ID)
	  send_string dvDebug,"'Firmware ID $',ITOHEX(devinfo.FIRMWARE_ID)";
	else
	  send_string dvDebug,"'Firmware ID not available'";

	if (length_array(devinfo.SERIAL_NUMBER))
	{
	                                // if serial_number is blank for Netlinx device,
									// first byte is $00
	  if (find_string(devinfo.SERIAL_NUMBER,"$00",1) = 1)
									// 16 0's
	    send_string dvDebug,"'Serial Number 0000000000000000'";

									// trim excess bytes if SN is not 16 chars long
	  else if (find_string(devinfo.SERIAL_NUMBER,"$00",1) > 1)
		send_string dvDebug,"'Serial Number ',
											LEFT_STRING(devinfo.SERIAL_NUMBER,
LENGTH_ARRAY(devinfo.SERIAL_NUMBER) - (17 - FIND_STRING(devinfo.serial_number,"$00",1))
													   )";

	  else                          // if no $00, just send the whole string
	    send_string dvDebug,"'Serial Number ',devinfo.SERIAL_NUMBER";
	}
	else                            // this should never happen
	  send_string dvDebug,"'Serial Number not available'";

	SWITCH (devinfo.SOURCE_TYPE)
	{
	  CASE SOURCE_TYPE_NO_ADDRESS:
		send_string dvDebug,FORMAT('Source Type $%02X: no address',devinfo.SOURCE_TYPE);
	  CASE SOURCE_TYPE_NEURON_ID:
		send_string dvDebug,FORMAT('Source Type $%02X: connected via ICSNet',devinfo.SOURCE_TYPE);
	  CASE SOURCE_TYPE_IP_ADDRESS:
		send_string dvDebug,FORMAT('Source Type $%02X: connected via ethernet',devinfo.SOURCE_TYPE);
	  CASE SOURCE_TYPE_AXLINK:
		send_string dvDebug,FORMAT('Source Type $%02X: connected via AXLink',devinfo.SOURCE_TYPE);
	  CASE SOURCE_TYPE_NEURON_SUBNODE_ICSP:
		send_string dvDebug,FORMAT('Source Type $%02X: connected via ICSNet',devinfo.SOURCE_TYPE);
	  CASE SOURCE_TYPE_NEURON_SUBNODE_PL:
		send_string dvDebug,FORMAT('Source Type $%02X: connected via ICSNet',devinfo.SOURCE_TYPE);
	  CASE SOURCE_TYPE_IP_SOCKET_ADDRESS:
		send_string dvDebug,FORMAT('Source Type $%02X: connected via ethernet',devinfo.SOURCE_TYPE);
	  CASE SOURCE_TYPE_RS232:
		send_string dvDebug,FORMAT('Source Type $%02X: connected via RS232',devinfo.SOURCE_TYPE);
	  CASE SOURCE_TYPE_INTERNAL:
		send_string dvDebug,FORMAT('Source Type $%02X: internal to the NetLinx controller',devinfo.SOURCE_TYPE);

		// next 3 source types added NetLinx.axi v1.25
	  CASE SOURCE_TYPE_MAC_ADDRESS:
		send_string dvDebug,FORMAT("'Source Type $%02X: connected via ethernet'",devinfo.SOURCE_TYPE)
	  CASE SOURCE_TYPE_IPv4_PORT:
		send_string dvDebug,FORMAT("'Source Type $%02X: connected via ethernet'",devinfo.SOURCE_TYPE)
	  CASE SOURCE_TYPE_IPv4_PORT_MAC_ADDRESS:
		send_string dvDebug,FORMAT("'Source Type $%02X: connected via ethernet'",devinfo.SOURCE_TYPE)

	  DEFAULT:
		send_string dvDebug,FORMAT('Source Type $%02X: unknown',devinfo.SOURCE_TYPE);
	}

	if (length_array(devinfo.SOURCE_STRING))
	  send_string dvDebug,"'Source String ',devinfo.SOURCE_STRING";
	else
	  send_string dvDebug,"'Source String not available'";
  }
  else
	send_string dvDebug,"'D:P:S ',DPS_TO_STRING(dvDEV),' not online'";
}

DEFINE_CALL 'SEND INTEGER TO DEBUG' (DEV dvDEBUG,CHAR STR1[],CHAR STR2[15999],INTEGER nLine,INTEGER nMode)
// 'SEND INTEGER TO DEBUG'
// DOES NOT FORMAT CHARACTERS, FORMATS LINE LENGTH ONLY
// PARAMETERS:
// dvDEBUG IS DESTINATION
// STR1 IS HEADER
// STR2 IS DATA TO BE FORMATTED
// nLine IS LINE LENGTH. 0 = default = 80 for nMode 2, 67 for all others.
// nMode is MSG ON mode. 2 = append <cr>,<lf> to lines.
{
  STACK_VAR	integer STR1_LEN;
  STACK_VAR LONG STR2_LEN;
  STACK_VAR LONG X;
  STACK_VAR char sTEMP[15999];
  STACK_VAR integer PACKET_LEN;
  STACK_VAR INTEGER FIRST_STRING_SENT;
  STACK_VAR CHAR HEADER[128];

  STR1_LEN = LENGTH_ARRAY(STR1);
  STR2_LEN = LENGTH_ARRAY(STR2);
  HEADER = STR1;

  SELECT                            // SET THE ASCII PACKET LENGTH
  {
//    If  nLine > 131, longer messages may be truncated
//    e.g. when dvDebug is the master (send_string 0).
    ACTIVE (nLINE > MAX_LENGTH_ARRAY(sTEMP)):
    {
      PACKET_LEN = MAX_LENGTH_ARRAY(sTEMP) - STR1_LEN;
    }
    ACTIVE (nLINE > STR1_LEN):
    {
      PACKET_LEN = nLINE - STR1_LEN;
    }
    ACTIVE (nMODE = 2):             // MODE 2 DEFAULT TO 80 CHARACTER LINE
    {
      PACKET_LEN = 80 - STR1_LEN;
    }
    ACTIVE (1):                     // DEFAULT TO 67 CHARACTER LINE
    {
      PACKET_LEN = 67 - STR1_LEN;
    }
  }

  FOR (X = 1; X <= STR2_LEN; X++)   // BUILD UP STRING FROM STR2
  {
	sTEMP = "sTEMP,STR2[X]"
									// if the line is ready, send it
	IF (LENGTH_ARRAY(sTEMP) >= PACKET_LEN)
	{
	  IF (nMODE = 2)
	  {
		SEND_STRING dvDEBUG,"HEADER,GET_BUFFER_STRING(sTEMP,PACKET_LEN),13,10";
	  }
	  ELSE
	  {
		SEND_STRING dvDEBUG,"HEADER,GET_BUFFER_STRING(sTEMP,PACKET_LEN)";
	  }

	  IF (!FIRST_STRING_SENT)		// replace HEADER with spaces after first pass
	  {
		ON[FIRST_STRING_SENT]
		HEADER = LEFT_STRING(__SPACES__,LENGTH_STRING(HEADER))
	  }
	}
  }
  IF (LENGTH_ARRAY(sTEMP))          // IF THERE'S ANYTHING LEFT SEND IT
  {
    IF (nMODE = 2)
    {
      SEND_STRING dvDEBUG,"HEADER,sTEMP,13,10";
    }
    ELSE
    {
      SEND_STRING dvDEBUG,"HEADER,sTEMP";
    }
  }
}

DEFINE_CALL 'SEND ITOHEX TO DEBUG' (DEV dvDEBUG,CHAR STR1[],CHAR STR2[15999],INTEGER nLine,INTEGER nMode)
// 'SEND ITOHEX TO DEBUG'
// FORMATS ALL CHARACTERS AS 2 DIGIT HEX, SEPERATED BY COMMAS
// PARAMETERS:
// dvDEBUG IS DESTINATION
// STR1 IS HEADER
// STR2 IS DATA TO BE FORMATTED
// nLine IS LINE LENGTH. 0 = default = 80 for nMode 2, 67 for all others.
// nMode is MSG ON mode. 2 = append <cr>,<lf> to lines.
{
  STACK_VAR	integer STR1_LEN;
  STACK_VAR LONG STR2_LEN;
  STACK_VAR LONG X;
  STACK_VAR char ASCII[2560];
  STACK_VAR integer ASCII_PACKET_LEN;
  STACK_VAR INTEGER FIRST_STRING_SENT;
  STACK_VAR CHAR HEADER[128];

  STR1_LEN = LENGTH_ARRAY(STR1);
  STR2_LEN = LENGTH_ARRAY(STR2);
  HEADER = STR1;

  SELECT                            // SET THE ASCII PACKET LENGTH
  {
//    If  nLine > 131, longer messages may be truncated
//    e.g. when dvDebug is the master (send_string 0).
    ACTIVE (nLINE > MAX_LENGTH_ARRAY(ASCII)):
    {
      ASCII_PACKET_LEN = MAX_LENGTH_ARRAY(ASCII) - STR1_LEN;
    }
    ACTIVE (nLINE > STR1_LEN):
    {
      ASCII_PACKET_LEN = nLINE - STR1_LEN;
    }
    ACTIVE (nMODE = 2):             // MODE 2 DEFAULT TO 80 CHARACTER LINE
    {
      ASCII_PACKET_LEN = 80 - STR1_LEN;
    }
    ACTIVE (1):                     // DEFAULT TO 67 CHARACTER LINE
    {
      ASCII_PACKET_LEN = 67 - STR1_LEN;
    }
  }

									// BUILD UP ASCII STRING FROM STR2
  FOR (X = 1; X <= STR2_LEN;/* x is incremented via formatHex() */ )
  {
	X = formatHex(STR2,STR2_LEN,ASCII,ASCII_PACKET_LEN,X)

									// if the line is ready, send it
	IF (LENGTH_ARRAY(ASCII) >= ASCII_PACKET_LEN)
	{
	  IF (nMODE = 2)
	  {
		SEND_STRING dvDEBUG,"HEADER,GET_BUFFER_STRING(ASCII,ASCII_PACKET_LEN),13,10";
	  }
	  ELSE
	  {
		SEND_STRING dvDEBUG,"HEADER,GET_BUFFER_STRING(ASCII,ASCII_PACKET_LEN)";
	  }

	  IF (!FIRST_STRING_SENT)		// replace HEADER with spaces after first pass
	  {
		ON[FIRST_STRING_SENT]
		HEADER = LEFT_STRING(__SPACES__,LENGTH_STRING(HEADER))
	  }
	}
  }

  IF (LENGTH_ARRAY(ASCII))          // IF THERE'S ANYTHING LEFT SEND IT
  {
    IF (nMODE = 2)
    {
      SEND_STRING dvDEBUG,"HEADER,ASCII,13,10";
    }
    ELSE
    {
      SEND_STRING dvDEBUG,"HEADER,ASCII";
    }
  }
}

DEFINE_CALL 'SEND ITOHEX TO MASTER' (CHAR STR1[],CHAR STR2[15999])
// SEND_ITOHEX_TO_MASTER
// v2.50 - now a wrapper for 'SEND ITOHEX TO DEBUG', using default settings
// FORMATS ALL CHARACTERS AS 2 DIGIT HEX, SEPERATED BY COMMAS
// PARAMETERS: STR1 IS HEADER; STR2 IS DATA TO BE FORMATTED;
{
  CALL 'SEND ITOHEX TO DEBUG' (0:1:0,STR1,STR2,0,0)
}

DEFINE_CALL 'SEND MIDI TO DEBUG' (DEV dvDEBUG,CHAR STR1[],CHAR STR2[15999],INTEGER nLINE,INTEGER nMODE, INTEGER nSTATUS)
// CALL 'SEND MIDI TO DEBUG'
// PURPOSE - Format MIDI channel voice messages with:
//              MIDI status bytes as 2 digit hex, seperated by commas.
//              MIDI data bytes as 3 digit decimal, seperated by commas.
//         - Formats MIDI system messages as 2 digit hex, seperated by commas.
// PARAMETERS:
// dvDEBUG IS DESTINATION
// STR1 IS HEADER
// STR2 IS DATA TO BE FORMATTED
// nLine IS LINE LENGTH. 0 = default = 80 for nMode 2, 67 for all others.
// nMode is MSG ON mode. 2 = append <cr>,<lf> to lines.
// nStatus is MIDI status byte:
//         - $EF and lower = default = format status bytes as hex, data bytes as decimal.
//         - $F0 and higher = format status and data as hex.
{
  STACK_VAR integer STR1_LEN;
  STACK_VAR LONG STR2_LEN;
  STACK_VAR LONG X;
  STACK_VAR char ASCII[2560];
  STACK_VAR integer ASCII_PACKET_LEN;
  STACK_VAR INTEGER FIRST_STRING_SENT;
  STACK_VAR CHAR HEADER[128];

  STR1_LEN = LENGTH_ARRAY(STR1);
  STR2_LEN = LENGTH_ARRAY(STR2);
  HEADER = STR1;

  SELECT                            // SET THE ASCII PACKET LENGTH
  {
//    If  nLine > 131, longer messages may be truncated
//    e.g. when dvDebug is the master (send_string 0).
    ACTIVE (nLINE > MAX_LENGTH_ARRAY(ASCII)):
    {
      ASCII_PACKET_LEN = MAX_LENGTH_ARRAY(ASCII) - STR1_LEN;
    }
    ACTIVE (nLINE > STR1_LEN):
    {
      ASCII_PACKET_LEN = nLINE - STR1_LEN;
    }
    ACTIVE (nMODE = 2):             // MODE 2 DEFAULT TO 80 CHARACTER LINE
    {
      ASCII_PACKET_LEN = 80 - STR1_LEN;
    }
    ACTIVE (1):                     // DEFAULT TO 67 CHARACTER LINE
    {
      ASCII_PACKET_LEN = 67 - STR1_LEN;
    }
  }

									// BUILD UP ASCII STRING FROM STR2
  FOR (X = 1; X <= STR2_LEN;/* x is incremented via formatMIDI() */ )
  {
	X = formatMIDI(STR2,STR2_LEN,ASCII,ASCII_PACKET_LEN,nSTATUS,X)

									// if the line is ready, send it
	IF (LENGTH_ARRAY(ASCII) >= ASCII_PACKET_LEN)
	{
	  IF (nMODE = 2)
	  {
		SEND_STRING dvDEBUG,"HEADER,GET_BUFFER_STRING(ASCII,ASCII_PACKET_LEN),13,10";
	  }
	  ELSE
	  {
		SEND_STRING dvDEBUG,"HEADER,GET_BUFFER_STRING(ASCII,ASCII_PACKET_LEN)";
	  }

	  IF (!FIRST_STRING_SENT)		// replace HEADER with spaces after first pass
	  {
		ON[FIRST_STRING_SENT]
		HEADER = LEFT_STRING(__SPACES__,LENGTH_STRING(HEADER))
	  }
	}
  }

  IF (LENGTH_ARRAY(ASCII))          // IF THERE'S ANYTHING LEFT SEND IT
  {
    IF (nMODE = 2)
    {
      SEND_STRING dvDEBUG,"HEADER,ASCII,13,10";
    }
    ELSE
    {
      SEND_STRING dvDEBUG,"HEADER,ASCII";
    }
  }
}

(***********************************************************)
(*                THE EVENTS GOES BELOW                    *)
(***********************************************************)
DEFINE_EVENT

DATA_EVENT[0:1:0]
{
  ONLINE:
  {
	IF (__display_lib_version__ == TRUE)
	{
	  wait 50                       // WAIT FOR DEVICE HOLDOFF
	  {
		// WHAT VERSION?
		SEND_STRING 0:0:0,"'Using lib ',__DEBUG_LIB_NAME__,
								  ', v',__DEBUG_LIB_VERSION__";
	  }
	}
  }
}

#END_IF
(***********************************************************)
(*                     END OF PROGRAM                      *)
(*        DO NOT PUT ANY CODE BELOW THIS COMMENT           *)
(***********************************************************)
